
// -*-c++-*-
//-------------------------------------------------------------------------
//   Copyright 2002-2019 National Technology & Engineering Solutions of
//   Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
//   NTESS, the U.S. Government retains certain rights in this software.
//
//   This file is part of the Xyce(TM) Parallel Electrical Simulator.
//
//   Xyce(TM) is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Xyce(TM) is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Xyce(TM).
//   If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  : Generated from verilog file BSIM6.1.1.va with ADMS
//                  interface for Xyce 6.11.0
//                  DO NOT EDIT THIS FILE DIRECTLY!  It may be overwritten!
//
// Creator        : admsXml-2.3.6
//
// Creation Date  : Tue, 02 Apr 2019 10:14:13
//
//-----------------------------------------------------------------------------
#ifndef Xyce_N_DEV_ADMSbsim6_h
#define Xyce_N_DEV_ADMSbsim6_h


#include <Sacado.hpp>

#include <N_DEV_Configuration.h>
#include <N_DEV_Const.h>
#include <N_DEV_DeviceBlock.h>
#include <N_DEV_DeviceInstance.h>
#include <N_DEV_DeviceModel.h>
#include <N_DEV_MOSFET1.h>


namespace Xyce {
namespace Device {
namespace ADMSbsim6 {

// This typedef is for our automatic differentiation:
typedef Sacado::Fad::SFad<double,21> AdmsFadType;
typedef Sacado::Fad::SFad<double,1> AdmsSensFadType;

class Model;
class Instance;
class InstanceSensitivity;

#ifdef Xyce_ADMS_SENSITIVITIES
//-----------------------------------------------------------------------------
// Class         : InstanceSensitivity
//
// Purpose       : This class is a functor for sensitivity
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class InstanceSensitivity : public baseSensitivity
{
public:
  InstanceSensitivity() :
    baseSensitivity() {};

  virtual ~InstanceSensitivity() {};

  virtual void operator()(
     const ParameterBase &entity,
     const std::string &param,
     std::vector<double> & dfdp,
     std::vector<double> & dqdp,
     std::vector<double> & dbdp,
     std::vector<int> & Findices,
     std::vector<int> & Qindices,
     std::vector<int> & Bindices
                          ) const ;
};

static InstanceSensitivity instSens;


//-----------------------------------------------------------------------------
// Class         : ModelSensitivity
//
// Purpose       : This class is a functor for sensitivity
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class ModelSensitivity : public baseSensitivity
{
public:
  ModelSensitivity() :
    baseSensitivity() {};

  virtual ~ModelSensitivity() {};

  virtual void operator()(
     const ParameterBase &entity,
     const std::string &param,
     std::vector<double> & dfdp,
     std::vector<double> & dqdp,
     std::vector<double> & dbdp,
     std::vector<int> & Findices,
     std::vector<int> & Qindices,
     std::vector<int> & Bindices
                          ) const ;
};

static ModelSensitivity modSens;
#endif // Xyce_ADMS_SENSITIVITIES

// general purpose free functions
// thermal voltage at kelvin temperature temp)
template <typename T> static inline T adms_vt(const T temp) {return(CONSTKoverQ*temp);};

// Figure out how to template this shiznit!
//-----------------------------------------------------------------------------


template<typename Tin>
static Tin adms_ternary_op(const bool cond, const Tin &ifTrue, const Tin &ifFalse)
{
  if (cond)
    return ifTrue;
  else
    return ifFalse;
}

template<typename Tin>
static Tin adms_ternary_op(const bool cond, const Tin &ifTrue, const double &ifFalse)
{
  if (cond)
    return ifTrue;
  else
    return Tin(ifFalse);
}

template<typename Tin>
static Tin adms_ternary_op(const bool cond, const double &ifTrue, const Tin &ifFalse)
{
  if (cond)
    return Tin(ifTrue);
  else
    return ifFalse;
}




#ifdef Xyce_ADMS_SENSITIVITIES
//-----------------------------------------------------------------------------
// Free functions used by sensitivity
//
//-----------------------------------------------------------------------------
void evaluateModelEquations(
   std::vector <double> & probeVars,
   // probe constants
   const int admsProbeID_V_bi_dbulk,
   const int admsProbeID_V_b_dbulk,
   const int admsProbeID_V_b_bi,
   const int admsProbeID_V_b_sbulk,
   const int admsProbeID_V_bi_sbulk,
   const int admsProbeID_V_di_si,
   const int admsProbeID_V_gm_gi,
   const int admsProbeID_V_g_gm,
   const int admsProbeID_V_s_si,
   const int admsProbeID_V_d_di,
   const int admsProbeID_V_gm_bi,
   const int admsProbeID_V_N1_GND,
   const int admsProbeID_V_N2_GND,
   const int admsProbeID_V_gm_si,
   const int admsProbeID_V_gm_di,
   const int admsProbeID_V_dbulk_di,
   const int admsProbeID_V_sbulk_si,
   const int admsProbeID_V_si_bi,
   const int admsProbeID_V_di_bi,
   const int admsProbeID_V_gi_bi,
   const int admsProbeID_Temp_t_GND,
   // node constants
   const int admsNodeID_d,
   const int admsNodeID_g,
   const int admsNodeID_s,
   const int admsNodeID_b,
   const int admsNodeID_di,
   const int admsNodeID_si,
   const int admsNodeID_gi,
   const int admsNodeID_gm,
   const int admsNodeID_bi,
   const int admsNodeID_sbulk,
   const int admsNodeID_dbulk,
   const int admsNodeID_N1,
   const int admsNodeID_N2,
   const int admsNodeID_t,
   // instance parameters
   // reals
   AdmsSensFadType & instancePar_L,
   bool instancePar_given_L,
   AdmsSensFadType & instancePar_W,
   bool instancePar_given_W,
   AdmsSensFadType & instancePar_NRS,
   bool instancePar_given_NRS,
   AdmsSensFadType & instancePar_NRD,
   bool instancePar_given_NRD,
   AdmsSensFadType & instancePar_VFBSDOFF,
   bool instancePar_given_VFBSDOFF,
   AdmsSensFadType & instancePar_XGW,
   bool instancePar_given_XGW,
   AdmsSensFadType & instancePar_RBPB,
   bool instancePar_given_RBPB,
   AdmsSensFadType & instancePar_RBPD,
   bool instancePar_given_RBPD,
   AdmsSensFadType & instancePar_RBPS,
   bool instancePar_given_RBPS,
   AdmsSensFadType & instancePar_RBDB,
   bool instancePar_given_RBDB,
   AdmsSensFadType & instancePar_RBSB,
   bool instancePar_given_RBSB,
   AdmsSensFadType & instancePar_SA,
   bool instancePar_given_SA,
   AdmsSensFadType & instancePar_SB,
   bool instancePar_given_SB,
   AdmsSensFadType & instancePar_SD,
   bool instancePar_given_SD,
   AdmsSensFadType & instancePar_AS,
   bool instancePar_given_AS,
   AdmsSensFadType & instancePar_AD,
   bool instancePar_given_AD,
   AdmsSensFadType & instancePar_PS,
   bool instancePar_given_PS,
   AdmsSensFadType & instancePar_PD,
   bool instancePar_given_PD,
   // non-reals(including hidden)
   int instancePar_NF,
   bool instancePar_given_NF,
   int instancePar_MINZ,
   bool instancePar_given_MINZ,
   int instancePar_NGCON,
   bool instancePar_given_NGCON,
   int instancePar_RGATEMOD,
   bool instancePar_given_RGATEMOD,
   int instancePar_RBODYMOD,
   bool instancePar_given_RBODYMOD,
   int instancePar_GEOMOD,
   bool instancePar_given_GEOMOD,
   int instancePar_RGEOMOD,
   bool instancePar_given_RGEOMOD,
   int instancePar_SCA,
   bool instancePar_given_SCA,
   int instancePar_SCB,
   bool instancePar_given_SCB,
   int instancePar_SCC,
   bool instancePar_given_SCC,
   int instancePar_SC,
   bool instancePar_given_SC,
   // instance variables
   // reals
   AdmsSensFadType & instanceVar_T0,
   AdmsSensFadType & instanceVar_T1,
   AdmsSensFadType & instanceVar_T2,
   AdmsSensFadType & instanceVar_epssi,
   AdmsSensFadType & instanceVar_epsox,
   AdmsSensFadType & instanceVar_Weff,
   AdmsSensFadType & instanceVar_Leff,
   AdmsSensFadType & instanceVar_Wact,
   AdmsSensFadType & instanceVar_Lact,
   AdmsSensFadType & instanceVar_Weffcj,
   AdmsSensFadType & instanceVar_Cox,
   AdmsSensFadType & instanceVar_epsratio,
   AdmsSensFadType & instanceVar_Lnew,
   AdmsSensFadType & instanceVar_L_mult,
   AdmsSensFadType & instanceVar_Wnew,
   AdmsSensFadType & instanceVar_DVTP0_i,
   AdmsSensFadType & instanceVar_DVTP1_i,
   AdmsSensFadType & instanceVar_DVTP2_i,
   AdmsSensFadType & instanceVar_DVTP3_i,
   AdmsSensFadType & instanceVar_DVTP4_i,
   AdmsSensFadType & instanceVar_DVTP5_i,
   AdmsSensFadType & instanceVar_A1_i,
   AdmsSensFadType & instanceVar_A11_i,
   AdmsSensFadType & instanceVar_A2_i,
   AdmsSensFadType & instanceVar_A21_i,
   AdmsSensFadType & instanceVar_PSAT_i,
   AdmsSensFadType & instanceVar_PSATR_i,
   AdmsSensFadType & instanceVar_BSIM6TOXP,
   AdmsSensFadType & instanceVar_DMCGeff,
   AdmsSensFadType & instanceVar_DMDGeff,
   AdmsSensFadType & instanceVar_DMCIeff,
   AdmsSensFadType & instanceVar_RSourceGeo,
   AdmsSensFadType & instanceVar_RDrainGeo,
   AdmsSensFadType & instanceVar_Grgeltd,
   AdmsSensFadType & instanceVar_Grbsb,
   AdmsSensFadType & instanceVar_Grbdb,
   AdmsSensFadType & instanceVar_Grbpb,
   AdmsSensFadType & instanceVar_Grbps,
   AdmsSensFadType & instanceVar_Grbpd,
   AdmsSensFadType & instanceVar_Aechvb,
   AdmsSensFadType & instanceVar_Bechvb,
   AdmsSensFadType & instanceVar_AechvbEdge,
   AdmsSensFadType & instanceVar_BechvbEdge,
   AdmsSensFadType & instanceVar_ToxRatio,
   AdmsSensFadType & instanceVar_U0R_t,
   AdmsSensFadType & instanceVar_UAR_t,
   AdmsSensFadType & instanceVar_UCR_t,
   AdmsSensFadType & instanceVar_UDR_t,
   AdmsSensFadType & instanceVar_UCSR_t,
   AdmsSensFadType & instanceVar_VSATR_t,
   AdmsSensFadType & instanceVar_PTWGR_t,
   AdmsSensFadType & instanceVar_ETA0R_t,
   AdmsSensFadType & instanceVar_PSATB_i,
   AdmsSensFadType & instanceVar_KT1_i,
   AdmsSensFadType & instanceVar_KT2_i,
   AdmsSensFadType & instanceVar_NGATE_i,
   AdmsSensFadType & instanceVar_NSD_i,
   AdmsSensFadType & instanceVar_NDEP_i,
   AdmsSensFadType & instanceVar_VFB_i,
   AdmsSensFadType & instanceVar_CIT_i,
   AdmsSensFadType & instanceVar_CDSCD_i,
   AdmsSensFadType & instanceVar_CDSCDR_i,
   AdmsSensFadType & instanceVar_CDSCB_i,
   AdmsSensFadType & instanceVar_NFACTOR_i,
   AdmsSensFadType & instanceVar_PHIN_i,
   AdmsSensFadType & instanceVar_ETA0_i,
   AdmsSensFadType & instanceVar_ETA0R_i,
   AdmsSensFadType & instanceVar_ETAB_i,
   AdmsSensFadType & instanceVar_K2_i,
   AdmsSensFadType & instanceVar_K1_i,
   AdmsSensFadType & instanceVar_DELTA_i,
   AdmsSensFadType & instanceVar_U0_i,
   AdmsSensFadType & instanceVar_U0R_i,
   AdmsSensFadType & instanceVar_VSAT_i,
   AdmsSensFadType & instanceVar_VSATR_i,
   AdmsSensFadType & instanceVar_VSATCV_i,
   AdmsSensFadType & instanceVar_UA_i,
   AdmsSensFadType & instanceVar_UAR_i,
   AdmsSensFadType & instanceVar_EU_i,
   AdmsSensFadType & instanceVar_UD_i,
   AdmsSensFadType & instanceVar_UDR_i,
   AdmsSensFadType & instanceVar_UD_a,
   AdmsSensFadType & instanceVar_UCS_i,
   AdmsSensFadType & instanceVar_UCSR_i,
   AdmsSensFadType & instanceVar_UC_i,
   AdmsSensFadType & instanceVar_UCR_i,
   AdmsSensFadType & instanceVar_PDIBLC_i,
   AdmsSensFadType & instanceVar_PDIBLCR_i,
   AdmsSensFadType & instanceVar_PDIBLCB_i,
   AdmsSensFadType & instanceVar_PSCBE1_i,
   AdmsSensFadType & instanceVar_PSCBE2_i,
   AdmsSensFadType & instanceVar_PDITS_i,
   AdmsSensFadType & instanceVar_PDITSD_i,
   AdmsSensFadType & instanceVar_FPROUT_i,
   AdmsSensFadType & instanceVar_PRWG_i,
   AdmsSensFadType & instanceVar_WR_i,
   AdmsSensFadType & instanceVar_RDWMIN_i,
   AdmsSensFadType & instanceVar_RSWMIN_i,
   AdmsSensFadType & instanceVar_RDW_i,
   AdmsSensFadType & instanceVar_RSW_i,
   AdmsSensFadType & instanceVar_RDSW_i,
   AdmsSensFadType & instanceVar_RDSWMIN_i,
   AdmsSensFadType & instanceVar_PTWG_i,
   AdmsSensFadType & instanceVar_PTWGR_i,
   AdmsSensFadType & instanceVar_PVAG_i,
   AdmsSensFadType & instanceVar_XJ_i,
   AdmsSensFadType & instanceVar_PCLM_i,
   AdmsSensFadType & instanceVar_PCLMR_i,
   AdmsSensFadType & instanceVar_PCLMCV_i,
   AdmsSensFadType & instanceVar_PRWB_i,
   AdmsSensFadType & instanceVar_CF_i,
   AdmsSensFadType & instanceVar_CGSL_i,
   AdmsSensFadType & instanceVar_CGDL_i,
   AdmsSensFadType & instanceVar_CKAPPAS_i,
   AdmsSensFadType & instanceVar_CKAPPAD_i,
   AdmsSensFadType & instanceVar_VFBCV_i,
   AdmsSensFadType & instanceVar_NDEPCV_i,
   AdmsSensFadType & instanceVar_ALPHA0_i,
   AdmsSensFadType & instanceVar_BETA0_i,
   AdmsSensFadType & instanceVar_AGIDL_i,
   AdmsSensFadType & instanceVar_BGIDL_i,
   AdmsSensFadType & instanceVar_CGIDL_i,
   AdmsSensFadType & instanceVar_EGIDL_i,
   AdmsSensFadType & instanceVar_AGISL_i,
   AdmsSensFadType & instanceVar_BGISL_i,
   AdmsSensFadType & instanceVar_CGISL_i,
   AdmsSensFadType & instanceVar_EGISL_i,
   AdmsSensFadType & instanceVar_UTE_i,
   AdmsSensFadType & instanceVar_UA1_i,
   AdmsSensFadType & instanceVar_UC1_i,
   AdmsSensFadType & instanceVar_UD1_i,
   AdmsSensFadType & instanceVar_UCSTE_i,
   AdmsSensFadType & instanceVar_PRT_i,
   AdmsSensFadType & instanceVar_AT_i,
   AdmsSensFadType & instanceVar_PTWGT_i,
   AdmsSensFadType & instanceVar_IIT_i,
   AdmsSensFadType & instanceVar_TGIDL_i,
   AdmsSensFadType & instanceVar_KVTH0WE_i,
   AdmsSensFadType & instanceVar_K2WE_i,
   AdmsSensFadType & instanceVar_KU0WE_i,
   AdmsSensFadType & instanceVar_AIGBINV_i,
   AdmsSensFadType & instanceVar_BIGBINV_i,
   AdmsSensFadType & instanceVar_CIGBINV_i,
   AdmsSensFadType & instanceVar_EIGBINV_i,
   AdmsSensFadType & instanceVar_NIGBINV_i,
   AdmsSensFadType & instanceVar_AIGBACC_i,
   AdmsSensFadType & instanceVar_BIGBACC_i,
   AdmsSensFadType & instanceVar_CIGBACC_i,
   AdmsSensFadType & instanceVar_NIGBACC_i,
   AdmsSensFadType & instanceVar_AIGC_i,
   AdmsSensFadType & instanceVar_BIGC_i,
   AdmsSensFadType & instanceVar_CIGC_i,
   AdmsSensFadType & instanceVar_AIGS_i,
   AdmsSensFadType & instanceVar_BIGS_i,
   AdmsSensFadType & instanceVar_CIGS_i,
   AdmsSensFadType & instanceVar_AIGD_i,
   AdmsSensFadType & instanceVar_BIGD_i,
   AdmsSensFadType & instanceVar_CIGD_i,
   AdmsSensFadType & instanceVar_PIGCD_i,
   AdmsSensFadType & instanceVar_DLCIG_i,
   AdmsSensFadType & instanceVar_DLCIGD_i,
   AdmsSensFadType & instanceVar_IGT_i,
   AdmsSensFadType & instanceVar_nuIntD,
   AdmsSensFadType & instanceVar_nuEndD,
   AdmsSensFadType & instanceVar_nuIntS,
   AdmsSensFadType & instanceVar_nuEndS,
   AdmsSensFadType & instanceVar_gth,
   AdmsSensFadType & instanceVar_cth,
   // non-reals
   int instanceVar_devsign,
   // model parameters
   // reals
   AdmsSensFadType & modelPar_L,
   bool modelPar_given_L,
   AdmsSensFadType & modelPar_VFBSDOFF,
   bool modelPar_given_VFBSDOFF,
   AdmsSensFadType & modelPar_XGW,
   bool modelPar_given_XGW,
   AdmsSensFadType & modelPar_RBPB,
   bool modelPar_given_RBPB,
   AdmsSensFadType & modelPar_RBPD,
   bool modelPar_given_RBPD,
   AdmsSensFadType & modelPar_RBPS,
   bool modelPar_given_RBPS,
   AdmsSensFadType & modelPar_RBDB,
   bool modelPar_given_RBDB,
   AdmsSensFadType & modelPar_RBSB,
   bool modelPar_given_RBSB,
   AdmsSensFadType & modelPar_SA,
   bool modelPar_given_SA,
   AdmsSensFadType & modelPar_SB,
   bool modelPar_given_SB,
   AdmsSensFadType & modelPar_SD,
   bool modelPar_given_SD,
   AdmsSensFadType & modelPar_LLONG,
   bool modelPar_given_LLONG,
   AdmsSensFadType & modelPar_LMLT,
   bool modelPar_given_LMLT,
   AdmsSensFadType & modelPar_WMLT,
   bool modelPar_given_WMLT,
   AdmsSensFadType & modelPar_XL,
   bool modelPar_given_XL,
   AdmsSensFadType & modelPar_WWIDE,
   bool modelPar_given_WWIDE,
   AdmsSensFadType & modelPar_XW,
   bool modelPar_given_XW,
   AdmsSensFadType & modelPar_LINT,
   bool modelPar_given_LINT,
   AdmsSensFadType & modelPar_LL,
   bool modelPar_given_LL,
   AdmsSensFadType & modelPar_LW,
   bool modelPar_given_LW,
   AdmsSensFadType & modelPar_LWL,
   bool modelPar_given_LWL,
   AdmsSensFadType & modelPar_LLN,
   bool modelPar_given_LLN,
   AdmsSensFadType & modelPar_LWN,
   bool modelPar_given_LWN,
   AdmsSensFadType & modelPar_WINT,
   bool modelPar_given_WINT,
   AdmsSensFadType & modelPar_WL,
   bool modelPar_given_WL,
   AdmsSensFadType & modelPar_WW,
   bool modelPar_given_WW,
   AdmsSensFadType & modelPar_WWL,
   bool modelPar_given_WWL,
   AdmsSensFadType & modelPar_WLN,
   bool modelPar_given_WLN,
   AdmsSensFadType & modelPar_WWN,
   bool modelPar_given_WWN,
   AdmsSensFadType & modelPar_DLC,
   bool modelPar_given_DLC,
   AdmsSensFadType & modelPar_LLC,
   bool modelPar_given_LLC,
   AdmsSensFadType & modelPar_LWC,
   bool modelPar_given_LWC,
   AdmsSensFadType & modelPar_LWLC,
   bool modelPar_given_LWLC,
   AdmsSensFadType & modelPar_DWC,
   bool modelPar_given_DWC,
   AdmsSensFadType & modelPar_WLC,
   bool modelPar_given_WLC,
   AdmsSensFadType & modelPar_WWC,
   bool modelPar_given_WWC,
   AdmsSensFadType & modelPar_WWLC,
   bool modelPar_given_WWLC,
   AdmsSensFadType & modelPar_TOXE,
   bool modelPar_given_TOXE,
   AdmsSensFadType & modelPar_TOXP,
   bool modelPar_given_TOXP,
   AdmsSensFadType & modelPar_DTOX,
   bool modelPar_given_DTOX,
   AdmsSensFadType & modelPar_NDEP,
   bool modelPar_given_NDEP,
   AdmsSensFadType & modelPar_NDEPL1,
   bool modelPar_given_NDEPL1,
   AdmsSensFadType & modelPar_NDEPLEXP1,
   bool modelPar_given_NDEPLEXP1,
   AdmsSensFadType & modelPar_NDEPL2,
   bool modelPar_given_NDEPL2,
   AdmsSensFadType & modelPar_NDEPLEXP2,
   bool modelPar_given_NDEPLEXP2,
   AdmsSensFadType & modelPar_NDEPW,
   bool modelPar_given_NDEPW,
   AdmsSensFadType & modelPar_NDEPWEXP,
   bool modelPar_given_NDEPWEXP,
   AdmsSensFadType & modelPar_NDEPWL,
   bool modelPar_given_NDEPWL,
   AdmsSensFadType & modelPar_NDEPWLEXP,
   bool modelPar_given_NDEPWLEXP,
   AdmsSensFadType & modelPar_LNDEP,
   bool modelPar_given_LNDEP,
   AdmsSensFadType & modelPar_WNDEP,
   bool modelPar_given_WNDEP,
   AdmsSensFadType & modelPar_PNDEP,
   bool modelPar_given_PNDEP,
   AdmsSensFadType & modelPar_NDEPCV,
   bool modelPar_given_NDEPCV,
   AdmsSensFadType & modelPar_NDEPCVL1,
   bool modelPar_given_NDEPCVL1,
   AdmsSensFadType & modelPar_NDEPCVLEXP1,
   bool modelPar_given_NDEPCVLEXP1,
   AdmsSensFadType & modelPar_NDEPCVL2,
   bool modelPar_given_NDEPCVL2,
   AdmsSensFadType & modelPar_NDEPCVLEXP2,
   bool modelPar_given_NDEPCVLEXP2,
   AdmsSensFadType & modelPar_NDEPCVW,
   bool modelPar_given_NDEPCVW,
   AdmsSensFadType & modelPar_NDEPCVWEXP,
   bool modelPar_given_NDEPCVWEXP,
   AdmsSensFadType & modelPar_NDEPCVWL,
   bool modelPar_given_NDEPCVWL,
   AdmsSensFadType & modelPar_NDEPCVWLEXP,
   bool modelPar_given_NDEPCVWLEXP,
   AdmsSensFadType & modelPar_LNDEPCV,
   bool modelPar_given_LNDEPCV,
   AdmsSensFadType & modelPar_WNDEPCV,
   bool modelPar_given_WNDEPCV,
   AdmsSensFadType & modelPar_PNDEPCV,
   bool modelPar_given_PNDEPCV,
   AdmsSensFadType & modelPar_NGATE,
   bool modelPar_given_NGATE,
   AdmsSensFadType & modelPar_LNGATE,
   bool modelPar_given_LNGATE,
   AdmsSensFadType & modelPar_WNGATE,
   bool modelPar_given_WNGATE,
   AdmsSensFadType & modelPar_PNGATE,
   bool modelPar_given_PNGATE,
   AdmsSensFadType & modelPar_EASUB,
   bool modelPar_given_EASUB,
   AdmsSensFadType & modelPar_NI0SUB,
   bool modelPar_given_NI0SUB,
   AdmsSensFadType & modelPar_BG0SUB,
   bool modelPar_given_BG0SUB,
   AdmsSensFadType & modelPar_EPSRSUB,
   bool modelPar_given_EPSRSUB,
   AdmsSensFadType & modelPar_EPSROX,
   bool modelPar_given_EPSROX,
   AdmsSensFadType & modelPar_XJ,
   bool modelPar_given_XJ,
   AdmsSensFadType & modelPar_LXJ,
   bool modelPar_given_LXJ,
   AdmsSensFadType & modelPar_WXJ,
   bool modelPar_given_WXJ,
   AdmsSensFadType & modelPar_PXJ,
   bool modelPar_given_PXJ,
   AdmsSensFadType & modelPar_LVFB,
   bool modelPar_given_LVFB,
   AdmsSensFadType & modelPar_WVFB,
   bool modelPar_given_WVFB,
   AdmsSensFadType & modelPar_PVFB,
   bool modelPar_given_PVFB,
   AdmsSensFadType & modelPar_VFB,
   bool modelPar_given_VFB,
   AdmsSensFadType & modelPar_VFBCV,
   bool modelPar_given_VFBCV,
   AdmsSensFadType & modelPar_LVFBCV,
   bool modelPar_given_LVFBCV,
   AdmsSensFadType & modelPar_WVFBCV,
   bool modelPar_given_WVFBCV,
   AdmsSensFadType & modelPar_PVFBCV,
   bool modelPar_given_PVFBCV,
   AdmsSensFadType & modelPar_VFBCVL,
   bool modelPar_given_VFBCVL,
   AdmsSensFadType & modelPar_VFBCVLEXP,
   bool modelPar_given_VFBCVLEXP,
   AdmsSensFadType & modelPar_VFBCVW,
   bool modelPar_given_VFBCVW,
   AdmsSensFadType & modelPar_VFBCVWEXP,
   bool modelPar_given_VFBCVWEXP,
   AdmsSensFadType & modelPar_VFBCVWL,
   bool modelPar_given_VFBCVWL,
   AdmsSensFadType & modelPar_VFBCVWLEXP,
   bool modelPar_given_VFBCVWLEXP,
   AdmsSensFadType & modelPar_DWJ,
   bool modelPar_given_DWJ,
   AdmsSensFadType & modelPar_NSD,
   bool modelPar_given_NSD,
   AdmsSensFadType & modelPar_LNSD,
   bool modelPar_given_LNSD,
   AdmsSensFadType & modelPar_WNSD,
   bool modelPar_given_WNSD,
   AdmsSensFadType & modelPar_PNSD,
   bool modelPar_given_PNSD,
   AdmsSensFadType & modelPar_DELVT0,
   bool modelPar_given_DELVT0,
   AdmsSensFadType & modelPar_DVTP0,
   bool modelPar_given_DVTP0,
   AdmsSensFadType & modelPar_LDVTP0,
   bool modelPar_given_LDVTP0,
   AdmsSensFadType & modelPar_WDVTP0,
   bool modelPar_given_WDVTP0,
   AdmsSensFadType & modelPar_PDVTP0,
   bool modelPar_given_PDVTP0,
   AdmsSensFadType & modelPar_DVTP1,
   bool modelPar_given_DVTP1,
   AdmsSensFadType & modelPar_LDVTP1,
   bool modelPar_given_LDVTP1,
   AdmsSensFadType & modelPar_WDVTP1,
   bool modelPar_given_WDVTP1,
   AdmsSensFadType & modelPar_PDVTP1,
   bool modelPar_given_PDVTP1,
   AdmsSensFadType & modelPar_DVTP2,
   bool modelPar_given_DVTP2,
   AdmsSensFadType & modelPar_LDVTP2,
   bool modelPar_given_LDVTP2,
   AdmsSensFadType & modelPar_WDVTP2,
   bool modelPar_given_WDVTP2,
   AdmsSensFadType & modelPar_PDVTP2,
   bool modelPar_given_PDVTP2,
   AdmsSensFadType & modelPar_DVTP3,
   bool modelPar_given_DVTP3,
   AdmsSensFadType & modelPar_LDVTP3,
   bool modelPar_given_LDVTP3,
   AdmsSensFadType & modelPar_WDVTP3,
   bool modelPar_given_WDVTP3,
   AdmsSensFadType & modelPar_PDVTP3,
   bool modelPar_given_PDVTP3,
   AdmsSensFadType & modelPar_DVTP4,
   bool modelPar_given_DVTP4,
   AdmsSensFadType & modelPar_LDVTP4,
   bool modelPar_given_LDVTP4,
   AdmsSensFadType & modelPar_WDVTP4,
   bool modelPar_given_WDVTP4,
   AdmsSensFadType & modelPar_PDVTP4,
   bool modelPar_given_PDVTP4,
   AdmsSensFadType & modelPar_DVTP5,
   bool modelPar_given_DVTP5,
   AdmsSensFadType & modelPar_LDVTP5,
   bool modelPar_given_LDVTP5,
   AdmsSensFadType & modelPar_WDVTP5,
   bool modelPar_given_WDVTP5,
   AdmsSensFadType & modelPar_PDVTP5,
   bool modelPar_given_PDVTP5,
   AdmsSensFadType & modelPar_PHIN,
   bool modelPar_given_PHIN,
   AdmsSensFadType & modelPar_LPHIN,
   bool modelPar_given_LPHIN,
   AdmsSensFadType & modelPar_WPHIN,
   bool modelPar_given_WPHIN,
   AdmsSensFadType & modelPar_PPHIN,
   bool modelPar_given_PPHIN,
   AdmsSensFadType & modelPar_ETA0,
   bool modelPar_given_ETA0,
   AdmsSensFadType & modelPar_LETA0,
   bool modelPar_given_LETA0,
   AdmsSensFadType & modelPar_WETA0,
   bool modelPar_given_WETA0,
   AdmsSensFadType & modelPar_PETA0,
   bool modelPar_given_PETA0,
   AdmsSensFadType & modelPar_ETA0R,
   bool modelPar_given_ETA0R,
   AdmsSensFadType & modelPar_LETA0R,
   bool modelPar_given_LETA0R,
   AdmsSensFadType & modelPar_WETA0R,
   bool modelPar_given_WETA0R,
   AdmsSensFadType & modelPar_PETA0R,
   bool modelPar_given_PETA0R,
   AdmsSensFadType & modelPar_DSUB,
   bool modelPar_given_DSUB,
   AdmsSensFadType & modelPar_ETAB,
   bool modelPar_given_ETAB,
   AdmsSensFadType & modelPar_ETABEXP,
   bool modelPar_given_ETABEXP,
   AdmsSensFadType & modelPar_LETAB,
   bool modelPar_given_LETAB,
   AdmsSensFadType & modelPar_WETAB,
   bool modelPar_given_WETAB,
   AdmsSensFadType & modelPar_PETAB,
   bool modelPar_given_PETAB,
   AdmsSensFadType & modelPar_K1,
   bool modelPar_given_K1,
   AdmsSensFadType & modelPar_K1L,
   bool modelPar_given_K1L,
   AdmsSensFadType & modelPar_K1LEXP,
   bool modelPar_given_K1LEXP,
   AdmsSensFadType & modelPar_K1W,
   bool modelPar_given_K1W,
   AdmsSensFadType & modelPar_K1WEXP,
   bool modelPar_given_K1WEXP,
   AdmsSensFadType & modelPar_K1WL,
   bool modelPar_given_K1WL,
   AdmsSensFadType & modelPar_K1WLEXP,
   bool modelPar_given_K1WLEXP,
   AdmsSensFadType & modelPar_LK1,
   bool modelPar_given_LK1,
   AdmsSensFadType & modelPar_WK1,
   bool modelPar_given_WK1,
   AdmsSensFadType & modelPar_PK1,
   bool modelPar_given_PK1,
   AdmsSensFadType & modelPar_K2,
   bool modelPar_given_K2,
   AdmsSensFadType & modelPar_K2L,
   bool modelPar_given_K2L,
   AdmsSensFadType & modelPar_K2LEXP,
   bool modelPar_given_K2LEXP,
   AdmsSensFadType & modelPar_K2W,
   bool modelPar_given_K2W,
   AdmsSensFadType & modelPar_K2WEXP,
   bool modelPar_given_K2WEXP,
   AdmsSensFadType & modelPar_K2WL,
   bool modelPar_given_K2WL,
   AdmsSensFadType & modelPar_K2WLEXP,
   bool modelPar_given_K2WLEXP,
   AdmsSensFadType & modelPar_LK2,
   bool modelPar_given_LK2,
   AdmsSensFadType & modelPar_WK2,
   bool modelPar_given_WK2,
   AdmsSensFadType & modelPar_PK2,
   bool modelPar_given_PK2,
   AdmsSensFadType & modelPar_ADOS,
   bool modelPar_given_ADOS,
   AdmsSensFadType & modelPar_BDOS,
   bool modelPar_given_BDOS,
   AdmsSensFadType & modelPar_QM0,
   bool modelPar_given_QM0,
   AdmsSensFadType & modelPar_ETAQM,
   bool modelPar_given_ETAQM,
   AdmsSensFadType & modelPar_CIT,
   bool modelPar_given_CIT,
   AdmsSensFadType & modelPar_LCIT,
   bool modelPar_given_LCIT,
   AdmsSensFadType & modelPar_WCIT,
   bool modelPar_given_WCIT,
   AdmsSensFadType & modelPar_PCIT,
   bool modelPar_given_PCIT,
   AdmsSensFadType & modelPar_NFACTOR,
   bool modelPar_given_NFACTOR,
   AdmsSensFadType & modelPar_NFACTORL,
   bool modelPar_given_NFACTORL,
   AdmsSensFadType & modelPar_NFACTORLEXP,
   bool modelPar_given_NFACTORLEXP,
   AdmsSensFadType & modelPar_NFACTORW,
   bool modelPar_given_NFACTORW,
   AdmsSensFadType & modelPar_NFACTORWEXP,
   bool modelPar_given_NFACTORWEXP,
   AdmsSensFadType & modelPar_NFACTORWL,
   bool modelPar_given_NFACTORWL,
   AdmsSensFadType & modelPar_NFACTORWLEXP,
   bool modelPar_given_NFACTORWLEXP,
   AdmsSensFadType & modelPar_LNFACTOR,
   bool modelPar_given_LNFACTOR,
   AdmsSensFadType & modelPar_WNFACTOR,
   bool modelPar_given_WNFACTOR,
   AdmsSensFadType & modelPar_PNFACTOR,
   bool modelPar_given_PNFACTOR,
   AdmsSensFadType & modelPar_CDSCD,
   bool modelPar_given_CDSCD,
   AdmsSensFadType & modelPar_CDSCDL,
   bool modelPar_given_CDSCDL,
   AdmsSensFadType & modelPar_CDSCDLEXP,
   bool modelPar_given_CDSCDLEXP,
   AdmsSensFadType & modelPar_LCDSCD,
   bool modelPar_given_LCDSCD,
   AdmsSensFadType & modelPar_WCDSCD,
   bool modelPar_given_WCDSCD,
   AdmsSensFadType & modelPar_PCDSCD,
   bool modelPar_given_PCDSCD,
   AdmsSensFadType & modelPar_CDSCDR,
   bool modelPar_given_CDSCDR,
   AdmsSensFadType & modelPar_CDSCDLR,
   bool modelPar_given_CDSCDLR,
   AdmsSensFadType & modelPar_LCDSCDR,
   bool modelPar_given_LCDSCDR,
   AdmsSensFadType & modelPar_WCDSCDR,
   bool modelPar_given_WCDSCDR,
   AdmsSensFadType & modelPar_PCDSCDR,
   bool modelPar_given_PCDSCDR,
   AdmsSensFadType & modelPar_CDSCB,
   bool modelPar_given_CDSCB,
   AdmsSensFadType & modelPar_CDSCBL,
   bool modelPar_given_CDSCBL,
   AdmsSensFadType & modelPar_CDSCBLEXP,
   bool modelPar_given_CDSCBLEXP,
   AdmsSensFadType & modelPar_LCDSCB,
   bool modelPar_given_LCDSCB,
   AdmsSensFadType & modelPar_WCDSCB,
   bool modelPar_given_WCDSCB,
   AdmsSensFadType & modelPar_PCDSCB,
   bool modelPar_given_PCDSCB,
   AdmsSensFadType & modelPar_VSAT,
   bool modelPar_given_VSAT,
   AdmsSensFadType & modelPar_LVSAT,
   bool modelPar_given_LVSAT,
   AdmsSensFadType & modelPar_WVSAT,
   bool modelPar_given_WVSAT,
   AdmsSensFadType & modelPar_PVSAT,
   bool modelPar_given_PVSAT,
   AdmsSensFadType & modelPar_VSATL,
   bool modelPar_given_VSATL,
   AdmsSensFadType & modelPar_VSATLEXP,
   bool modelPar_given_VSATLEXP,
   AdmsSensFadType & modelPar_VSATW,
   bool modelPar_given_VSATW,
   AdmsSensFadType & modelPar_VSATWEXP,
   bool modelPar_given_VSATWEXP,
   AdmsSensFadType & modelPar_VSATWL,
   bool modelPar_given_VSATWL,
   AdmsSensFadType & modelPar_VSATWLEXP,
   bool modelPar_given_VSATWLEXP,
   AdmsSensFadType & modelPar_VSATR,
   bool modelPar_given_VSATR,
   AdmsSensFadType & modelPar_LVSATR,
   bool modelPar_given_LVSATR,
   AdmsSensFadType & modelPar_WVSATR,
   bool modelPar_given_WVSATR,
   AdmsSensFadType & modelPar_PVSATR,
   bool modelPar_given_PVSATR,
   AdmsSensFadType & modelPar_DELTA,
   bool modelPar_given_DELTA,
   AdmsSensFadType & modelPar_LDELTA,
   bool modelPar_given_LDELTA,
   AdmsSensFadType & modelPar_WDELTA,
   bool modelPar_given_WDELTA,
   AdmsSensFadType & modelPar_PDELTA,
   bool modelPar_given_PDELTA,
   AdmsSensFadType & modelPar_DELTAL,
   bool modelPar_given_DELTAL,
   AdmsSensFadType & modelPar_DELTALEXP,
   bool modelPar_given_DELTALEXP,
   AdmsSensFadType & modelPar_VSATCV,
   bool modelPar_given_VSATCV,
   AdmsSensFadType & modelPar_LVSATCV,
   bool modelPar_given_LVSATCV,
   AdmsSensFadType & modelPar_WVSATCV,
   bool modelPar_given_WVSATCV,
   AdmsSensFadType & modelPar_PVSATCV,
   bool modelPar_given_PVSATCV,
   AdmsSensFadType & modelPar_VSATCVL,
   bool modelPar_given_VSATCVL,
   AdmsSensFadType & modelPar_VSATCVLEXP,
   bool modelPar_given_VSATCVLEXP,
   AdmsSensFadType & modelPar_VSATCVW,
   bool modelPar_given_VSATCVW,
   AdmsSensFadType & modelPar_VSATCVWEXP,
   bool modelPar_given_VSATCVWEXP,
   AdmsSensFadType & modelPar_VSATCVWL,
   bool modelPar_given_VSATCVWL,
   AdmsSensFadType & modelPar_VSATCVWLEXP,
   bool modelPar_given_VSATCVWLEXP,
   AdmsSensFadType & modelPar_UP1,
   bool modelPar_given_UP1,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_UP2,
   bool modelPar_given_UP2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_MULU0,
   bool modelPar_given_MULU0,
   AdmsSensFadType & modelPar_U0,
   bool modelPar_given_U0,
   AdmsSensFadType & modelPar_U0L,
   bool modelPar_given_U0L,
   AdmsSensFadType & modelPar_U0LEXP,
   bool modelPar_given_U0LEXP,
   AdmsSensFadType & modelPar_LU0,
   bool modelPar_given_LU0,
   AdmsSensFadType & modelPar_WU0,
   bool modelPar_given_WU0,
   AdmsSensFadType & modelPar_PU0,
   bool modelPar_given_PU0,
   AdmsSensFadType & modelPar_U0R,
   bool modelPar_given_U0R,
   AdmsSensFadType & modelPar_LU0R,
   bool modelPar_given_LU0R,
   AdmsSensFadType & modelPar_WU0R,
   bool modelPar_given_WU0R,
   AdmsSensFadType & modelPar_PU0R,
   bool modelPar_given_PU0R,
   AdmsSensFadType & modelPar_ETAMOB,
   bool modelPar_given_ETAMOB,
   AdmsSensFadType & modelPar_UA,
   bool modelPar_given_UA,
   AdmsSensFadType & modelPar_UAL,
   bool modelPar_given_UAL,
   AdmsSensFadType & modelPar_UALEXP,
   bool modelPar_given_UALEXP,
   AdmsSensFadType & modelPar_UAW,
   bool modelPar_given_UAW,
   AdmsSensFadType & modelPar_UAWEXP,
   bool modelPar_given_UAWEXP,
   AdmsSensFadType & modelPar_UAWL,
   bool modelPar_given_UAWL,
   AdmsSensFadType & modelPar_UAWLEXP,
   bool modelPar_given_UAWLEXP,
   AdmsSensFadType & modelPar_LUA,
   bool modelPar_given_LUA,
   AdmsSensFadType & modelPar_WUA,
   bool modelPar_given_WUA,
   AdmsSensFadType & modelPar_PUA,
   bool modelPar_given_PUA,
   AdmsSensFadType & modelPar_UAR,
   bool modelPar_given_UAR,
   AdmsSensFadType & modelPar_LUAR,
   bool modelPar_given_LUAR,
   AdmsSensFadType & modelPar_WUAR,
   bool modelPar_given_WUAR,
   AdmsSensFadType & modelPar_PUAR,
   bool modelPar_given_PUAR,
   AdmsSensFadType & modelPar_EU,
   bool modelPar_given_EU,
   AdmsSensFadType & modelPar_LEU,
   bool modelPar_given_LEU,
   AdmsSensFadType & modelPar_WEU,
   bool modelPar_given_WEU,
   AdmsSensFadType & modelPar_PEU,
   bool modelPar_given_PEU,
   AdmsSensFadType & modelPar_EUL,
   bool modelPar_given_EUL,
   AdmsSensFadType & modelPar_EULEXP,
   bool modelPar_given_EULEXP,
   AdmsSensFadType & modelPar_EUW,
   bool modelPar_given_EUW,
   AdmsSensFadType & modelPar_EUWEXP,
   bool modelPar_given_EUWEXP,
   AdmsSensFadType & modelPar_EUWL,
   bool modelPar_given_EUWL,
   AdmsSensFadType & modelPar_EUWLEXP,
   bool modelPar_given_EUWLEXP,
   AdmsSensFadType & modelPar_UD,
   bool modelPar_given_UD,
   AdmsSensFadType & modelPar_UDL,
   bool modelPar_given_UDL,
   AdmsSensFadType & modelPar_UDLEXP,
   bool modelPar_given_UDLEXP,
   AdmsSensFadType & modelPar_LUD,
   bool modelPar_given_LUD,
   AdmsSensFadType & modelPar_WUD,
   bool modelPar_given_WUD,
   AdmsSensFadType & modelPar_PUD,
   bool modelPar_given_PUD,
   AdmsSensFadType & modelPar_UDR,
   bool modelPar_given_UDR,
   AdmsSensFadType & modelPar_LUDR,
   bool modelPar_given_LUDR,
   AdmsSensFadType & modelPar_WUDR,
   bool modelPar_given_WUDR,
   AdmsSensFadType & modelPar_PUDR,
   bool modelPar_given_PUDR,
   AdmsSensFadType & modelPar_UCS,
   bool modelPar_given_UCS,
   AdmsSensFadType & modelPar_LUCS,
   bool modelPar_given_LUCS,
   AdmsSensFadType & modelPar_WUCS,
   bool modelPar_given_WUCS,
   AdmsSensFadType & modelPar_PUCS,
   bool modelPar_given_PUCS,
   AdmsSensFadType & modelPar_UCSR,
   bool modelPar_given_UCSR,
   AdmsSensFadType & modelPar_LUCSR,
   bool modelPar_given_LUCSR,
   AdmsSensFadType & modelPar_WUCSR,
   bool modelPar_given_WUCSR,
   AdmsSensFadType & modelPar_PUCSR,
   bool modelPar_given_PUCSR,
   AdmsSensFadType & modelPar_UC,
   bool modelPar_given_UC,
   AdmsSensFadType & modelPar_UCL,
   bool modelPar_given_UCL,
   AdmsSensFadType & modelPar_UCLEXP,
   bool modelPar_given_UCLEXP,
   AdmsSensFadType & modelPar_UCW,
   bool modelPar_given_UCW,
   AdmsSensFadType & modelPar_UCWEXP,
   bool modelPar_given_UCWEXP,
   AdmsSensFadType & modelPar_UCWL,
   bool modelPar_given_UCWL,
   AdmsSensFadType & modelPar_UCWLEXP,
   bool modelPar_given_UCWLEXP,
   AdmsSensFadType & modelPar_LUC,
   bool modelPar_given_LUC,
   AdmsSensFadType & modelPar_WUC,
   bool modelPar_given_WUC,
   AdmsSensFadType & modelPar_PUC,
   bool modelPar_given_PUC,
   AdmsSensFadType & modelPar_UCR,
   bool modelPar_given_UCR,
   AdmsSensFadType & modelPar_LUCR,
   bool modelPar_given_LUCR,
   AdmsSensFadType & modelPar_WUCR,
   bool modelPar_given_WUCR,
   AdmsSensFadType & modelPar_PUCR,
   bool modelPar_given_PUCR,
   AdmsSensFadType & modelPar_PCLM,
   bool modelPar_given_PCLM,
   AdmsSensFadType & modelPar_PCLML,
   bool modelPar_given_PCLML,
   AdmsSensFadType & modelPar_PCLMLEXP,
   bool modelPar_given_PCLMLEXP,
   AdmsSensFadType & modelPar_LPCLM,
   bool modelPar_given_LPCLM,
   AdmsSensFadType & modelPar_WPCLM,
   bool modelPar_given_WPCLM,
   AdmsSensFadType & modelPar_PPCLM,
   bool modelPar_given_PPCLM,
   AdmsSensFadType & modelPar_PCLMR,
   bool modelPar_given_PCLMR,
   AdmsSensFadType & modelPar_LPCLMR,
   bool modelPar_given_LPCLMR,
   AdmsSensFadType & modelPar_WPCLMR,
   bool modelPar_given_WPCLMR,
   AdmsSensFadType & modelPar_PPCLMR,
   bool modelPar_given_PPCLMR,
   AdmsSensFadType & modelPar_PCLMG,
   bool modelPar_given_PCLMG,
   AdmsSensFadType & modelPar_PCLMCV,
   bool modelPar_given_PCLMCV,
   AdmsSensFadType & modelPar_PCLMCVL,
   bool modelPar_given_PCLMCVL,
   AdmsSensFadType & modelPar_PCLMCVLEXP,
   bool modelPar_given_PCLMCVLEXP,
   AdmsSensFadType & modelPar_LPCLMCV,
   bool modelPar_given_LPCLMCV,
   AdmsSensFadType & modelPar_WPCLMCV,
   bool modelPar_given_WPCLMCV,
   AdmsSensFadType & modelPar_PPCLMCV,
   bool modelPar_given_PPCLMCV,
   AdmsSensFadType & modelPar_PSCBE1,
   bool modelPar_given_PSCBE1,
   AdmsSensFadType & modelPar_LPSCBE1,
   bool modelPar_given_LPSCBE1,
   AdmsSensFadType & modelPar_WPSCBE1,
   bool modelPar_given_WPSCBE1,
   AdmsSensFadType & modelPar_PPSCBE1,
   bool modelPar_given_PPSCBE1,
   AdmsSensFadType & modelPar_PSCBE2,
   bool modelPar_given_PSCBE2,
   AdmsSensFadType & modelPar_LPSCBE2,
   bool modelPar_given_LPSCBE2,
   AdmsSensFadType & modelPar_WPSCBE2,
   bool modelPar_given_WPSCBE2,
   AdmsSensFadType & modelPar_PPSCBE2,
   bool modelPar_given_PPSCBE2,
   AdmsSensFadType & modelPar_PDITS,
   bool modelPar_given_PDITS,
   AdmsSensFadType & modelPar_LPDITS,
   bool modelPar_given_LPDITS,
   AdmsSensFadType & modelPar_WPDITS,
   bool modelPar_given_WPDITS,
   AdmsSensFadType & modelPar_PPDITS,
   bool modelPar_given_PPDITS,
   AdmsSensFadType & modelPar_PDITSL,
   bool modelPar_given_PDITSL,
   AdmsSensFadType & modelPar_PDITSD,
   bool modelPar_given_PDITSD,
   AdmsSensFadType & modelPar_LPDITSD,
   bool modelPar_given_LPDITSD,
   AdmsSensFadType & modelPar_WPDITSD,
   bool modelPar_given_WPDITSD,
   AdmsSensFadType & modelPar_PPDITSD,
   bool modelPar_given_PPDITSD,
   AdmsSensFadType & modelPar_RSH,
   bool modelPar_given_RSH,
   AdmsSensFadType & modelPar_PRWG,
   bool modelPar_given_PRWG,
   AdmsSensFadType & modelPar_LPRWG,
   bool modelPar_given_LPRWG,
   AdmsSensFadType & modelPar_WPRWG,
   bool modelPar_given_WPRWG,
   AdmsSensFadType & modelPar_PPRWG,
   bool modelPar_given_PPRWG,
   AdmsSensFadType & modelPar_PRWB,
   bool modelPar_given_PRWB,
   AdmsSensFadType & modelPar_LPRWB,
   bool modelPar_given_LPRWB,
   AdmsSensFadType & modelPar_WPRWB,
   bool modelPar_given_WPRWB,
   AdmsSensFadType & modelPar_PPRWB,
   bool modelPar_given_PPRWB,
   AdmsSensFadType & modelPar_PRWBL,
   bool modelPar_given_PRWBL,
   AdmsSensFadType & modelPar_PRWBLEXP,
   bool modelPar_given_PRWBLEXP,
   AdmsSensFadType & modelPar_WR,
   bool modelPar_given_WR,
   AdmsSensFadType & modelPar_LWR,
   bool modelPar_given_LWR,
   AdmsSensFadType & modelPar_WWR,
   bool modelPar_given_WWR,
   AdmsSensFadType & modelPar_PWR,
   bool modelPar_given_PWR,
   AdmsSensFadType & modelPar_RSWMIN,
   bool modelPar_given_RSWMIN,
   AdmsSensFadType & modelPar_LRSWMIN,
   bool modelPar_given_LRSWMIN,
   AdmsSensFadType & modelPar_WRSWMIN,
   bool modelPar_given_WRSWMIN,
   AdmsSensFadType & modelPar_PRSWMIN,
   bool modelPar_given_PRSWMIN,
   AdmsSensFadType & modelPar_RSW,
   bool modelPar_given_RSW,
   AdmsSensFadType & modelPar_LRSW,
   bool modelPar_given_LRSW,
   AdmsSensFadType & modelPar_WRSW,
   bool modelPar_given_WRSW,
   AdmsSensFadType & modelPar_PRSW,
   bool modelPar_given_PRSW,
   AdmsSensFadType & modelPar_RSWL,
   bool modelPar_given_RSWL,
   AdmsSensFadType & modelPar_RSWLEXP,
   bool modelPar_given_RSWLEXP,
   AdmsSensFadType & modelPar_RDWMIN,
   bool modelPar_given_RDWMIN,
   AdmsSensFadType & modelPar_LRDWMIN,
   bool modelPar_given_LRDWMIN,
   AdmsSensFadType & modelPar_WRDWMIN,
   bool modelPar_given_WRDWMIN,
   AdmsSensFadType & modelPar_PRDWMIN,
   bool modelPar_given_PRDWMIN,
   AdmsSensFadType & modelPar_RDW,
   bool modelPar_given_RDW,
   AdmsSensFadType & modelPar_LRDW,
   bool modelPar_given_LRDW,
   AdmsSensFadType & modelPar_WRDW,
   bool modelPar_given_WRDW,
   AdmsSensFadType & modelPar_PRDW,
   bool modelPar_given_PRDW,
   AdmsSensFadType & modelPar_RDWL,
   bool modelPar_given_RDWL,
   AdmsSensFadType & modelPar_RDWLEXP,
   bool modelPar_given_RDWLEXP,
   AdmsSensFadType & modelPar_RDSWMIN,
   bool modelPar_given_RDSWMIN,
   AdmsSensFadType & modelPar_LRDSWMIN,
   bool modelPar_given_LRDSWMIN,
   AdmsSensFadType & modelPar_WRDSWMIN,
   bool modelPar_given_WRDSWMIN,
   AdmsSensFadType & modelPar_PRDSWMIN,
   bool modelPar_given_PRDSWMIN,
   AdmsSensFadType & modelPar_RDSW,
   bool modelPar_given_RDSW,
   AdmsSensFadType & modelPar_RDSWL,
   bool modelPar_given_RDSWL,
   AdmsSensFadType & modelPar_RDSWLEXP,
   bool modelPar_given_RDSWLEXP,
   AdmsSensFadType & modelPar_LRDSW,
   bool modelPar_given_LRDSW,
   AdmsSensFadType & modelPar_WRDSW,
   bool modelPar_given_WRDSW,
   AdmsSensFadType & modelPar_PRDSW,
   bool modelPar_given_PRDSW,
   AdmsSensFadType & modelPar_PSAT,
   bool modelPar_given_PSAT,
   AdmsSensFadType & modelPar_LPSAT,
   bool modelPar_given_LPSAT,
   AdmsSensFadType & modelPar_WPSAT,
   bool modelPar_given_WPSAT,
   AdmsSensFadType & modelPar_PPSAT,
   bool modelPar_given_PPSAT,
   AdmsSensFadType & modelPar_PSATL,
   bool modelPar_given_PSATL,
   AdmsSensFadType & modelPar_PSATLEXP,
   bool modelPar_given_PSATLEXP,
   AdmsSensFadType & modelPar_PSATB,
   bool modelPar_given_PSATB,
   AdmsSensFadType & modelPar_PSATR,
   bool modelPar_given_PSATR,
   AdmsSensFadType & modelPar_LPSATR,
   bool modelPar_given_LPSATR,
   AdmsSensFadType & modelPar_WPSATR,
   bool modelPar_given_WPSATR,
   AdmsSensFadType & modelPar_PPSATR,
   bool modelPar_given_PPSATR,
   AdmsSensFadType & modelPar_LPSATB,
   bool modelPar_given_LPSATB,
   AdmsSensFadType & modelPar_WPSATB,
   bool modelPar_given_WPSATB,
   AdmsSensFadType & modelPar_PPSATB,
   bool modelPar_given_PPSATB,
   AdmsSensFadType & modelPar_PSATX,
   bool modelPar_given_PSATX,
   AdmsSensFadType & modelPar_PTWG,
   bool modelPar_given_PTWG,
   AdmsSensFadType & modelPar_LPTWG,
   bool modelPar_given_LPTWG,
   AdmsSensFadType & modelPar_WPTWG,
   bool modelPar_given_WPTWG,
   AdmsSensFadType & modelPar_PPTWG,
   bool modelPar_given_PPTWG,
   AdmsSensFadType & modelPar_PTWGL,
   bool modelPar_given_PTWGL,
   AdmsSensFadType & modelPar_PTWGLEXP,
   bool modelPar_given_PTWGLEXP,
   AdmsSensFadType & modelPar_PTWGR,
   bool modelPar_given_PTWGR,
   AdmsSensFadType & modelPar_LPTWGR,
   bool modelPar_given_LPTWGR,
   AdmsSensFadType & modelPar_WPTWGR,
   bool modelPar_given_WPTWGR,
   AdmsSensFadType & modelPar_PPTWGR,
   bool modelPar_given_PPTWGR,
   AdmsSensFadType & modelPar_PTWGLR,
   bool modelPar_given_PTWGLR,
   AdmsSensFadType & modelPar_PTWGLEXPR,
   bool modelPar_given_PTWGLEXPR,
   AdmsSensFadType & modelPar_A1,
   bool modelPar_given_A1,
   AdmsSensFadType & modelPar_LA1,
   bool modelPar_given_LA1,
   AdmsSensFadType & modelPar_WA1,
   bool modelPar_given_WA1,
   AdmsSensFadType & modelPar_PA1,
   bool modelPar_given_PA1,
   AdmsSensFadType & modelPar_A11,
   bool modelPar_given_A11,
   AdmsSensFadType & modelPar_LA11,
   bool modelPar_given_LA11,
   AdmsSensFadType & modelPar_WA11,
   bool modelPar_given_WA11,
   AdmsSensFadType & modelPar_PA11,
   bool modelPar_given_PA11,
   AdmsSensFadType & modelPar_A2,
   bool modelPar_given_A2,
   AdmsSensFadType & modelPar_LA2,
   bool modelPar_given_LA2,
   AdmsSensFadType & modelPar_WA2,
   bool modelPar_given_WA2,
   AdmsSensFadType & modelPar_PA2,
   bool modelPar_given_PA2,
   AdmsSensFadType & modelPar_A21,
   bool modelPar_given_A21,
   AdmsSensFadType & modelPar_LA21,
   bool modelPar_given_LA21,
   AdmsSensFadType & modelPar_WA21,
   bool modelPar_given_WA21,
   AdmsSensFadType & modelPar_PA21,
   bool modelPar_given_PA21,
   AdmsSensFadType & modelPar_PDIBLC,
   bool modelPar_given_PDIBLC,
   AdmsSensFadType & modelPar_PDIBLCL,
   bool modelPar_given_PDIBLCL,
   AdmsSensFadType & modelPar_PDIBLCLEXP,
   bool modelPar_given_PDIBLCLEXP,
   AdmsSensFadType & modelPar_LPDIBLC,
   bool modelPar_given_LPDIBLC,
   AdmsSensFadType & modelPar_WPDIBLC,
   bool modelPar_given_WPDIBLC,
   AdmsSensFadType & modelPar_PPDIBLC,
   bool modelPar_given_PPDIBLC,
   AdmsSensFadType & modelPar_PDIBLCR,
   bool modelPar_given_PDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCLR,
   bool modelPar_given_PDIBLCLR,
   AdmsSensFadType & modelPar_PDIBLCLEXPR,
   bool modelPar_given_PDIBLCLEXPR,
   AdmsSensFadType & modelPar_LPDIBLCR,
   bool modelPar_given_LPDIBLCR,
   AdmsSensFadType & modelPar_WPDIBLCR,
   bool modelPar_given_WPDIBLCR,
   AdmsSensFadType & modelPar_PPDIBLCR,
   bool modelPar_given_PPDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCB,
   bool modelPar_given_PDIBLCB,
   AdmsSensFadType & modelPar_LPDIBLCB,
   bool modelPar_given_LPDIBLCB,
   AdmsSensFadType & modelPar_WPDIBLCB,
   bool modelPar_given_WPDIBLCB,
   AdmsSensFadType & modelPar_PPDIBLCB,
   bool modelPar_given_PPDIBLCB,
   AdmsSensFadType & modelPar_PVAG,
   bool modelPar_given_PVAG,
   AdmsSensFadType & modelPar_LPVAG,
   bool modelPar_given_LPVAG,
   AdmsSensFadType & modelPar_WPVAG,
   bool modelPar_given_WPVAG,
   AdmsSensFadType & modelPar_PPVAG,
   bool modelPar_given_PPVAG,
   AdmsSensFadType & modelPar_FPROUT,
   bool modelPar_given_FPROUT,
   AdmsSensFadType & modelPar_FPROUTL,
   bool modelPar_given_FPROUTL,
   AdmsSensFadType & modelPar_FPROUTLEXP,
   bool modelPar_given_FPROUTLEXP,
   AdmsSensFadType & modelPar_LFPROUT,
   bool modelPar_given_LFPROUT,
   AdmsSensFadType & modelPar_WFPROUT,
   bool modelPar_given_WFPROUT,
   AdmsSensFadType & modelPar_PFPROUT,
   bool modelPar_given_PFPROUT,
   AdmsSensFadType & modelPar_ALPHA0,
   bool modelPar_given_ALPHA0,
   AdmsSensFadType & modelPar_ALPHA0L,
   bool modelPar_given_ALPHA0L,
   AdmsSensFadType & modelPar_ALPHA0LEXP,
   bool modelPar_given_ALPHA0LEXP,
   AdmsSensFadType & modelPar_LALPHA0,
   bool modelPar_given_LALPHA0,
   AdmsSensFadType & modelPar_WALPHA0,
   bool modelPar_given_WALPHA0,
   AdmsSensFadType & modelPar_PALPHA0,
   bool modelPar_given_PALPHA0,
   AdmsSensFadType & modelPar_BETA0,
   bool modelPar_given_BETA0,
   AdmsSensFadType & modelPar_LBETA0,
   bool modelPar_given_LBETA0,
   AdmsSensFadType & modelPar_WBETA0,
   bool modelPar_given_WBETA0,
   AdmsSensFadType & modelPar_PBETA0,
   bool modelPar_given_PBETA0,
   AdmsSensFadType & modelPar_AIGBACC,
   bool modelPar_given_AIGBACC,
   AdmsSensFadType & modelPar_BIGBACC,
   bool modelPar_given_BIGBACC,
   AdmsSensFadType & modelPar_CIGBACC,
   bool modelPar_given_CIGBACC,
   AdmsSensFadType & modelPar_NIGBACC,
   bool modelPar_given_NIGBACC,
   AdmsSensFadType & modelPar_AIGBINV,
   bool modelPar_given_AIGBINV,
   AdmsSensFadType & modelPar_BIGBINV,
   bool modelPar_given_BIGBINV,
   AdmsSensFadType & modelPar_CIGBINV,
   bool modelPar_given_CIGBINV,
   AdmsSensFadType & modelPar_EIGBINV,
   bool modelPar_given_EIGBINV,
   AdmsSensFadType & modelPar_NIGBINV,
   bool modelPar_given_NIGBINV,
   AdmsSensFadType & modelPar_AIGC,
   bool modelPar_given_AIGC,
   AdmsSensFadType & modelPar_BIGC,
   bool modelPar_given_BIGC,
   AdmsSensFadType & modelPar_CIGC,
   bool modelPar_given_CIGC,
   AdmsSensFadType & modelPar_AIGS,
   bool modelPar_given_AIGS,
   AdmsSensFadType & modelPar_BIGS,
   bool modelPar_given_BIGS,
   AdmsSensFadType & modelPar_CIGS,
   bool modelPar_given_CIGS,
   AdmsSensFadType & modelPar_AIGD,
   bool modelPar_given_AIGD,
   AdmsSensFadType & modelPar_BIGD,
   bool modelPar_given_BIGD,
   AdmsSensFadType & modelPar_CIGD,
   bool modelPar_given_CIGD,
   AdmsSensFadType & modelPar_DLCIG,
   bool modelPar_given_DLCIG,
   AdmsSensFadType & modelPar_DLCIGD,
   bool modelPar_given_DLCIGD,
   AdmsSensFadType & modelPar_POXEDGE,
   bool modelPar_given_POXEDGE,
   AdmsSensFadType & modelPar_NTOX,
   bool modelPar_given_NTOX,
   AdmsSensFadType & modelPar_TOXREF,
   bool modelPar_given_TOXREF,
   AdmsSensFadType & modelPar_PIGCD,
   bool modelPar_given_PIGCD,
   AdmsSensFadType & modelPar_AIGCL,
   bool modelPar_given_AIGCL,
   AdmsSensFadType & modelPar_AIGCW,
   bool modelPar_given_AIGCW,
   AdmsSensFadType & modelPar_AIGSL,
   bool modelPar_given_AIGSL,
   AdmsSensFadType & modelPar_AIGSW,
   bool modelPar_given_AIGSW,
   AdmsSensFadType & modelPar_AIGDL,
   bool modelPar_given_AIGDL,
   AdmsSensFadType & modelPar_AIGDW,
   bool modelPar_given_AIGDW,
   AdmsSensFadType & modelPar_PIGCDL,
   bool modelPar_given_PIGCDL,
   AdmsSensFadType & modelPar_LAIGBINV,
   bool modelPar_given_LAIGBINV,
   AdmsSensFadType & modelPar_WAIGBINV,
   bool modelPar_given_WAIGBINV,
   AdmsSensFadType & modelPar_PAIGBINV,
   bool modelPar_given_PAIGBINV,
   AdmsSensFadType & modelPar_LBIGBINV,
   bool modelPar_given_LBIGBINV,
   AdmsSensFadType & modelPar_WBIGBINV,
   bool modelPar_given_WBIGBINV,
   AdmsSensFadType & modelPar_PBIGBINV,
   bool modelPar_given_PBIGBINV,
   AdmsSensFadType & modelPar_LCIGBINV,
   bool modelPar_given_LCIGBINV,
   AdmsSensFadType & modelPar_WCIGBINV,
   bool modelPar_given_WCIGBINV,
   AdmsSensFadType & modelPar_PCIGBINV,
   bool modelPar_given_PCIGBINV,
   AdmsSensFadType & modelPar_LEIGBINV,
   bool modelPar_given_LEIGBINV,
   AdmsSensFadType & modelPar_WEIGBINV,
   bool modelPar_given_WEIGBINV,
   AdmsSensFadType & modelPar_PEIGBINV,
   bool modelPar_given_PEIGBINV,
   AdmsSensFadType & modelPar_LNIGBINV,
   bool modelPar_given_LNIGBINV,
   AdmsSensFadType & modelPar_WNIGBINV,
   bool modelPar_given_WNIGBINV,
   AdmsSensFadType & modelPar_PNIGBINV,
   bool modelPar_given_PNIGBINV,
   AdmsSensFadType & modelPar_LAIGBACC,
   bool modelPar_given_LAIGBACC,
   AdmsSensFadType & modelPar_WAIGBACC,
   bool modelPar_given_WAIGBACC,
   AdmsSensFadType & modelPar_PAIGBACC,
   bool modelPar_given_PAIGBACC,
   AdmsSensFadType & modelPar_LBIGBACC,
   bool modelPar_given_LBIGBACC,
   AdmsSensFadType & modelPar_WBIGBACC,
   bool modelPar_given_WBIGBACC,
   AdmsSensFadType & modelPar_PBIGBACC,
   bool modelPar_given_PBIGBACC,
   AdmsSensFadType & modelPar_LCIGBACC,
   bool modelPar_given_LCIGBACC,
   AdmsSensFadType & modelPar_WCIGBACC,
   bool modelPar_given_WCIGBACC,
   AdmsSensFadType & modelPar_PCIGBACC,
   bool modelPar_given_PCIGBACC,
   AdmsSensFadType & modelPar_LNIGBACC,
   bool modelPar_given_LNIGBACC,
   AdmsSensFadType & modelPar_WNIGBACC,
   bool modelPar_given_WNIGBACC,
   AdmsSensFadType & modelPar_PNIGBACC,
   bool modelPar_given_PNIGBACC,
   AdmsSensFadType & modelPar_LAIGC,
   bool modelPar_given_LAIGC,
   AdmsSensFadType & modelPar_WAIGC,
   bool modelPar_given_WAIGC,
   AdmsSensFadType & modelPar_PAIGC,
   bool modelPar_given_PAIGC,
   AdmsSensFadType & modelPar_LBIGC,
   bool modelPar_given_LBIGC,
   AdmsSensFadType & modelPar_WBIGC,
   bool modelPar_given_WBIGC,
   AdmsSensFadType & modelPar_PBIGC,
   bool modelPar_given_PBIGC,
   AdmsSensFadType & modelPar_LCIGC,
   bool modelPar_given_LCIGC,
   AdmsSensFadType & modelPar_WCIGC,
   bool modelPar_given_WCIGC,
   AdmsSensFadType & modelPar_PCIGC,
   bool modelPar_given_PCIGC,
   AdmsSensFadType & modelPar_LAIGS,
   bool modelPar_given_LAIGS,
   AdmsSensFadType & modelPar_WAIGS,
   bool modelPar_given_WAIGS,
   AdmsSensFadType & modelPar_PAIGS,
   bool modelPar_given_PAIGS,
   AdmsSensFadType & modelPar_LBIGS,
   bool modelPar_given_LBIGS,
   AdmsSensFadType & modelPar_WBIGS,
   bool modelPar_given_WBIGS,
   AdmsSensFadType & modelPar_PBIGS,
   bool modelPar_given_PBIGS,
   AdmsSensFadType & modelPar_LCIGS,
   bool modelPar_given_LCIGS,
   AdmsSensFadType & modelPar_WCIGS,
   bool modelPar_given_WCIGS,
   AdmsSensFadType & modelPar_PCIGS,
   bool modelPar_given_PCIGS,
   AdmsSensFadType & modelPar_LAIGD,
   bool modelPar_given_LAIGD,
   AdmsSensFadType & modelPar_WAIGD,
   bool modelPar_given_WAIGD,
   AdmsSensFadType & modelPar_PAIGD,
   bool modelPar_given_PAIGD,
   AdmsSensFadType & modelPar_LBIGD,
   bool modelPar_given_LBIGD,
   AdmsSensFadType & modelPar_WBIGD,
   bool modelPar_given_WBIGD,
   AdmsSensFadType & modelPar_PBIGD,
   bool modelPar_given_PBIGD,
   AdmsSensFadType & modelPar_LCIGD,
   bool modelPar_given_LCIGD,
   AdmsSensFadType & modelPar_WCIGD,
   bool modelPar_given_WCIGD,
   AdmsSensFadType & modelPar_PCIGD,
   bool modelPar_given_PCIGD,
   AdmsSensFadType & modelPar_LPOXEDGE,
   bool modelPar_given_LPOXEDGE,
   AdmsSensFadType & modelPar_WPOXEDGE,
   bool modelPar_given_WPOXEDGE,
   AdmsSensFadType & modelPar_PPOXEDGE,
   bool modelPar_given_PPOXEDGE,
   AdmsSensFadType & modelPar_LDLCIG,
   bool modelPar_given_LDLCIG,
   AdmsSensFadType & modelPar_WDLCIG,
   bool modelPar_given_WDLCIG,
   AdmsSensFadType & modelPar_PDLCIG,
   bool modelPar_given_PDLCIG,
   AdmsSensFadType & modelPar_LDLCIGD,
   bool modelPar_given_LDLCIGD,
   AdmsSensFadType & modelPar_WDLCIGD,
   bool modelPar_given_WDLCIGD,
   AdmsSensFadType & modelPar_PDLCIGD,
   bool modelPar_given_PDLCIGD,
   AdmsSensFadType & modelPar_LNTOX,
   bool modelPar_given_LNTOX,
   AdmsSensFadType & modelPar_WNTOX,
   bool modelPar_given_WNTOX,
   AdmsSensFadType & modelPar_PNTOX,
   bool modelPar_given_PNTOX,
   AdmsSensFadType & modelPar_AGIDL,
   bool modelPar_given_AGIDL,
   AdmsSensFadType & modelPar_AGIDLL,
   bool modelPar_given_AGIDLL,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_LAGIDL,
   bool modelPar_given_LAGIDL,
   AdmsSensFadType & modelPar_WAGIDL,
   bool modelPar_given_WAGIDL,
   AdmsSensFadType & modelPar_PAGIDL,
   bool modelPar_given_PAGIDL,
   AdmsSensFadType & modelPar_BGIDL,
   bool modelPar_given_BGIDL,
   AdmsSensFadType & modelPar_LBGIDL,
   bool modelPar_given_LBGIDL,
   AdmsSensFadType & modelPar_WBGIDL,
   bool modelPar_given_WBGIDL,
   AdmsSensFadType & modelPar_PBGIDL,
   bool modelPar_given_PBGIDL,
   AdmsSensFadType & modelPar_CGIDL,
   bool modelPar_given_CGIDL,
   AdmsSensFadType & modelPar_LCGIDL,
   bool modelPar_given_LCGIDL,
   AdmsSensFadType & modelPar_WCGIDL,
   bool modelPar_given_WCGIDL,
   AdmsSensFadType & modelPar_PCGIDL,
   bool modelPar_given_PCGIDL,
   AdmsSensFadType & modelPar_EGIDL,
   bool modelPar_given_EGIDL,
   AdmsSensFadType & modelPar_LEGIDL,
   bool modelPar_given_LEGIDL,
   AdmsSensFadType & modelPar_WEGIDL,
   bool modelPar_given_WEGIDL,
   AdmsSensFadType & modelPar_PEGIDL,
   bool modelPar_given_PEGIDL,
   AdmsSensFadType & modelPar_AGISL,
   bool modelPar_given_AGISL,
   AdmsSensFadType & modelPar_AGISLL,
   bool modelPar_given_AGISLL,
   AdmsSensFadType & modelPar_AGISLW,
   bool modelPar_given_AGISLW,
   AdmsSensFadType & modelPar_LAGISL,
   bool modelPar_given_LAGISL,
   AdmsSensFadType & modelPar_WAGISL,
   bool modelPar_given_WAGISL,
   AdmsSensFadType & modelPar_PAGISL,
   bool modelPar_given_PAGISL,
   AdmsSensFadType & modelPar_BGISL,
   bool modelPar_given_BGISL,
   AdmsSensFadType & modelPar_LBGISL,
   bool modelPar_given_LBGISL,
   AdmsSensFadType & modelPar_WBGISL,
   bool modelPar_given_WBGISL,
   AdmsSensFadType & modelPar_PBGISL,
   bool modelPar_given_PBGISL,
   AdmsSensFadType & modelPar_CGISL,
   bool modelPar_given_CGISL,
   AdmsSensFadType & modelPar_LCGISL,
   bool modelPar_given_LCGISL,
   AdmsSensFadType & modelPar_WCGISL,
   bool modelPar_given_WCGISL,
   AdmsSensFadType & modelPar_PCGISL,
   bool modelPar_given_PCGISL,
   AdmsSensFadType & modelPar_EGISL,
   bool modelPar_given_EGISL,
   AdmsSensFadType & modelPar_LEGISL,
   bool modelPar_given_LEGISL,
   AdmsSensFadType & modelPar_WEGISL,
   bool modelPar_given_WEGISL,
   AdmsSensFadType & modelPar_PEGISL,
   bool modelPar_given_PEGISL,
   AdmsSensFadType & modelPar_CF,
   bool modelPar_given_CF,
   AdmsSensFadType & modelPar_LCF,
   bool modelPar_given_LCF,
   AdmsSensFadType & modelPar_WCF,
   bool modelPar_given_WCF,
   AdmsSensFadType & modelPar_PCF,
   bool modelPar_given_PCF,
   AdmsSensFadType & modelPar_CFRCOEFF,
   bool modelPar_given_CFRCOEFF,
   AdmsSensFadType & modelPar_CGSO,
   bool modelPar_given_CGSO,
   AdmsSensFadType & modelPar_CGDO,
   bool modelPar_given_CGDO,
   AdmsSensFadType & modelPar_CGBO,
   bool modelPar_given_CGBO,
   AdmsSensFadType & modelPar_CGSL,
   bool modelPar_given_CGSL,
   AdmsSensFadType & modelPar_LCGSL,
   bool modelPar_given_LCGSL,
   AdmsSensFadType & modelPar_WCGSL,
   bool modelPar_given_WCGSL,
   AdmsSensFadType & modelPar_PCGSL,
   bool modelPar_given_PCGSL,
   AdmsSensFadType & modelPar_CGDL,
   bool modelPar_given_CGDL,
   AdmsSensFadType & modelPar_LCGDL,
   bool modelPar_given_LCGDL,
   AdmsSensFadType & modelPar_WCGDL,
   bool modelPar_given_WCGDL,
   AdmsSensFadType & modelPar_PCGDL,
   bool modelPar_given_PCGDL,
   AdmsSensFadType & modelPar_CKAPPAS,
   bool modelPar_given_CKAPPAS,
   AdmsSensFadType & modelPar_LCKAPPAS,
   bool modelPar_given_LCKAPPAS,
   AdmsSensFadType & modelPar_WCKAPPAS,
   bool modelPar_given_WCKAPPAS,
   AdmsSensFadType & modelPar_PCKAPPAS,
   bool modelPar_given_PCKAPPAS,
   AdmsSensFadType & modelPar_CKAPPAD,
   bool modelPar_given_CKAPPAD,
   AdmsSensFadType & modelPar_LCKAPPAD,
   bool modelPar_given_LCKAPPAD,
   AdmsSensFadType & modelPar_WCKAPPAD,
   bool modelPar_given_WCKAPPAD,
   AdmsSensFadType & modelPar_PCKAPPAD,
   bool modelPar_given_PCKAPPAD,
   AdmsSensFadType & modelPar_DMCG,
   bool modelPar_given_DMCG,
   AdmsSensFadType & modelPar_DMCI,
   bool modelPar_given_DMCI,
   AdmsSensFadType & modelPar_DMDG,
   bool modelPar_given_DMDG,
   AdmsSensFadType & modelPar_DMCGT,
   bool modelPar_given_DMCGT,
   AdmsSensFadType & modelPar_XGL,
   bool modelPar_given_XGL,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_CJS,
   bool modelPar_given_CJS,
   AdmsSensFadType & modelPar_CJD,
   bool modelPar_given_CJD,
   AdmsSensFadType & modelPar_CJSWS,
   bool modelPar_given_CJSWS,
   AdmsSensFadType & modelPar_CJSWD,
   bool modelPar_given_CJSWD,
   AdmsSensFadType & modelPar_CJSWGS,
   bool modelPar_given_CJSWGS,
   AdmsSensFadType & modelPar_CJSWGD,
   bool modelPar_given_CJSWGD,
   AdmsSensFadType & modelPar_PBS,
   bool modelPar_given_PBS,
   AdmsSensFadType & modelPar_PBD,
   bool modelPar_given_PBD,
   AdmsSensFadType & modelPar_PBSWS,
   bool modelPar_given_PBSWS,
   AdmsSensFadType & modelPar_PBSWD,
   bool modelPar_given_PBSWD,
   AdmsSensFadType & modelPar_PBSWGS,
   bool modelPar_given_PBSWGS,
   AdmsSensFadType & modelPar_PBSWGD,
   bool modelPar_given_PBSWGD,
   AdmsSensFadType & modelPar_MJS,
   bool modelPar_given_MJS,
   AdmsSensFadType & modelPar_MJD,
   bool modelPar_given_MJD,
   AdmsSensFadType & modelPar_MJSWS,
   bool modelPar_given_MJSWS,
   AdmsSensFadType & modelPar_MJSWD,
   bool modelPar_given_MJSWD,
   AdmsSensFadType & modelPar_MJSWGS,
   bool modelPar_given_MJSWGS,
   AdmsSensFadType & modelPar_MJSWGD,
   bool modelPar_given_MJSWGD,
   AdmsSensFadType & modelPar_JSS,
   bool modelPar_given_JSS,
   AdmsSensFadType & modelPar_JSD,
   bool modelPar_given_JSD,
   AdmsSensFadType & modelPar_JSWS,
   bool modelPar_given_JSWS,
   AdmsSensFadType & modelPar_JSWD,
   bool modelPar_given_JSWD,
   AdmsSensFadType & modelPar_JSWGS,
   bool modelPar_given_JSWGS,
   AdmsSensFadType & modelPar_JSWGD,
   bool modelPar_given_JSWGD,
   AdmsSensFadType & modelPar_NJS,
   bool modelPar_given_NJS,
   AdmsSensFadType & modelPar_NJD,
   bool modelPar_given_NJD,
   AdmsSensFadType & modelPar_IJTHSFWD,
   bool modelPar_given_IJTHSFWD,
   AdmsSensFadType & modelPar_IJTHDFWD,
   bool modelPar_given_IJTHDFWD,
   AdmsSensFadType & modelPar_IJTHSREV,
   bool modelPar_given_IJTHSREV,
   AdmsSensFadType & modelPar_IJTHDREV,
   bool modelPar_given_IJTHDREV,
   AdmsSensFadType & modelPar_BVS,
   bool modelPar_given_BVS,
   AdmsSensFadType & modelPar_BVD,
   bool modelPar_given_BVD,
   AdmsSensFadType & modelPar_XJBVS,
   bool modelPar_given_XJBVS,
   AdmsSensFadType & modelPar_XJBVD,
   bool modelPar_given_XJBVD,
   AdmsSensFadType & modelPar_JTSS,
   bool modelPar_given_JTSS,
   AdmsSensFadType & modelPar_JTSD,
   bool modelPar_given_JTSD,
   AdmsSensFadType & modelPar_JTSSWS,
   bool modelPar_given_JTSSWS,
   AdmsSensFadType & modelPar_JTSSWD,
   bool modelPar_given_JTSSWD,
   AdmsSensFadType & modelPar_JTSSWGS,
   bool modelPar_given_JTSSWGS,
   AdmsSensFadType & modelPar_JTSSWGD,
   bool modelPar_given_JTSSWGD,
   AdmsSensFadType & modelPar_JTWEFF,
   bool modelPar_given_JTWEFF,
   AdmsSensFadType & modelPar_NJTS,
   bool modelPar_given_NJTS,
   AdmsSensFadType & modelPar_NJTSD,
   bool modelPar_given_NJTSD,
   AdmsSensFadType & modelPar_NJTSSW,
   bool modelPar_given_NJTSSW,
   AdmsSensFadType & modelPar_NJTSSWD,
   bool modelPar_given_NJTSSWD,
   AdmsSensFadType & modelPar_NJTSSWG,
   bool modelPar_given_NJTSSWG,
   AdmsSensFadType & modelPar_NJTSSWGD,
   bool modelPar_given_NJTSSWGD,
   AdmsSensFadType & modelPar_VTSS,
   bool modelPar_given_VTSS,
   AdmsSensFadType & modelPar_VTSD,
   bool modelPar_given_VTSD,
   AdmsSensFadType & modelPar_VTSSWS,
   bool modelPar_given_VTSSWS,
   AdmsSensFadType & modelPar_VTSSWD,
   bool modelPar_given_VTSSWD,
   AdmsSensFadType & modelPar_VTSSWGS,
   bool modelPar_given_VTSSWGS,
   AdmsSensFadType & modelPar_VTSSWGD,
   bool modelPar_given_VTSSWGD,
   AdmsSensFadType & modelPar_XRCRG1,
   bool modelPar_given_XRCRG1,
   AdmsSensFadType & modelPar_XRCRG2,
   bool modelPar_given_XRCRG2,
   AdmsSensFadType & modelPar_GBMIN,
   bool modelPar_given_GBMIN,
   AdmsSensFadType & modelPar_RBPS0,
   bool modelPar_given_RBPS0,
   AdmsSensFadType & modelPar_RBPSL,
   bool modelPar_given_RBPSL,
   AdmsSensFadType & modelPar_RBPSW,
   bool modelPar_given_RBPSW,
   AdmsSensFadType & modelPar_RBPSNF,
   bool modelPar_given_RBPSNF,
   AdmsSensFadType & modelPar_RBPD0,
   bool modelPar_given_RBPD0,
   AdmsSensFadType & modelPar_RBPDL,
   bool modelPar_given_RBPDL,
   AdmsSensFadType & modelPar_RBPDW,
   bool modelPar_given_RBPDW,
   AdmsSensFadType & modelPar_RBPDNF,
   bool modelPar_given_RBPDNF,
   AdmsSensFadType & modelPar_RBPBX0,
   bool modelPar_given_RBPBX0,
   AdmsSensFadType & modelPar_RBPBXL,
   bool modelPar_given_RBPBXL,
   AdmsSensFadType & modelPar_RBPBXW,
   bool modelPar_given_RBPBXW,
   AdmsSensFadType & modelPar_RBPBXNF,
   bool modelPar_given_RBPBXNF,
   AdmsSensFadType & modelPar_RBPBY0,
   bool modelPar_given_RBPBY0,
   AdmsSensFadType & modelPar_RBPBYL,
   bool modelPar_given_RBPBYL,
   AdmsSensFadType & modelPar_RBPBYW,
   bool modelPar_given_RBPBYW,
   AdmsSensFadType & modelPar_RBPBYNF,
   bool modelPar_given_RBPBYNF,
   AdmsSensFadType & modelPar_RBSBX0,
   bool modelPar_given_RBSBX0,
   AdmsSensFadType & modelPar_RBSBY0,
   bool modelPar_given_RBSBY0,
   AdmsSensFadType & modelPar_RBDBX0,
   bool modelPar_given_RBDBX0,
   AdmsSensFadType & modelPar_RBDBY0,
   bool modelPar_given_RBDBY0,
   AdmsSensFadType & modelPar_RBSDBXL,
   bool modelPar_given_RBSDBXL,
   AdmsSensFadType & modelPar_RBSDBXW,
   bool modelPar_given_RBSDBXW,
   AdmsSensFadType & modelPar_RBSDBXNF,
   bool modelPar_given_RBSDBXNF,
   AdmsSensFadType & modelPar_RBSDBYL,
   bool modelPar_given_RBSDBYL,
   AdmsSensFadType & modelPar_RBSDBYW,
   bool modelPar_given_RBSDBYW,
   AdmsSensFadType & modelPar_RBSDBYNF,
   bool modelPar_given_RBSDBYNF,
   AdmsSensFadType & modelPar_EF,
   bool modelPar_given_EF,
   AdmsSensFadType & modelPar_EM,
   bool modelPar_given_EM,
   AdmsSensFadType & modelPar_NOIA,
   bool modelPar_given_NOIA,
   AdmsSensFadType & modelPar_NOIB,
   bool modelPar_given_NOIB,
   AdmsSensFadType & modelPar_NOIC,
   bool modelPar_given_NOIC,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_NTNOI,
   bool modelPar_given_NTNOI,
   AdmsSensFadType & modelPar_RNOIA,
   bool modelPar_given_RNOIA,
   AdmsSensFadType & modelPar_RNOIB,
   bool modelPar_given_RNOIB,
   AdmsSensFadType & modelPar_RNOIC,
   bool modelPar_given_RNOIC,
   AdmsSensFadType & modelPar_TNOIA,
   bool modelPar_given_TNOIA,
   AdmsSensFadType & modelPar_TNOIB,
   bool modelPar_given_TNOIB,
   AdmsSensFadType & modelPar_TNOIC,
   bool modelPar_given_TNOIC,
   AdmsSensFadType & modelPar_DLBIN,
   bool modelPar_given_DLBIN,
   AdmsSensFadType & modelPar_DWBIN,
   bool modelPar_given_DWBIN,
   AdmsSensFadType & modelPar_TNOM,
   bool modelPar_given_TNOM,
   AdmsSensFadType & modelPar_DTEMP,
   bool modelPar_given_DTEMP,
   AdmsSensFadType & modelPar_TBGASUB,
   bool modelPar_given_TBGASUB,
   AdmsSensFadType & modelPar_TBGBSUB,
   bool modelPar_given_TBGBSUB,
   AdmsSensFadType & modelPar_TNFACTOR,
   bool modelPar_given_TNFACTOR,
   AdmsSensFadType & modelPar_UTE,
   bool modelPar_given_UTE,
   AdmsSensFadType & modelPar_LUTE,
   bool modelPar_given_LUTE,
   AdmsSensFadType & modelPar_WUTE,
   bool modelPar_given_WUTE,
   AdmsSensFadType & modelPar_PUTE,
   bool modelPar_given_PUTE,
   AdmsSensFadType & modelPar_UTEL,
   bool modelPar_given_UTEL,
   AdmsSensFadType & modelPar_UA1,
   bool modelPar_given_UA1,
   AdmsSensFadType & modelPar_LUA1,
   bool modelPar_given_LUA1,
   AdmsSensFadType & modelPar_WUA1,
   bool modelPar_given_WUA1,
   AdmsSensFadType & modelPar_PUA1,
   bool modelPar_given_PUA1,
   AdmsSensFadType & modelPar_UA1L,
   bool modelPar_given_UA1L,
   AdmsSensFadType & modelPar_UC1,
   bool modelPar_given_UC1,
   AdmsSensFadType & modelPar_LUC1,
   bool modelPar_given_LUC1,
   AdmsSensFadType & modelPar_WUC1,
   bool modelPar_given_WUC1,
   AdmsSensFadType & modelPar_PUC1,
   bool modelPar_given_PUC1,
   AdmsSensFadType & modelPar_UD1,
   bool modelPar_given_UD1,
   AdmsSensFadType & modelPar_LUD1,
   bool modelPar_given_LUD1,
   AdmsSensFadType & modelPar_WUD1,
   bool modelPar_given_WUD1,
   AdmsSensFadType & modelPar_PUD1,
   bool modelPar_given_PUD1,
   AdmsSensFadType & modelPar_UD1L,
   bool modelPar_given_UD1L,
   AdmsSensFadType & modelPar_UCSTE,
   bool modelPar_given_UCSTE,
   AdmsSensFadType & modelPar_LUCSTE,
   bool modelPar_given_LUCSTE,
   AdmsSensFadType & modelPar_WUCSTE,
   bool modelPar_given_WUCSTE,
   AdmsSensFadType & modelPar_PUCSTE,
   bool modelPar_given_PUCSTE,
   AdmsSensFadType & modelPar_TETA0,
   bool modelPar_given_TETA0,
   AdmsSensFadType & modelPar_PRT,
   bool modelPar_given_PRT,
   AdmsSensFadType & modelPar_LPRT,
   bool modelPar_given_LPRT,
   AdmsSensFadType & modelPar_WPRT,
   bool modelPar_given_WPRT,
   AdmsSensFadType & modelPar_PPRT,
   bool modelPar_given_PPRT,
   AdmsSensFadType & modelPar_AT,
   bool modelPar_given_AT,
   AdmsSensFadType & modelPar_LAT,
   bool modelPar_given_LAT,
   AdmsSensFadType & modelPar_WAT,
   bool modelPar_given_WAT,
   AdmsSensFadType & modelPar_PAT,
   bool modelPar_given_PAT,
   AdmsSensFadType & modelPar_ATL,
   bool modelPar_given_ATL,
   AdmsSensFadType & modelPar_TDELTA,
   bool modelPar_given_TDELTA,
   AdmsSensFadType & modelPar_PTWGT,
   bool modelPar_given_PTWGT,
   AdmsSensFadType & modelPar_LPTWGT,
   bool modelPar_given_LPTWGT,
   AdmsSensFadType & modelPar_WPTWGT,
   bool modelPar_given_WPTWGT,
   AdmsSensFadType & modelPar_PPTWGT,
   bool modelPar_given_PPTWGT,
   AdmsSensFadType & modelPar_PTWGTL,
   bool modelPar_given_PTWGTL,
   AdmsSensFadType & modelPar_KT1,
   bool modelPar_given_KT1,
   AdmsSensFadType & modelPar_KT1EXP,
   bool modelPar_given_KT1EXP,
   AdmsSensFadType & modelPar_KT1L,
   bool modelPar_given_KT1L,
   AdmsSensFadType & modelPar_LKT1,
   bool modelPar_given_LKT1,
   AdmsSensFadType & modelPar_WKT1,
   bool modelPar_given_WKT1,
   AdmsSensFadType & modelPar_PKT1,
   bool modelPar_given_PKT1,
   AdmsSensFadType & modelPar_KT2,
   bool modelPar_given_KT2,
   AdmsSensFadType & modelPar_LKT2,
   bool modelPar_given_LKT2,
   AdmsSensFadType & modelPar_WKT2,
   bool modelPar_given_WKT2,
   AdmsSensFadType & modelPar_PKT2,
   bool modelPar_given_PKT2,
   AdmsSensFadType & modelPar_IIT,
   bool modelPar_given_IIT,
   AdmsSensFadType & modelPar_LIIT,
   bool modelPar_given_LIIT,
   AdmsSensFadType & modelPar_WIIT,
   bool modelPar_given_WIIT,
   AdmsSensFadType & modelPar_PIIT,
   bool modelPar_given_PIIT,
   AdmsSensFadType & modelPar_IGT,
   bool modelPar_given_IGT,
   AdmsSensFadType & modelPar_LIGT,
   bool modelPar_given_LIGT,
   AdmsSensFadType & modelPar_WIGT,
   bool modelPar_given_WIGT,
   AdmsSensFadType & modelPar_PIGT,
   bool modelPar_given_PIGT,
   AdmsSensFadType & modelPar_TGIDL,
   bool modelPar_given_TGIDL,
   AdmsSensFadType & modelPar_LTGIDL,
   bool modelPar_given_LTGIDL,
   AdmsSensFadType & modelPar_WTGIDL,
   bool modelPar_given_WTGIDL,
   AdmsSensFadType & modelPar_PTGIDL,
   bool modelPar_given_PTGIDL,
   AdmsSensFadType & modelPar_TCJ,
   bool modelPar_given_TCJ,
   AdmsSensFadType & modelPar_TCJSW,
   bool modelPar_given_TCJSW,
   AdmsSensFadType & modelPar_TCJSWG,
   bool modelPar_given_TCJSWG,
   AdmsSensFadType & modelPar_TPB,
   bool modelPar_given_TPB,
   AdmsSensFadType & modelPar_TPBSW,
   bool modelPar_given_TPBSW,
   AdmsSensFadType & modelPar_TPBSWG,
   bool modelPar_given_TPBSWG,
   AdmsSensFadType & modelPar_XTIS,
   bool modelPar_given_XTIS,
   AdmsSensFadType & modelPar_XTID,
   bool modelPar_given_XTID,
   AdmsSensFadType & modelPar_XTSS,
   bool modelPar_given_XTSS,
   AdmsSensFadType & modelPar_XTSD,
   bool modelPar_given_XTSD,
   AdmsSensFadType & modelPar_XTSSWS,
   bool modelPar_given_XTSSWS,
   AdmsSensFadType & modelPar_XTSSWD,
   bool modelPar_given_XTSSWD,
   AdmsSensFadType & modelPar_XTSSWGS,
   bool modelPar_given_XTSSWGS,
   AdmsSensFadType & modelPar_XTSSWGD,
   bool modelPar_given_XTSSWGD,
   AdmsSensFadType & modelPar_TNJTS,
   bool modelPar_given_TNJTS,
   AdmsSensFadType & modelPar_TNJTSD,
   bool modelPar_given_TNJTSD,
   AdmsSensFadType & modelPar_TNJTSSW,
   bool modelPar_given_TNJTSSW,
   AdmsSensFadType & modelPar_TNJTSSWD,
   bool modelPar_given_TNJTSSWD,
   AdmsSensFadType & modelPar_TNJTSSWG,
   bool modelPar_given_TNJTSSWG,
   AdmsSensFadType & modelPar_TNJTSSWGD,
   bool modelPar_given_TNJTSSWGD,
   AdmsSensFadType & modelPar_RTH0,
   bool modelPar_given_RTH0,
   AdmsSensFadType & modelPar_CTH0,
   bool modelPar_given_CTH0,
   AdmsSensFadType & modelPar_WTH0,
   bool modelPar_given_WTH0,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KU0,
   bool modelPar_given_KU0,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKU0,
   bool modelPar_given_TKU0,
   AdmsSensFadType & modelPar_LKU0,
   bool modelPar_given_LKU0,
   AdmsSensFadType & modelPar_WKU0,
   bool modelPar_given_WKU0,
   AdmsSensFadType & modelPar_PKU0,
   bool modelPar_given_PKU0,
   AdmsSensFadType & modelPar_LLODKU0,
   bool modelPar_given_LLODKU0,
   AdmsSensFadType & modelPar_WLODKU0,
   bool modelPar_given_WLODKU0,
   AdmsSensFadType & modelPar_KVTH0,
   bool modelPar_given_KVTH0,
   AdmsSensFadType & modelPar_LKVTH0,
   bool modelPar_given_LKVTH0,
   AdmsSensFadType & modelPar_WKVTH0,
   bool modelPar_given_WKVTH0,
   AdmsSensFadType & modelPar_PKVTH0,
   bool modelPar_given_PKVTH0,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STK2,
   bool modelPar_given_STK2,
   AdmsSensFadType & modelPar_LODK2,
   bool modelPar_given_LODK2,
   AdmsSensFadType & modelPar_STETA0,
   bool modelPar_given_STETA0,
   AdmsSensFadType & modelPar_LODETA0,
   bool modelPar_given_LODETA0,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_KVTH0WE,
   bool modelPar_given_KVTH0WE,
   AdmsSensFadType & modelPar_LKVTH0WE,
   bool modelPar_given_LKVTH0WE,
   AdmsSensFadType & modelPar_WKVTH0WE,
   bool modelPar_given_WKVTH0WE,
   AdmsSensFadType & modelPar_PKVTH0WE,
   bool modelPar_given_PKVTH0WE,
   AdmsSensFadType & modelPar_K2WE,
   bool modelPar_given_K2WE,
   AdmsSensFadType & modelPar_LK2WE,
   bool modelPar_given_LK2WE,
   AdmsSensFadType & modelPar_WK2WE,
   bool modelPar_given_WK2WE,
   AdmsSensFadType & modelPar_PK2WE,
   bool modelPar_given_PK2WE,
   AdmsSensFadType & modelPar_KU0WE,
   bool modelPar_given_KU0WE,
   AdmsSensFadType & modelPar_LKU0WE,
   bool modelPar_given_LKU0WE,
   AdmsSensFadType & modelPar_WKU0WE,
   bool modelPar_given_WKU0WE,
   AdmsSensFadType & modelPar_PKU0WE,
   bool modelPar_given_PKU0WE,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEDGE,
   bool modelPar_given_WEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGE,
   bool modelPar_given_DGAMMAEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGEL,
   bool modelPar_given_DGAMMAEDGEL,
   AdmsSensFadType & modelPar_DGAMMAEDGELEXP,
   bool modelPar_given_DGAMMAEDGELEXP,
   AdmsSensFadType & modelPar_DVTEDGE,
   bool modelPar_given_DVTEDGE,
   AdmsSensFadType & modelPar_NFACTOREDGE,
   bool modelPar_given_NFACTOREDGE,
   AdmsSensFadType & modelPar_CITEDGE,
   bool modelPar_given_CITEDGE,
   AdmsSensFadType & modelPar_CDSCDEDGE,
   bool modelPar_given_CDSCDEDGE,
   AdmsSensFadType & modelPar_CDSCBEDGE,
   bool modelPar_given_CDSCBEDGE,
   AdmsSensFadType & modelPar_ETA0EDGE,
   bool modelPar_given_ETA0EDGE,
   AdmsSensFadType & modelPar_ETABEDGE,
   bool modelPar_given_ETABEDGE,
   AdmsSensFadType & modelPar_KT1EDGE,
   bool modelPar_given_KT1EDGE,
   AdmsSensFadType & modelPar_KT1LEDGE,
   bool modelPar_given_KT1LEDGE,
   AdmsSensFadType & modelPar_KT2EDGE,
   bool modelPar_given_KT2EDGE,
   AdmsSensFadType & modelPar_KT1EXPEDGE,
   bool modelPar_given_KT1EXPEDGE,
   AdmsSensFadType & modelPar_TNFACTOREDGE,
   bool modelPar_given_TNFACTOREDGE,
   AdmsSensFadType & modelPar_TETA0EDGE,
   bool modelPar_given_TETA0EDGE,
   AdmsSensFadType & modelPar_DVT0EDGE,
   bool modelPar_given_DVT0EDGE,
   AdmsSensFadType & modelPar_DVT1EDGE,
   bool modelPar_given_DVT1EDGE,
   AdmsSensFadType & modelPar_DVT2EDGE,
   bool modelPar_given_DVT2EDGE,
   // non-reals (including hidden)
   int modelPar_NGCON,
   bool modelPar_given_NGCON,
   int modelPar_RGATEMOD,
   bool modelPar_given_RGATEMOD,
   int modelPar_RBODYMOD,
   bool modelPar_given_RBODYMOD,
   int modelPar_GEOMOD,
   bool modelPar_given_GEOMOD,
   int modelPar_RGEOMOD,
   bool modelPar_given_RGEOMOD,
   int modelPar_SCA,
   bool modelPar_given_SCA,
   int modelPar_SCB,
   bool modelPar_given_SCB,
   int modelPar_SCC,
   bool modelPar_given_SCC,
   int modelPar_SC,
   bool modelPar_given_SC,
   int modelPar_TYPE,
   bool modelPar_given_TYPE,
   int modelPar_CVMOD,
   bool modelPar_given_CVMOD,
   int modelPar_COVMOD,
   bool modelPar_given_COVMOD,
   int modelPar_RDSMOD,
   bool modelPar_given_RDSMOD,
   int modelPar_WPEMOD,
   bool modelPar_given_WPEMOD,
   int modelPar_ASYMMOD,
   bool modelPar_given_ASYMMOD,
   int modelPar_GIDLMOD,
   bool modelPar_given_GIDLMOD,
   int modelPar_IGCMOD,
   bool modelPar_given_IGCMOD,
   int modelPar_IGBMOD,
   bool modelPar_given_IGBMOD,
   int modelPar_TNOIMOD,
   bool modelPar_given_TNOIMOD,
   int modelPar_SHMOD,
   bool modelPar_given_SHMOD,
   int modelPar_MOBSCALE,
   bool modelPar_given_MOBSCALE,
   int modelPar_PERMOD,
   bool modelPar_given_PERMOD,
   int modelPar_BINUNIT,
   bool modelPar_given_BINUNIT,
   int modelPar_EDGEFET,
   bool modelPar_given_EDGEFET,
   // basic variables
   double admsTemperature, double adms_vt_nom, double ADMSgmin_arg, std::vector <AdmsSensFadType> & staticContributions, std::vector <AdmsSensFadType> & dynamicContributions, const Instance & theInstance);

void evaluateInitialInstance(
   // instance parameters
   // reals
   AdmsSensFadType & instancePar_L,
   bool instancePar_given_L,
   AdmsSensFadType & instancePar_W,
   bool instancePar_given_W,
   AdmsSensFadType & instancePar_NRS,
   bool instancePar_given_NRS,
   AdmsSensFadType & instancePar_NRD,
   bool instancePar_given_NRD,
   AdmsSensFadType & instancePar_VFBSDOFF,
   bool instancePar_given_VFBSDOFF,
   AdmsSensFadType & instancePar_XGW,
   bool instancePar_given_XGW,
   AdmsSensFadType & instancePar_RBPB,
   bool instancePar_given_RBPB,
   AdmsSensFadType & instancePar_RBPD,
   bool instancePar_given_RBPD,
   AdmsSensFadType & instancePar_RBPS,
   bool instancePar_given_RBPS,
   AdmsSensFadType & instancePar_RBDB,
   bool instancePar_given_RBDB,
   AdmsSensFadType & instancePar_RBSB,
   bool instancePar_given_RBSB,
   AdmsSensFadType & instancePar_SA,
   bool instancePar_given_SA,
   AdmsSensFadType & instancePar_SB,
   bool instancePar_given_SB,
   AdmsSensFadType & instancePar_SD,
   bool instancePar_given_SD,
   AdmsSensFadType & instancePar_AS,
   bool instancePar_given_AS,
   AdmsSensFadType & instancePar_AD,
   bool instancePar_given_AD,
   AdmsSensFadType & instancePar_PS,
   bool instancePar_given_PS,
   AdmsSensFadType & instancePar_PD,
   bool instancePar_given_PD,
   // non-reals(including hidden)
   int instancePar_NF,
   bool instancePar_given_NF,
   int instancePar_MINZ,
   bool instancePar_given_MINZ,
   int instancePar_NGCON,
   bool instancePar_given_NGCON,
   int instancePar_RGATEMOD,
   bool instancePar_given_RGATEMOD,
   int instancePar_RBODYMOD,
   bool instancePar_given_RBODYMOD,
   int instancePar_GEOMOD,
   bool instancePar_given_GEOMOD,
   int instancePar_RGEOMOD,
   bool instancePar_given_RGEOMOD,
   int instancePar_SCA,
   bool instancePar_given_SCA,
   int instancePar_SCB,
   bool instancePar_given_SCB,
   int instancePar_SCC,
   bool instancePar_given_SCC,
   int instancePar_SC,
   bool instancePar_given_SC,
   // instance variables
   // reals
   AdmsSensFadType & instanceVar_T0,
   AdmsSensFadType & instanceVar_T1,
   AdmsSensFadType & instanceVar_T2,
   AdmsSensFadType & instanceVar_epssi,
   AdmsSensFadType & instanceVar_epsox,
   AdmsSensFadType & instanceVar_Weff,
   AdmsSensFadType & instanceVar_Leff,
   AdmsSensFadType & instanceVar_Wact,
   AdmsSensFadType & instanceVar_Lact,
   AdmsSensFadType & instanceVar_Weffcj,
   AdmsSensFadType & instanceVar_Cox,
   AdmsSensFadType & instanceVar_epsratio,
   AdmsSensFadType & instanceVar_Lnew,
   AdmsSensFadType & instanceVar_L_mult,
   AdmsSensFadType & instanceVar_Wnew,
   AdmsSensFadType & instanceVar_DVTP0_i,
   AdmsSensFadType & instanceVar_DVTP1_i,
   AdmsSensFadType & instanceVar_DVTP2_i,
   AdmsSensFadType & instanceVar_DVTP3_i,
   AdmsSensFadType & instanceVar_DVTP4_i,
   AdmsSensFadType & instanceVar_DVTP5_i,
   AdmsSensFadType & instanceVar_A1_i,
   AdmsSensFadType & instanceVar_A11_i,
   AdmsSensFadType & instanceVar_A2_i,
   AdmsSensFadType & instanceVar_A21_i,
   AdmsSensFadType & instanceVar_PSAT_i,
   AdmsSensFadType & instanceVar_PSATR_i,
   AdmsSensFadType & instanceVar_BSIM6TOXP,
   AdmsSensFadType & instanceVar_DMCGeff,
   AdmsSensFadType & instanceVar_DMDGeff,
   AdmsSensFadType & instanceVar_DMCIeff,
   AdmsSensFadType & instanceVar_RSourceGeo,
   AdmsSensFadType & instanceVar_RDrainGeo,
   AdmsSensFadType & instanceVar_Grgeltd,
   AdmsSensFadType & instanceVar_Grbsb,
   AdmsSensFadType & instanceVar_Grbdb,
   AdmsSensFadType & instanceVar_Grbpb,
   AdmsSensFadType & instanceVar_Grbps,
   AdmsSensFadType & instanceVar_Grbpd,
   AdmsSensFadType & instanceVar_Aechvb,
   AdmsSensFadType & instanceVar_Bechvb,
   AdmsSensFadType & instanceVar_AechvbEdge,
   AdmsSensFadType & instanceVar_BechvbEdge,
   AdmsSensFadType & instanceVar_ToxRatio,
   AdmsSensFadType & instanceVar_U0R_t,
   AdmsSensFadType & instanceVar_UAR_t,
   AdmsSensFadType & instanceVar_UCR_t,
   AdmsSensFadType & instanceVar_UDR_t,
   AdmsSensFadType & instanceVar_UCSR_t,
   AdmsSensFadType & instanceVar_VSATR_t,
   AdmsSensFadType & instanceVar_PTWGR_t,
   AdmsSensFadType & instanceVar_ETA0R_t,
   AdmsSensFadType & instanceVar_PSATB_i,
   AdmsSensFadType & instanceVar_KT1_i,
   AdmsSensFadType & instanceVar_KT2_i,
   AdmsSensFadType & instanceVar_NGATE_i,
   AdmsSensFadType & instanceVar_NSD_i,
   AdmsSensFadType & instanceVar_NDEP_i,
   AdmsSensFadType & instanceVar_VFB_i,
   AdmsSensFadType & instanceVar_CIT_i,
   AdmsSensFadType & instanceVar_CDSCD_i,
   AdmsSensFadType & instanceVar_CDSCDR_i,
   AdmsSensFadType & instanceVar_CDSCB_i,
   AdmsSensFadType & instanceVar_NFACTOR_i,
   AdmsSensFadType & instanceVar_PHIN_i,
   AdmsSensFadType & instanceVar_ETA0_i,
   AdmsSensFadType & instanceVar_ETA0R_i,
   AdmsSensFadType & instanceVar_ETAB_i,
   AdmsSensFadType & instanceVar_K2_i,
   AdmsSensFadType & instanceVar_K1_i,
   AdmsSensFadType & instanceVar_DELTA_i,
   AdmsSensFadType & instanceVar_U0_i,
   AdmsSensFadType & instanceVar_U0R_i,
   AdmsSensFadType & instanceVar_VSAT_i,
   AdmsSensFadType & instanceVar_VSATR_i,
   AdmsSensFadType & instanceVar_VSATCV_i,
   AdmsSensFadType & instanceVar_UA_i,
   AdmsSensFadType & instanceVar_UAR_i,
   AdmsSensFadType & instanceVar_EU_i,
   AdmsSensFadType & instanceVar_UD_i,
   AdmsSensFadType & instanceVar_UDR_i,
   AdmsSensFadType & instanceVar_UD_a,
   AdmsSensFadType & instanceVar_UCS_i,
   AdmsSensFadType & instanceVar_UCSR_i,
   AdmsSensFadType & instanceVar_UC_i,
   AdmsSensFadType & instanceVar_UCR_i,
   AdmsSensFadType & instanceVar_PDIBLC_i,
   AdmsSensFadType & instanceVar_PDIBLCR_i,
   AdmsSensFadType & instanceVar_PDIBLCB_i,
   AdmsSensFadType & instanceVar_PSCBE1_i,
   AdmsSensFadType & instanceVar_PSCBE2_i,
   AdmsSensFadType & instanceVar_PDITS_i,
   AdmsSensFadType & instanceVar_PDITSD_i,
   AdmsSensFadType & instanceVar_FPROUT_i,
   AdmsSensFadType & instanceVar_PRWG_i,
   AdmsSensFadType & instanceVar_WR_i,
   AdmsSensFadType & instanceVar_RDWMIN_i,
   AdmsSensFadType & instanceVar_RSWMIN_i,
   AdmsSensFadType & instanceVar_RDW_i,
   AdmsSensFadType & instanceVar_RSW_i,
   AdmsSensFadType & instanceVar_RDSW_i,
   AdmsSensFadType & instanceVar_RDSWMIN_i,
   AdmsSensFadType & instanceVar_PTWG_i,
   AdmsSensFadType & instanceVar_PTWGR_i,
   AdmsSensFadType & instanceVar_PVAG_i,
   AdmsSensFadType & instanceVar_XJ_i,
   AdmsSensFadType & instanceVar_PCLM_i,
   AdmsSensFadType & instanceVar_PCLMR_i,
   AdmsSensFadType & instanceVar_PCLMCV_i,
   AdmsSensFadType & instanceVar_PRWB_i,
   AdmsSensFadType & instanceVar_CF_i,
   AdmsSensFadType & instanceVar_CGSL_i,
   AdmsSensFadType & instanceVar_CGDL_i,
   AdmsSensFadType & instanceVar_CKAPPAS_i,
   AdmsSensFadType & instanceVar_CKAPPAD_i,
   AdmsSensFadType & instanceVar_VFBCV_i,
   AdmsSensFadType & instanceVar_NDEPCV_i,
   AdmsSensFadType & instanceVar_ALPHA0_i,
   AdmsSensFadType & instanceVar_BETA0_i,
   AdmsSensFadType & instanceVar_AGIDL_i,
   AdmsSensFadType & instanceVar_BGIDL_i,
   AdmsSensFadType & instanceVar_CGIDL_i,
   AdmsSensFadType & instanceVar_EGIDL_i,
   AdmsSensFadType & instanceVar_AGISL_i,
   AdmsSensFadType & instanceVar_BGISL_i,
   AdmsSensFadType & instanceVar_CGISL_i,
   AdmsSensFadType & instanceVar_EGISL_i,
   AdmsSensFadType & instanceVar_UTE_i,
   AdmsSensFadType & instanceVar_UA1_i,
   AdmsSensFadType & instanceVar_UC1_i,
   AdmsSensFadType & instanceVar_UD1_i,
   AdmsSensFadType & instanceVar_UCSTE_i,
   AdmsSensFadType & instanceVar_PRT_i,
   AdmsSensFadType & instanceVar_AT_i,
   AdmsSensFadType & instanceVar_PTWGT_i,
   AdmsSensFadType & instanceVar_IIT_i,
   AdmsSensFadType & instanceVar_TGIDL_i,
   AdmsSensFadType & instanceVar_KVTH0WE_i,
   AdmsSensFadType & instanceVar_K2WE_i,
   AdmsSensFadType & instanceVar_KU0WE_i,
   AdmsSensFadType & instanceVar_AIGBINV_i,
   AdmsSensFadType & instanceVar_BIGBINV_i,
   AdmsSensFadType & instanceVar_CIGBINV_i,
   AdmsSensFadType & instanceVar_EIGBINV_i,
   AdmsSensFadType & instanceVar_NIGBINV_i,
   AdmsSensFadType & instanceVar_AIGBACC_i,
   AdmsSensFadType & instanceVar_BIGBACC_i,
   AdmsSensFadType & instanceVar_CIGBACC_i,
   AdmsSensFadType & instanceVar_NIGBACC_i,
   AdmsSensFadType & instanceVar_AIGC_i,
   AdmsSensFadType & instanceVar_BIGC_i,
   AdmsSensFadType & instanceVar_CIGC_i,
   AdmsSensFadType & instanceVar_AIGS_i,
   AdmsSensFadType & instanceVar_BIGS_i,
   AdmsSensFadType & instanceVar_CIGS_i,
   AdmsSensFadType & instanceVar_AIGD_i,
   AdmsSensFadType & instanceVar_BIGD_i,
   AdmsSensFadType & instanceVar_CIGD_i,
   AdmsSensFadType & instanceVar_PIGCD_i,
   AdmsSensFadType & instanceVar_DLCIG_i,
   AdmsSensFadType & instanceVar_DLCIGD_i,
   AdmsSensFadType & instanceVar_IGT_i,
   AdmsSensFadType & instanceVar_nuIntD,
   AdmsSensFadType & instanceVar_nuEndD,
   AdmsSensFadType & instanceVar_nuIntS,
   AdmsSensFadType & instanceVar_nuEndS,
   AdmsSensFadType & instanceVar_gth,
   AdmsSensFadType & instanceVar_cth,
   // non-reals
   int instanceVar_devsign,
   // model parameters
   // reals
   AdmsSensFadType & modelPar_L,
   bool modelPar_given_L,
   AdmsSensFadType & modelPar_VFBSDOFF,
   bool modelPar_given_VFBSDOFF,
   AdmsSensFadType & modelPar_XGW,
   bool modelPar_given_XGW,
   AdmsSensFadType & modelPar_RBPB,
   bool modelPar_given_RBPB,
   AdmsSensFadType & modelPar_RBPD,
   bool modelPar_given_RBPD,
   AdmsSensFadType & modelPar_RBPS,
   bool modelPar_given_RBPS,
   AdmsSensFadType & modelPar_RBDB,
   bool modelPar_given_RBDB,
   AdmsSensFadType & modelPar_RBSB,
   bool modelPar_given_RBSB,
   AdmsSensFadType & modelPar_SA,
   bool modelPar_given_SA,
   AdmsSensFadType & modelPar_SB,
   bool modelPar_given_SB,
   AdmsSensFadType & modelPar_SD,
   bool modelPar_given_SD,
   AdmsSensFadType & modelPar_LLONG,
   bool modelPar_given_LLONG,
   AdmsSensFadType & modelPar_LMLT,
   bool modelPar_given_LMLT,
   AdmsSensFadType & modelPar_WMLT,
   bool modelPar_given_WMLT,
   AdmsSensFadType & modelPar_XL,
   bool modelPar_given_XL,
   AdmsSensFadType & modelPar_WWIDE,
   bool modelPar_given_WWIDE,
   AdmsSensFadType & modelPar_XW,
   bool modelPar_given_XW,
   AdmsSensFadType & modelPar_LINT,
   bool modelPar_given_LINT,
   AdmsSensFadType & modelPar_LL,
   bool modelPar_given_LL,
   AdmsSensFadType & modelPar_LW,
   bool modelPar_given_LW,
   AdmsSensFadType & modelPar_LWL,
   bool modelPar_given_LWL,
   AdmsSensFadType & modelPar_LLN,
   bool modelPar_given_LLN,
   AdmsSensFadType & modelPar_LWN,
   bool modelPar_given_LWN,
   AdmsSensFadType & modelPar_WINT,
   bool modelPar_given_WINT,
   AdmsSensFadType & modelPar_WL,
   bool modelPar_given_WL,
   AdmsSensFadType & modelPar_WW,
   bool modelPar_given_WW,
   AdmsSensFadType & modelPar_WWL,
   bool modelPar_given_WWL,
   AdmsSensFadType & modelPar_WLN,
   bool modelPar_given_WLN,
   AdmsSensFadType & modelPar_WWN,
   bool modelPar_given_WWN,
   AdmsSensFadType & modelPar_DLC,
   bool modelPar_given_DLC,
   AdmsSensFadType & modelPar_LLC,
   bool modelPar_given_LLC,
   AdmsSensFadType & modelPar_LWC,
   bool modelPar_given_LWC,
   AdmsSensFadType & modelPar_LWLC,
   bool modelPar_given_LWLC,
   AdmsSensFadType & modelPar_DWC,
   bool modelPar_given_DWC,
   AdmsSensFadType & modelPar_WLC,
   bool modelPar_given_WLC,
   AdmsSensFadType & modelPar_WWC,
   bool modelPar_given_WWC,
   AdmsSensFadType & modelPar_WWLC,
   bool modelPar_given_WWLC,
   AdmsSensFadType & modelPar_TOXE,
   bool modelPar_given_TOXE,
   AdmsSensFadType & modelPar_TOXP,
   bool modelPar_given_TOXP,
   AdmsSensFadType & modelPar_DTOX,
   bool modelPar_given_DTOX,
   AdmsSensFadType & modelPar_NDEP,
   bool modelPar_given_NDEP,
   AdmsSensFadType & modelPar_NDEPL1,
   bool modelPar_given_NDEPL1,
   AdmsSensFadType & modelPar_NDEPLEXP1,
   bool modelPar_given_NDEPLEXP1,
   AdmsSensFadType & modelPar_NDEPL2,
   bool modelPar_given_NDEPL2,
   AdmsSensFadType & modelPar_NDEPLEXP2,
   bool modelPar_given_NDEPLEXP2,
   AdmsSensFadType & modelPar_NDEPW,
   bool modelPar_given_NDEPW,
   AdmsSensFadType & modelPar_NDEPWEXP,
   bool modelPar_given_NDEPWEXP,
   AdmsSensFadType & modelPar_NDEPWL,
   bool modelPar_given_NDEPWL,
   AdmsSensFadType & modelPar_NDEPWLEXP,
   bool modelPar_given_NDEPWLEXP,
   AdmsSensFadType & modelPar_LNDEP,
   bool modelPar_given_LNDEP,
   AdmsSensFadType & modelPar_WNDEP,
   bool modelPar_given_WNDEP,
   AdmsSensFadType & modelPar_PNDEP,
   bool modelPar_given_PNDEP,
   AdmsSensFadType & modelPar_NDEPCV,
   bool modelPar_given_NDEPCV,
   AdmsSensFadType & modelPar_NDEPCVL1,
   bool modelPar_given_NDEPCVL1,
   AdmsSensFadType & modelPar_NDEPCVLEXP1,
   bool modelPar_given_NDEPCVLEXP1,
   AdmsSensFadType & modelPar_NDEPCVL2,
   bool modelPar_given_NDEPCVL2,
   AdmsSensFadType & modelPar_NDEPCVLEXP2,
   bool modelPar_given_NDEPCVLEXP2,
   AdmsSensFadType & modelPar_NDEPCVW,
   bool modelPar_given_NDEPCVW,
   AdmsSensFadType & modelPar_NDEPCVWEXP,
   bool modelPar_given_NDEPCVWEXP,
   AdmsSensFadType & modelPar_NDEPCVWL,
   bool modelPar_given_NDEPCVWL,
   AdmsSensFadType & modelPar_NDEPCVWLEXP,
   bool modelPar_given_NDEPCVWLEXP,
   AdmsSensFadType & modelPar_LNDEPCV,
   bool modelPar_given_LNDEPCV,
   AdmsSensFadType & modelPar_WNDEPCV,
   bool modelPar_given_WNDEPCV,
   AdmsSensFadType & modelPar_PNDEPCV,
   bool modelPar_given_PNDEPCV,
   AdmsSensFadType & modelPar_NGATE,
   bool modelPar_given_NGATE,
   AdmsSensFadType & modelPar_LNGATE,
   bool modelPar_given_LNGATE,
   AdmsSensFadType & modelPar_WNGATE,
   bool modelPar_given_WNGATE,
   AdmsSensFadType & modelPar_PNGATE,
   bool modelPar_given_PNGATE,
   AdmsSensFadType & modelPar_EASUB,
   bool modelPar_given_EASUB,
   AdmsSensFadType & modelPar_NI0SUB,
   bool modelPar_given_NI0SUB,
   AdmsSensFadType & modelPar_BG0SUB,
   bool modelPar_given_BG0SUB,
   AdmsSensFadType & modelPar_EPSRSUB,
   bool modelPar_given_EPSRSUB,
   AdmsSensFadType & modelPar_EPSROX,
   bool modelPar_given_EPSROX,
   AdmsSensFadType & modelPar_XJ,
   bool modelPar_given_XJ,
   AdmsSensFadType & modelPar_LXJ,
   bool modelPar_given_LXJ,
   AdmsSensFadType & modelPar_WXJ,
   bool modelPar_given_WXJ,
   AdmsSensFadType & modelPar_PXJ,
   bool modelPar_given_PXJ,
   AdmsSensFadType & modelPar_LVFB,
   bool modelPar_given_LVFB,
   AdmsSensFadType & modelPar_WVFB,
   bool modelPar_given_WVFB,
   AdmsSensFadType & modelPar_PVFB,
   bool modelPar_given_PVFB,
   AdmsSensFadType & modelPar_VFB,
   bool modelPar_given_VFB,
   AdmsSensFadType & modelPar_VFBCV,
   bool modelPar_given_VFBCV,
   AdmsSensFadType & modelPar_LVFBCV,
   bool modelPar_given_LVFBCV,
   AdmsSensFadType & modelPar_WVFBCV,
   bool modelPar_given_WVFBCV,
   AdmsSensFadType & modelPar_PVFBCV,
   bool modelPar_given_PVFBCV,
   AdmsSensFadType & modelPar_VFBCVL,
   bool modelPar_given_VFBCVL,
   AdmsSensFadType & modelPar_VFBCVLEXP,
   bool modelPar_given_VFBCVLEXP,
   AdmsSensFadType & modelPar_VFBCVW,
   bool modelPar_given_VFBCVW,
   AdmsSensFadType & modelPar_VFBCVWEXP,
   bool modelPar_given_VFBCVWEXP,
   AdmsSensFadType & modelPar_VFBCVWL,
   bool modelPar_given_VFBCVWL,
   AdmsSensFadType & modelPar_VFBCVWLEXP,
   bool modelPar_given_VFBCVWLEXP,
   AdmsSensFadType & modelPar_DWJ,
   bool modelPar_given_DWJ,
   AdmsSensFadType & modelPar_NSD,
   bool modelPar_given_NSD,
   AdmsSensFadType & modelPar_LNSD,
   bool modelPar_given_LNSD,
   AdmsSensFadType & modelPar_WNSD,
   bool modelPar_given_WNSD,
   AdmsSensFadType & modelPar_PNSD,
   bool modelPar_given_PNSD,
   AdmsSensFadType & modelPar_DELVT0,
   bool modelPar_given_DELVT0,
   AdmsSensFadType & modelPar_DVTP0,
   bool modelPar_given_DVTP0,
   AdmsSensFadType & modelPar_LDVTP0,
   bool modelPar_given_LDVTP0,
   AdmsSensFadType & modelPar_WDVTP0,
   bool modelPar_given_WDVTP0,
   AdmsSensFadType & modelPar_PDVTP0,
   bool modelPar_given_PDVTP0,
   AdmsSensFadType & modelPar_DVTP1,
   bool modelPar_given_DVTP1,
   AdmsSensFadType & modelPar_LDVTP1,
   bool modelPar_given_LDVTP1,
   AdmsSensFadType & modelPar_WDVTP1,
   bool modelPar_given_WDVTP1,
   AdmsSensFadType & modelPar_PDVTP1,
   bool modelPar_given_PDVTP1,
   AdmsSensFadType & modelPar_DVTP2,
   bool modelPar_given_DVTP2,
   AdmsSensFadType & modelPar_LDVTP2,
   bool modelPar_given_LDVTP2,
   AdmsSensFadType & modelPar_WDVTP2,
   bool modelPar_given_WDVTP2,
   AdmsSensFadType & modelPar_PDVTP2,
   bool modelPar_given_PDVTP2,
   AdmsSensFadType & modelPar_DVTP3,
   bool modelPar_given_DVTP3,
   AdmsSensFadType & modelPar_LDVTP3,
   bool modelPar_given_LDVTP3,
   AdmsSensFadType & modelPar_WDVTP3,
   bool modelPar_given_WDVTP3,
   AdmsSensFadType & modelPar_PDVTP3,
   bool modelPar_given_PDVTP3,
   AdmsSensFadType & modelPar_DVTP4,
   bool modelPar_given_DVTP4,
   AdmsSensFadType & modelPar_LDVTP4,
   bool modelPar_given_LDVTP4,
   AdmsSensFadType & modelPar_WDVTP4,
   bool modelPar_given_WDVTP4,
   AdmsSensFadType & modelPar_PDVTP4,
   bool modelPar_given_PDVTP4,
   AdmsSensFadType & modelPar_DVTP5,
   bool modelPar_given_DVTP5,
   AdmsSensFadType & modelPar_LDVTP5,
   bool modelPar_given_LDVTP5,
   AdmsSensFadType & modelPar_WDVTP5,
   bool modelPar_given_WDVTP5,
   AdmsSensFadType & modelPar_PDVTP5,
   bool modelPar_given_PDVTP5,
   AdmsSensFadType & modelPar_PHIN,
   bool modelPar_given_PHIN,
   AdmsSensFadType & modelPar_LPHIN,
   bool modelPar_given_LPHIN,
   AdmsSensFadType & modelPar_WPHIN,
   bool modelPar_given_WPHIN,
   AdmsSensFadType & modelPar_PPHIN,
   bool modelPar_given_PPHIN,
   AdmsSensFadType & modelPar_ETA0,
   bool modelPar_given_ETA0,
   AdmsSensFadType & modelPar_LETA0,
   bool modelPar_given_LETA0,
   AdmsSensFadType & modelPar_WETA0,
   bool modelPar_given_WETA0,
   AdmsSensFadType & modelPar_PETA0,
   bool modelPar_given_PETA0,
   AdmsSensFadType & modelPar_ETA0R,
   bool modelPar_given_ETA0R,
   AdmsSensFadType & modelPar_LETA0R,
   bool modelPar_given_LETA0R,
   AdmsSensFadType & modelPar_WETA0R,
   bool modelPar_given_WETA0R,
   AdmsSensFadType & modelPar_PETA0R,
   bool modelPar_given_PETA0R,
   AdmsSensFadType & modelPar_DSUB,
   bool modelPar_given_DSUB,
   AdmsSensFadType & modelPar_ETAB,
   bool modelPar_given_ETAB,
   AdmsSensFadType & modelPar_ETABEXP,
   bool modelPar_given_ETABEXP,
   AdmsSensFadType & modelPar_LETAB,
   bool modelPar_given_LETAB,
   AdmsSensFadType & modelPar_WETAB,
   bool modelPar_given_WETAB,
   AdmsSensFadType & modelPar_PETAB,
   bool modelPar_given_PETAB,
   AdmsSensFadType & modelPar_K1,
   bool modelPar_given_K1,
   AdmsSensFadType & modelPar_K1L,
   bool modelPar_given_K1L,
   AdmsSensFadType & modelPar_K1LEXP,
   bool modelPar_given_K1LEXP,
   AdmsSensFadType & modelPar_K1W,
   bool modelPar_given_K1W,
   AdmsSensFadType & modelPar_K1WEXP,
   bool modelPar_given_K1WEXP,
   AdmsSensFadType & modelPar_K1WL,
   bool modelPar_given_K1WL,
   AdmsSensFadType & modelPar_K1WLEXP,
   bool modelPar_given_K1WLEXP,
   AdmsSensFadType & modelPar_LK1,
   bool modelPar_given_LK1,
   AdmsSensFadType & modelPar_WK1,
   bool modelPar_given_WK1,
   AdmsSensFadType & modelPar_PK1,
   bool modelPar_given_PK1,
   AdmsSensFadType & modelPar_K2,
   bool modelPar_given_K2,
   AdmsSensFadType & modelPar_K2L,
   bool modelPar_given_K2L,
   AdmsSensFadType & modelPar_K2LEXP,
   bool modelPar_given_K2LEXP,
   AdmsSensFadType & modelPar_K2W,
   bool modelPar_given_K2W,
   AdmsSensFadType & modelPar_K2WEXP,
   bool modelPar_given_K2WEXP,
   AdmsSensFadType & modelPar_K2WL,
   bool modelPar_given_K2WL,
   AdmsSensFadType & modelPar_K2WLEXP,
   bool modelPar_given_K2WLEXP,
   AdmsSensFadType & modelPar_LK2,
   bool modelPar_given_LK2,
   AdmsSensFadType & modelPar_WK2,
   bool modelPar_given_WK2,
   AdmsSensFadType & modelPar_PK2,
   bool modelPar_given_PK2,
   AdmsSensFadType & modelPar_ADOS,
   bool modelPar_given_ADOS,
   AdmsSensFadType & modelPar_BDOS,
   bool modelPar_given_BDOS,
   AdmsSensFadType & modelPar_QM0,
   bool modelPar_given_QM0,
   AdmsSensFadType & modelPar_ETAQM,
   bool modelPar_given_ETAQM,
   AdmsSensFadType & modelPar_CIT,
   bool modelPar_given_CIT,
   AdmsSensFadType & modelPar_LCIT,
   bool modelPar_given_LCIT,
   AdmsSensFadType & modelPar_WCIT,
   bool modelPar_given_WCIT,
   AdmsSensFadType & modelPar_PCIT,
   bool modelPar_given_PCIT,
   AdmsSensFadType & modelPar_NFACTOR,
   bool modelPar_given_NFACTOR,
   AdmsSensFadType & modelPar_NFACTORL,
   bool modelPar_given_NFACTORL,
   AdmsSensFadType & modelPar_NFACTORLEXP,
   bool modelPar_given_NFACTORLEXP,
   AdmsSensFadType & modelPar_NFACTORW,
   bool modelPar_given_NFACTORW,
   AdmsSensFadType & modelPar_NFACTORWEXP,
   bool modelPar_given_NFACTORWEXP,
   AdmsSensFadType & modelPar_NFACTORWL,
   bool modelPar_given_NFACTORWL,
   AdmsSensFadType & modelPar_NFACTORWLEXP,
   bool modelPar_given_NFACTORWLEXP,
   AdmsSensFadType & modelPar_LNFACTOR,
   bool modelPar_given_LNFACTOR,
   AdmsSensFadType & modelPar_WNFACTOR,
   bool modelPar_given_WNFACTOR,
   AdmsSensFadType & modelPar_PNFACTOR,
   bool modelPar_given_PNFACTOR,
   AdmsSensFadType & modelPar_CDSCD,
   bool modelPar_given_CDSCD,
   AdmsSensFadType & modelPar_CDSCDL,
   bool modelPar_given_CDSCDL,
   AdmsSensFadType & modelPar_CDSCDLEXP,
   bool modelPar_given_CDSCDLEXP,
   AdmsSensFadType & modelPar_LCDSCD,
   bool modelPar_given_LCDSCD,
   AdmsSensFadType & modelPar_WCDSCD,
   bool modelPar_given_WCDSCD,
   AdmsSensFadType & modelPar_PCDSCD,
   bool modelPar_given_PCDSCD,
   AdmsSensFadType & modelPar_CDSCDR,
   bool modelPar_given_CDSCDR,
   AdmsSensFadType & modelPar_CDSCDLR,
   bool modelPar_given_CDSCDLR,
   AdmsSensFadType & modelPar_LCDSCDR,
   bool modelPar_given_LCDSCDR,
   AdmsSensFadType & modelPar_WCDSCDR,
   bool modelPar_given_WCDSCDR,
   AdmsSensFadType & modelPar_PCDSCDR,
   bool modelPar_given_PCDSCDR,
   AdmsSensFadType & modelPar_CDSCB,
   bool modelPar_given_CDSCB,
   AdmsSensFadType & modelPar_CDSCBL,
   bool modelPar_given_CDSCBL,
   AdmsSensFadType & modelPar_CDSCBLEXP,
   bool modelPar_given_CDSCBLEXP,
   AdmsSensFadType & modelPar_LCDSCB,
   bool modelPar_given_LCDSCB,
   AdmsSensFadType & modelPar_WCDSCB,
   bool modelPar_given_WCDSCB,
   AdmsSensFadType & modelPar_PCDSCB,
   bool modelPar_given_PCDSCB,
   AdmsSensFadType & modelPar_VSAT,
   bool modelPar_given_VSAT,
   AdmsSensFadType & modelPar_LVSAT,
   bool modelPar_given_LVSAT,
   AdmsSensFadType & modelPar_WVSAT,
   bool modelPar_given_WVSAT,
   AdmsSensFadType & modelPar_PVSAT,
   bool modelPar_given_PVSAT,
   AdmsSensFadType & modelPar_VSATL,
   bool modelPar_given_VSATL,
   AdmsSensFadType & modelPar_VSATLEXP,
   bool modelPar_given_VSATLEXP,
   AdmsSensFadType & modelPar_VSATW,
   bool modelPar_given_VSATW,
   AdmsSensFadType & modelPar_VSATWEXP,
   bool modelPar_given_VSATWEXP,
   AdmsSensFadType & modelPar_VSATWL,
   bool modelPar_given_VSATWL,
   AdmsSensFadType & modelPar_VSATWLEXP,
   bool modelPar_given_VSATWLEXP,
   AdmsSensFadType & modelPar_VSATR,
   bool modelPar_given_VSATR,
   AdmsSensFadType & modelPar_LVSATR,
   bool modelPar_given_LVSATR,
   AdmsSensFadType & modelPar_WVSATR,
   bool modelPar_given_WVSATR,
   AdmsSensFadType & modelPar_PVSATR,
   bool modelPar_given_PVSATR,
   AdmsSensFadType & modelPar_DELTA,
   bool modelPar_given_DELTA,
   AdmsSensFadType & modelPar_LDELTA,
   bool modelPar_given_LDELTA,
   AdmsSensFadType & modelPar_WDELTA,
   bool modelPar_given_WDELTA,
   AdmsSensFadType & modelPar_PDELTA,
   bool modelPar_given_PDELTA,
   AdmsSensFadType & modelPar_DELTAL,
   bool modelPar_given_DELTAL,
   AdmsSensFadType & modelPar_DELTALEXP,
   bool modelPar_given_DELTALEXP,
   AdmsSensFadType & modelPar_VSATCV,
   bool modelPar_given_VSATCV,
   AdmsSensFadType & modelPar_LVSATCV,
   bool modelPar_given_LVSATCV,
   AdmsSensFadType & modelPar_WVSATCV,
   bool modelPar_given_WVSATCV,
   AdmsSensFadType & modelPar_PVSATCV,
   bool modelPar_given_PVSATCV,
   AdmsSensFadType & modelPar_VSATCVL,
   bool modelPar_given_VSATCVL,
   AdmsSensFadType & modelPar_VSATCVLEXP,
   bool modelPar_given_VSATCVLEXP,
   AdmsSensFadType & modelPar_VSATCVW,
   bool modelPar_given_VSATCVW,
   AdmsSensFadType & modelPar_VSATCVWEXP,
   bool modelPar_given_VSATCVWEXP,
   AdmsSensFadType & modelPar_VSATCVWL,
   bool modelPar_given_VSATCVWL,
   AdmsSensFadType & modelPar_VSATCVWLEXP,
   bool modelPar_given_VSATCVWLEXP,
   AdmsSensFadType & modelPar_UP1,
   bool modelPar_given_UP1,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_UP2,
   bool modelPar_given_UP2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_MULU0,
   bool modelPar_given_MULU0,
   AdmsSensFadType & modelPar_U0,
   bool modelPar_given_U0,
   AdmsSensFadType & modelPar_U0L,
   bool modelPar_given_U0L,
   AdmsSensFadType & modelPar_U0LEXP,
   bool modelPar_given_U0LEXP,
   AdmsSensFadType & modelPar_LU0,
   bool modelPar_given_LU0,
   AdmsSensFadType & modelPar_WU0,
   bool modelPar_given_WU0,
   AdmsSensFadType & modelPar_PU0,
   bool modelPar_given_PU0,
   AdmsSensFadType & modelPar_U0R,
   bool modelPar_given_U0R,
   AdmsSensFadType & modelPar_LU0R,
   bool modelPar_given_LU0R,
   AdmsSensFadType & modelPar_WU0R,
   bool modelPar_given_WU0R,
   AdmsSensFadType & modelPar_PU0R,
   bool modelPar_given_PU0R,
   AdmsSensFadType & modelPar_ETAMOB,
   bool modelPar_given_ETAMOB,
   AdmsSensFadType & modelPar_UA,
   bool modelPar_given_UA,
   AdmsSensFadType & modelPar_UAL,
   bool modelPar_given_UAL,
   AdmsSensFadType & modelPar_UALEXP,
   bool modelPar_given_UALEXP,
   AdmsSensFadType & modelPar_UAW,
   bool modelPar_given_UAW,
   AdmsSensFadType & modelPar_UAWEXP,
   bool modelPar_given_UAWEXP,
   AdmsSensFadType & modelPar_UAWL,
   bool modelPar_given_UAWL,
   AdmsSensFadType & modelPar_UAWLEXP,
   bool modelPar_given_UAWLEXP,
   AdmsSensFadType & modelPar_LUA,
   bool modelPar_given_LUA,
   AdmsSensFadType & modelPar_WUA,
   bool modelPar_given_WUA,
   AdmsSensFadType & modelPar_PUA,
   bool modelPar_given_PUA,
   AdmsSensFadType & modelPar_UAR,
   bool modelPar_given_UAR,
   AdmsSensFadType & modelPar_LUAR,
   bool modelPar_given_LUAR,
   AdmsSensFadType & modelPar_WUAR,
   bool modelPar_given_WUAR,
   AdmsSensFadType & modelPar_PUAR,
   bool modelPar_given_PUAR,
   AdmsSensFadType & modelPar_EU,
   bool modelPar_given_EU,
   AdmsSensFadType & modelPar_LEU,
   bool modelPar_given_LEU,
   AdmsSensFadType & modelPar_WEU,
   bool modelPar_given_WEU,
   AdmsSensFadType & modelPar_PEU,
   bool modelPar_given_PEU,
   AdmsSensFadType & modelPar_EUL,
   bool modelPar_given_EUL,
   AdmsSensFadType & modelPar_EULEXP,
   bool modelPar_given_EULEXP,
   AdmsSensFadType & modelPar_EUW,
   bool modelPar_given_EUW,
   AdmsSensFadType & modelPar_EUWEXP,
   bool modelPar_given_EUWEXP,
   AdmsSensFadType & modelPar_EUWL,
   bool modelPar_given_EUWL,
   AdmsSensFadType & modelPar_EUWLEXP,
   bool modelPar_given_EUWLEXP,
   AdmsSensFadType & modelPar_UD,
   bool modelPar_given_UD,
   AdmsSensFadType & modelPar_UDL,
   bool modelPar_given_UDL,
   AdmsSensFadType & modelPar_UDLEXP,
   bool modelPar_given_UDLEXP,
   AdmsSensFadType & modelPar_LUD,
   bool modelPar_given_LUD,
   AdmsSensFadType & modelPar_WUD,
   bool modelPar_given_WUD,
   AdmsSensFadType & modelPar_PUD,
   bool modelPar_given_PUD,
   AdmsSensFadType & modelPar_UDR,
   bool modelPar_given_UDR,
   AdmsSensFadType & modelPar_LUDR,
   bool modelPar_given_LUDR,
   AdmsSensFadType & modelPar_WUDR,
   bool modelPar_given_WUDR,
   AdmsSensFadType & modelPar_PUDR,
   bool modelPar_given_PUDR,
   AdmsSensFadType & modelPar_UCS,
   bool modelPar_given_UCS,
   AdmsSensFadType & modelPar_LUCS,
   bool modelPar_given_LUCS,
   AdmsSensFadType & modelPar_WUCS,
   bool modelPar_given_WUCS,
   AdmsSensFadType & modelPar_PUCS,
   bool modelPar_given_PUCS,
   AdmsSensFadType & modelPar_UCSR,
   bool modelPar_given_UCSR,
   AdmsSensFadType & modelPar_LUCSR,
   bool modelPar_given_LUCSR,
   AdmsSensFadType & modelPar_WUCSR,
   bool modelPar_given_WUCSR,
   AdmsSensFadType & modelPar_PUCSR,
   bool modelPar_given_PUCSR,
   AdmsSensFadType & modelPar_UC,
   bool modelPar_given_UC,
   AdmsSensFadType & modelPar_UCL,
   bool modelPar_given_UCL,
   AdmsSensFadType & modelPar_UCLEXP,
   bool modelPar_given_UCLEXP,
   AdmsSensFadType & modelPar_UCW,
   bool modelPar_given_UCW,
   AdmsSensFadType & modelPar_UCWEXP,
   bool modelPar_given_UCWEXP,
   AdmsSensFadType & modelPar_UCWL,
   bool modelPar_given_UCWL,
   AdmsSensFadType & modelPar_UCWLEXP,
   bool modelPar_given_UCWLEXP,
   AdmsSensFadType & modelPar_LUC,
   bool modelPar_given_LUC,
   AdmsSensFadType & modelPar_WUC,
   bool modelPar_given_WUC,
   AdmsSensFadType & modelPar_PUC,
   bool modelPar_given_PUC,
   AdmsSensFadType & modelPar_UCR,
   bool modelPar_given_UCR,
   AdmsSensFadType & modelPar_LUCR,
   bool modelPar_given_LUCR,
   AdmsSensFadType & modelPar_WUCR,
   bool modelPar_given_WUCR,
   AdmsSensFadType & modelPar_PUCR,
   bool modelPar_given_PUCR,
   AdmsSensFadType & modelPar_PCLM,
   bool modelPar_given_PCLM,
   AdmsSensFadType & modelPar_PCLML,
   bool modelPar_given_PCLML,
   AdmsSensFadType & modelPar_PCLMLEXP,
   bool modelPar_given_PCLMLEXP,
   AdmsSensFadType & modelPar_LPCLM,
   bool modelPar_given_LPCLM,
   AdmsSensFadType & modelPar_WPCLM,
   bool modelPar_given_WPCLM,
   AdmsSensFadType & modelPar_PPCLM,
   bool modelPar_given_PPCLM,
   AdmsSensFadType & modelPar_PCLMR,
   bool modelPar_given_PCLMR,
   AdmsSensFadType & modelPar_LPCLMR,
   bool modelPar_given_LPCLMR,
   AdmsSensFadType & modelPar_WPCLMR,
   bool modelPar_given_WPCLMR,
   AdmsSensFadType & modelPar_PPCLMR,
   bool modelPar_given_PPCLMR,
   AdmsSensFadType & modelPar_PCLMG,
   bool modelPar_given_PCLMG,
   AdmsSensFadType & modelPar_PCLMCV,
   bool modelPar_given_PCLMCV,
   AdmsSensFadType & modelPar_PCLMCVL,
   bool modelPar_given_PCLMCVL,
   AdmsSensFadType & modelPar_PCLMCVLEXP,
   bool modelPar_given_PCLMCVLEXP,
   AdmsSensFadType & modelPar_LPCLMCV,
   bool modelPar_given_LPCLMCV,
   AdmsSensFadType & modelPar_WPCLMCV,
   bool modelPar_given_WPCLMCV,
   AdmsSensFadType & modelPar_PPCLMCV,
   bool modelPar_given_PPCLMCV,
   AdmsSensFadType & modelPar_PSCBE1,
   bool modelPar_given_PSCBE1,
   AdmsSensFadType & modelPar_LPSCBE1,
   bool modelPar_given_LPSCBE1,
   AdmsSensFadType & modelPar_WPSCBE1,
   bool modelPar_given_WPSCBE1,
   AdmsSensFadType & modelPar_PPSCBE1,
   bool modelPar_given_PPSCBE1,
   AdmsSensFadType & modelPar_PSCBE2,
   bool modelPar_given_PSCBE2,
   AdmsSensFadType & modelPar_LPSCBE2,
   bool modelPar_given_LPSCBE2,
   AdmsSensFadType & modelPar_WPSCBE2,
   bool modelPar_given_WPSCBE2,
   AdmsSensFadType & modelPar_PPSCBE2,
   bool modelPar_given_PPSCBE2,
   AdmsSensFadType & modelPar_PDITS,
   bool modelPar_given_PDITS,
   AdmsSensFadType & modelPar_LPDITS,
   bool modelPar_given_LPDITS,
   AdmsSensFadType & modelPar_WPDITS,
   bool modelPar_given_WPDITS,
   AdmsSensFadType & modelPar_PPDITS,
   bool modelPar_given_PPDITS,
   AdmsSensFadType & modelPar_PDITSL,
   bool modelPar_given_PDITSL,
   AdmsSensFadType & modelPar_PDITSD,
   bool modelPar_given_PDITSD,
   AdmsSensFadType & modelPar_LPDITSD,
   bool modelPar_given_LPDITSD,
   AdmsSensFadType & modelPar_WPDITSD,
   bool modelPar_given_WPDITSD,
   AdmsSensFadType & modelPar_PPDITSD,
   bool modelPar_given_PPDITSD,
   AdmsSensFadType & modelPar_RSH,
   bool modelPar_given_RSH,
   AdmsSensFadType & modelPar_PRWG,
   bool modelPar_given_PRWG,
   AdmsSensFadType & modelPar_LPRWG,
   bool modelPar_given_LPRWG,
   AdmsSensFadType & modelPar_WPRWG,
   bool modelPar_given_WPRWG,
   AdmsSensFadType & modelPar_PPRWG,
   bool modelPar_given_PPRWG,
   AdmsSensFadType & modelPar_PRWB,
   bool modelPar_given_PRWB,
   AdmsSensFadType & modelPar_LPRWB,
   bool modelPar_given_LPRWB,
   AdmsSensFadType & modelPar_WPRWB,
   bool modelPar_given_WPRWB,
   AdmsSensFadType & modelPar_PPRWB,
   bool modelPar_given_PPRWB,
   AdmsSensFadType & modelPar_PRWBL,
   bool modelPar_given_PRWBL,
   AdmsSensFadType & modelPar_PRWBLEXP,
   bool modelPar_given_PRWBLEXP,
   AdmsSensFadType & modelPar_WR,
   bool modelPar_given_WR,
   AdmsSensFadType & modelPar_LWR,
   bool modelPar_given_LWR,
   AdmsSensFadType & modelPar_WWR,
   bool modelPar_given_WWR,
   AdmsSensFadType & modelPar_PWR,
   bool modelPar_given_PWR,
   AdmsSensFadType & modelPar_RSWMIN,
   bool modelPar_given_RSWMIN,
   AdmsSensFadType & modelPar_LRSWMIN,
   bool modelPar_given_LRSWMIN,
   AdmsSensFadType & modelPar_WRSWMIN,
   bool modelPar_given_WRSWMIN,
   AdmsSensFadType & modelPar_PRSWMIN,
   bool modelPar_given_PRSWMIN,
   AdmsSensFadType & modelPar_RSW,
   bool modelPar_given_RSW,
   AdmsSensFadType & modelPar_LRSW,
   bool modelPar_given_LRSW,
   AdmsSensFadType & modelPar_WRSW,
   bool modelPar_given_WRSW,
   AdmsSensFadType & modelPar_PRSW,
   bool modelPar_given_PRSW,
   AdmsSensFadType & modelPar_RSWL,
   bool modelPar_given_RSWL,
   AdmsSensFadType & modelPar_RSWLEXP,
   bool modelPar_given_RSWLEXP,
   AdmsSensFadType & modelPar_RDWMIN,
   bool modelPar_given_RDWMIN,
   AdmsSensFadType & modelPar_LRDWMIN,
   bool modelPar_given_LRDWMIN,
   AdmsSensFadType & modelPar_WRDWMIN,
   bool modelPar_given_WRDWMIN,
   AdmsSensFadType & modelPar_PRDWMIN,
   bool modelPar_given_PRDWMIN,
   AdmsSensFadType & modelPar_RDW,
   bool modelPar_given_RDW,
   AdmsSensFadType & modelPar_LRDW,
   bool modelPar_given_LRDW,
   AdmsSensFadType & modelPar_WRDW,
   bool modelPar_given_WRDW,
   AdmsSensFadType & modelPar_PRDW,
   bool modelPar_given_PRDW,
   AdmsSensFadType & modelPar_RDWL,
   bool modelPar_given_RDWL,
   AdmsSensFadType & modelPar_RDWLEXP,
   bool modelPar_given_RDWLEXP,
   AdmsSensFadType & modelPar_RDSWMIN,
   bool modelPar_given_RDSWMIN,
   AdmsSensFadType & modelPar_LRDSWMIN,
   bool modelPar_given_LRDSWMIN,
   AdmsSensFadType & modelPar_WRDSWMIN,
   bool modelPar_given_WRDSWMIN,
   AdmsSensFadType & modelPar_PRDSWMIN,
   bool modelPar_given_PRDSWMIN,
   AdmsSensFadType & modelPar_RDSW,
   bool modelPar_given_RDSW,
   AdmsSensFadType & modelPar_RDSWL,
   bool modelPar_given_RDSWL,
   AdmsSensFadType & modelPar_RDSWLEXP,
   bool modelPar_given_RDSWLEXP,
   AdmsSensFadType & modelPar_LRDSW,
   bool modelPar_given_LRDSW,
   AdmsSensFadType & modelPar_WRDSW,
   bool modelPar_given_WRDSW,
   AdmsSensFadType & modelPar_PRDSW,
   bool modelPar_given_PRDSW,
   AdmsSensFadType & modelPar_PSAT,
   bool modelPar_given_PSAT,
   AdmsSensFadType & modelPar_LPSAT,
   bool modelPar_given_LPSAT,
   AdmsSensFadType & modelPar_WPSAT,
   bool modelPar_given_WPSAT,
   AdmsSensFadType & modelPar_PPSAT,
   bool modelPar_given_PPSAT,
   AdmsSensFadType & modelPar_PSATL,
   bool modelPar_given_PSATL,
   AdmsSensFadType & modelPar_PSATLEXP,
   bool modelPar_given_PSATLEXP,
   AdmsSensFadType & modelPar_PSATB,
   bool modelPar_given_PSATB,
   AdmsSensFadType & modelPar_PSATR,
   bool modelPar_given_PSATR,
   AdmsSensFadType & modelPar_LPSATR,
   bool modelPar_given_LPSATR,
   AdmsSensFadType & modelPar_WPSATR,
   bool modelPar_given_WPSATR,
   AdmsSensFadType & modelPar_PPSATR,
   bool modelPar_given_PPSATR,
   AdmsSensFadType & modelPar_LPSATB,
   bool modelPar_given_LPSATB,
   AdmsSensFadType & modelPar_WPSATB,
   bool modelPar_given_WPSATB,
   AdmsSensFadType & modelPar_PPSATB,
   bool modelPar_given_PPSATB,
   AdmsSensFadType & modelPar_PSATX,
   bool modelPar_given_PSATX,
   AdmsSensFadType & modelPar_PTWG,
   bool modelPar_given_PTWG,
   AdmsSensFadType & modelPar_LPTWG,
   bool modelPar_given_LPTWG,
   AdmsSensFadType & modelPar_WPTWG,
   bool modelPar_given_WPTWG,
   AdmsSensFadType & modelPar_PPTWG,
   bool modelPar_given_PPTWG,
   AdmsSensFadType & modelPar_PTWGL,
   bool modelPar_given_PTWGL,
   AdmsSensFadType & modelPar_PTWGLEXP,
   bool modelPar_given_PTWGLEXP,
   AdmsSensFadType & modelPar_PTWGR,
   bool modelPar_given_PTWGR,
   AdmsSensFadType & modelPar_LPTWGR,
   bool modelPar_given_LPTWGR,
   AdmsSensFadType & modelPar_WPTWGR,
   bool modelPar_given_WPTWGR,
   AdmsSensFadType & modelPar_PPTWGR,
   bool modelPar_given_PPTWGR,
   AdmsSensFadType & modelPar_PTWGLR,
   bool modelPar_given_PTWGLR,
   AdmsSensFadType & modelPar_PTWGLEXPR,
   bool modelPar_given_PTWGLEXPR,
   AdmsSensFadType & modelPar_A1,
   bool modelPar_given_A1,
   AdmsSensFadType & modelPar_LA1,
   bool modelPar_given_LA1,
   AdmsSensFadType & modelPar_WA1,
   bool modelPar_given_WA1,
   AdmsSensFadType & modelPar_PA1,
   bool modelPar_given_PA1,
   AdmsSensFadType & modelPar_A11,
   bool modelPar_given_A11,
   AdmsSensFadType & modelPar_LA11,
   bool modelPar_given_LA11,
   AdmsSensFadType & modelPar_WA11,
   bool modelPar_given_WA11,
   AdmsSensFadType & modelPar_PA11,
   bool modelPar_given_PA11,
   AdmsSensFadType & modelPar_A2,
   bool modelPar_given_A2,
   AdmsSensFadType & modelPar_LA2,
   bool modelPar_given_LA2,
   AdmsSensFadType & modelPar_WA2,
   bool modelPar_given_WA2,
   AdmsSensFadType & modelPar_PA2,
   bool modelPar_given_PA2,
   AdmsSensFadType & modelPar_A21,
   bool modelPar_given_A21,
   AdmsSensFadType & modelPar_LA21,
   bool modelPar_given_LA21,
   AdmsSensFadType & modelPar_WA21,
   bool modelPar_given_WA21,
   AdmsSensFadType & modelPar_PA21,
   bool modelPar_given_PA21,
   AdmsSensFadType & modelPar_PDIBLC,
   bool modelPar_given_PDIBLC,
   AdmsSensFadType & modelPar_PDIBLCL,
   bool modelPar_given_PDIBLCL,
   AdmsSensFadType & modelPar_PDIBLCLEXP,
   bool modelPar_given_PDIBLCLEXP,
   AdmsSensFadType & modelPar_LPDIBLC,
   bool modelPar_given_LPDIBLC,
   AdmsSensFadType & modelPar_WPDIBLC,
   bool modelPar_given_WPDIBLC,
   AdmsSensFadType & modelPar_PPDIBLC,
   bool modelPar_given_PPDIBLC,
   AdmsSensFadType & modelPar_PDIBLCR,
   bool modelPar_given_PDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCLR,
   bool modelPar_given_PDIBLCLR,
   AdmsSensFadType & modelPar_PDIBLCLEXPR,
   bool modelPar_given_PDIBLCLEXPR,
   AdmsSensFadType & modelPar_LPDIBLCR,
   bool modelPar_given_LPDIBLCR,
   AdmsSensFadType & modelPar_WPDIBLCR,
   bool modelPar_given_WPDIBLCR,
   AdmsSensFadType & modelPar_PPDIBLCR,
   bool modelPar_given_PPDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCB,
   bool modelPar_given_PDIBLCB,
   AdmsSensFadType & modelPar_LPDIBLCB,
   bool modelPar_given_LPDIBLCB,
   AdmsSensFadType & modelPar_WPDIBLCB,
   bool modelPar_given_WPDIBLCB,
   AdmsSensFadType & modelPar_PPDIBLCB,
   bool modelPar_given_PPDIBLCB,
   AdmsSensFadType & modelPar_PVAG,
   bool modelPar_given_PVAG,
   AdmsSensFadType & modelPar_LPVAG,
   bool modelPar_given_LPVAG,
   AdmsSensFadType & modelPar_WPVAG,
   bool modelPar_given_WPVAG,
   AdmsSensFadType & modelPar_PPVAG,
   bool modelPar_given_PPVAG,
   AdmsSensFadType & modelPar_FPROUT,
   bool modelPar_given_FPROUT,
   AdmsSensFadType & modelPar_FPROUTL,
   bool modelPar_given_FPROUTL,
   AdmsSensFadType & modelPar_FPROUTLEXP,
   bool modelPar_given_FPROUTLEXP,
   AdmsSensFadType & modelPar_LFPROUT,
   bool modelPar_given_LFPROUT,
   AdmsSensFadType & modelPar_WFPROUT,
   bool modelPar_given_WFPROUT,
   AdmsSensFadType & modelPar_PFPROUT,
   bool modelPar_given_PFPROUT,
   AdmsSensFadType & modelPar_ALPHA0,
   bool modelPar_given_ALPHA0,
   AdmsSensFadType & modelPar_ALPHA0L,
   bool modelPar_given_ALPHA0L,
   AdmsSensFadType & modelPar_ALPHA0LEXP,
   bool modelPar_given_ALPHA0LEXP,
   AdmsSensFadType & modelPar_LALPHA0,
   bool modelPar_given_LALPHA0,
   AdmsSensFadType & modelPar_WALPHA0,
   bool modelPar_given_WALPHA0,
   AdmsSensFadType & modelPar_PALPHA0,
   bool modelPar_given_PALPHA0,
   AdmsSensFadType & modelPar_BETA0,
   bool modelPar_given_BETA0,
   AdmsSensFadType & modelPar_LBETA0,
   bool modelPar_given_LBETA0,
   AdmsSensFadType & modelPar_WBETA0,
   bool modelPar_given_WBETA0,
   AdmsSensFadType & modelPar_PBETA0,
   bool modelPar_given_PBETA0,
   AdmsSensFadType & modelPar_AIGBACC,
   bool modelPar_given_AIGBACC,
   AdmsSensFadType & modelPar_BIGBACC,
   bool modelPar_given_BIGBACC,
   AdmsSensFadType & modelPar_CIGBACC,
   bool modelPar_given_CIGBACC,
   AdmsSensFadType & modelPar_NIGBACC,
   bool modelPar_given_NIGBACC,
   AdmsSensFadType & modelPar_AIGBINV,
   bool modelPar_given_AIGBINV,
   AdmsSensFadType & modelPar_BIGBINV,
   bool modelPar_given_BIGBINV,
   AdmsSensFadType & modelPar_CIGBINV,
   bool modelPar_given_CIGBINV,
   AdmsSensFadType & modelPar_EIGBINV,
   bool modelPar_given_EIGBINV,
   AdmsSensFadType & modelPar_NIGBINV,
   bool modelPar_given_NIGBINV,
   AdmsSensFadType & modelPar_AIGC,
   bool modelPar_given_AIGC,
   AdmsSensFadType & modelPar_BIGC,
   bool modelPar_given_BIGC,
   AdmsSensFadType & modelPar_CIGC,
   bool modelPar_given_CIGC,
   AdmsSensFadType & modelPar_AIGS,
   bool modelPar_given_AIGS,
   AdmsSensFadType & modelPar_BIGS,
   bool modelPar_given_BIGS,
   AdmsSensFadType & modelPar_CIGS,
   bool modelPar_given_CIGS,
   AdmsSensFadType & modelPar_AIGD,
   bool modelPar_given_AIGD,
   AdmsSensFadType & modelPar_BIGD,
   bool modelPar_given_BIGD,
   AdmsSensFadType & modelPar_CIGD,
   bool modelPar_given_CIGD,
   AdmsSensFadType & modelPar_DLCIG,
   bool modelPar_given_DLCIG,
   AdmsSensFadType & modelPar_DLCIGD,
   bool modelPar_given_DLCIGD,
   AdmsSensFadType & modelPar_POXEDGE,
   bool modelPar_given_POXEDGE,
   AdmsSensFadType & modelPar_NTOX,
   bool modelPar_given_NTOX,
   AdmsSensFadType & modelPar_TOXREF,
   bool modelPar_given_TOXREF,
   AdmsSensFadType & modelPar_PIGCD,
   bool modelPar_given_PIGCD,
   AdmsSensFadType & modelPar_AIGCL,
   bool modelPar_given_AIGCL,
   AdmsSensFadType & modelPar_AIGCW,
   bool modelPar_given_AIGCW,
   AdmsSensFadType & modelPar_AIGSL,
   bool modelPar_given_AIGSL,
   AdmsSensFadType & modelPar_AIGSW,
   bool modelPar_given_AIGSW,
   AdmsSensFadType & modelPar_AIGDL,
   bool modelPar_given_AIGDL,
   AdmsSensFadType & modelPar_AIGDW,
   bool modelPar_given_AIGDW,
   AdmsSensFadType & modelPar_PIGCDL,
   bool modelPar_given_PIGCDL,
   AdmsSensFadType & modelPar_LAIGBINV,
   bool modelPar_given_LAIGBINV,
   AdmsSensFadType & modelPar_WAIGBINV,
   bool modelPar_given_WAIGBINV,
   AdmsSensFadType & modelPar_PAIGBINV,
   bool modelPar_given_PAIGBINV,
   AdmsSensFadType & modelPar_LBIGBINV,
   bool modelPar_given_LBIGBINV,
   AdmsSensFadType & modelPar_WBIGBINV,
   bool modelPar_given_WBIGBINV,
   AdmsSensFadType & modelPar_PBIGBINV,
   bool modelPar_given_PBIGBINV,
   AdmsSensFadType & modelPar_LCIGBINV,
   bool modelPar_given_LCIGBINV,
   AdmsSensFadType & modelPar_WCIGBINV,
   bool modelPar_given_WCIGBINV,
   AdmsSensFadType & modelPar_PCIGBINV,
   bool modelPar_given_PCIGBINV,
   AdmsSensFadType & modelPar_LEIGBINV,
   bool modelPar_given_LEIGBINV,
   AdmsSensFadType & modelPar_WEIGBINV,
   bool modelPar_given_WEIGBINV,
   AdmsSensFadType & modelPar_PEIGBINV,
   bool modelPar_given_PEIGBINV,
   AdmsSensFadType & modelPar_LNIGBINV,
   bool modelPar_given_LNIGBINV,
   AdmsSensFadType & modelPar_WNIGBINV,
   bool modelPar_given_WNIGBINV,
   AdmsSensFadType & modelPar_PNIGBINV,
   bool modelPar_given_PNIGBINV,
   AdmsSensFadType & modelPar_LAIGBACC,
   bool modelPar_given_LAIGBACC,
   AdmsSensFadType & modelPar_WAIGBACC,
   bool modelPar_given_WAIGBACC,
   AdmsSensFadType & modelPar_PAIGBACC,
   bool modelPar_given_PAIGBACC,
   AdmsSensFadType & modelPar_LBIGBACC,
   bool modelPar_given_LBIGBACC,
   AdmsSensFadType & modelPar_WBIGBACC,
   bool modelPar_given_WBIGBACC,
   AdmsSensFadType & modelPar_PBIGBACC,
   bool modelPar_given_PBIGBACC,
   AdmsSensFadType & modelPar_LCIGBACC,
   bool modelPar_given_LCIGBACC,
   AdmsSensFadType & modelPar_WCIGBACC,
   bool modelPar_given_WCIGBACC,
   AdmsSensFadType & modelPar_PCIGBACC,
   bool modelPar_given_PCIGBACC,
   AdmsSensFadType & modelPar_LNIGBACC,
   bool modelPar_given_LNIGBACC,
   AdmsSensFadType & modelPar_WNIGBACC,
   bool modelPar_given_WNIGBACC,
   AdmsSensFadType & modelPar_PNIGBACC,
   bool modelPar_given_PNIGBACC,
   AdmsSensFadType & modelPar_LAIGC,
   bool modelPar_given_LAIGC,
   AdmsSensFadType & modelPar_WAIGC,
   bool modelPar_given_WAIGC,
   AdmsSensFadType & modelPar_PAIGC,
   bool modelPar_given_PAIGC,
   AdmsSensFadType & modelPar_LBIGC,
   bool modelPar_given_LBIGC,
   AdmsSensFadType & modelPar_WBIGC,
   bool modelPar_given_WBIGC,
   AdmsSensFadType & modelPar_PBIGC,
   bool modelPar_given_PBIGC,
   AdmsSensFadType & modelPar_LCIGC,
   bool modelPar_given_LCIGC,
   AdmsSensFadType & modelPar_WCIGC,
   bool modelPar_given_WCIGC,
   AdmsSensFadType & modelPar_PCIGC,
   bool modelPar_given_PCIGC,
   AdmsSensFadType & modelPar_LAIGS,
   bool modelPar_given_LAIGS,
   AdmsSensFadType & modelPar_WAIGS,
   bool modelPar_given_WAIGS,
   AdmsSensFadType & modelPar_PAIGS,
   bool modelPar_given_PAIGS,
   AdmsSensFadType & modelPar_LBIGS,
   bool modelPar_given_LBIGS,
   AdmsSensFadType & modelPar_WBIGS,
   bool modelPar_given_WBIGS,
   AdmsSensFadType & modelPar_PBIGS,
   bool modelPar_given_PBIGS,
   AdmsSensFadType & modelPar_LCIGS,
   bool modelPar_given_LCIGS,
   AdmsSensFadType & modelPar_WCIGS,
   bool modelPar_given_WCIGS,
   AdmsSensFadType & modelPar_PCIGS,
   bool modelPar_given_PCIGS,
   AdmsSensFadType & modelPar_LAIGD,
   bool modelPar_given_LAIGD,
   AdmsSensFadType & modelPar_WAIGD,
   bool modelPar_given_WAIGD,
   AdmsSensFadType & modelPar_PAIGD,
   bool modelPar_given_PAIGD,
   AdmsSensFadType & modelPar_LBIGD,
   bool modelPar_given_LBIGD,
   AdmsSensFadType & modelPar_WBIGD,
   bool modelPar_given_WBIGD,
   AdmsSensFadType & modelPar_PBIGD,
   bool modelPar_given_PBIGD,
   AdmsSensFadType & modelPar_LCIGD,
   bool modelPar_given_LCIGD,
   AdmsSensFadType & modelPar_WCIGD,
   bool modelPar_given_WCIGD,
   AdmsSensFadType & modelPar_PCIGD,
   bool modelPar_given_PCIGD,
   AdmsSensFadType & modelPar_LPOXEDGE,
   bool modelPar_given_LPOXEDGE,
   AdmsSensFadType & modelPar_WPOXEDGE,
   bool modelPar_given_WPOXEDGE,
   AdmsSensFadType & modelPar_PPOXEDGE,
   bool modelPar_given_PPOXEDGE,
   AdmsSensFadType & modelPar_LDLCIG,
   bool modelPar_given_LDLCIG,
   AdmsSensFadType & modelPar_WDLCIG,
   bool modelPar_given_WDLCIG,
   AdmsSensFadType & modelPar_PDLCIG,
   bool modelPar_given_PDLCIG,
   AdmsSensFadType & modelPar_LDLCIGD,
   bool modelPar_given_LDLCIGD,
   AdmsSensFadType & modelPar_WDLCIGD,
   bool modelPar_given_WDLCIGD,
   AdmsSensFadType & modelPar_PDLCIGD,
   bool modelPar_given_PDLCIGD,
   AdmsSensFadType & modelPar_LNTOX,
   bool modelPar_given_LNTOX,
   AdmsSensFadType & modelPar_WNTOX,
   bool modelPar_given_WNTOX,
   AdmsSensFadType & modelPar_PNTOX,
   bool modelPar_given_PNTOX,
   AdmsSensFadType & modelPar_AGIDL,
   bool modelPar_given_AGIDL,
   AdmsSensFadType & modelPar_AGIDLL,
   bool modelPar_given_AGIDLL,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_LAGIDL,
   bool modelPar_given_LAGIDL,
   AdmsSensFadType & modelPar_WAGIDL,
   bool modelPar_given_WAGIDL,
   AdmsSensFadType & modelPar_PAGIDL,
   bool modelPar_given_PAGIDL,
   AdmsSensFadType & modelPar_BGIDL,
   bool modelPar_given_BGIDL,
   AdmsSensFadType & modelPar_LBGIDL,
   bool modelPar_given_LBGIDL,
   AdmsSensFadType & modelPar_WBGIDL,
   bool modelPar_given_WBGIDL,
   AdmsSensFadType & modelPar_PBGIDL,
   bool modelPar_given_PBGIDL,
   AdmsSensFadType & modelPar_CGIDL,
   bool modelPar_given_CGIDL,
   AdmsSensFadType & modelPar_LCGIDL,
   bool modelPar_given_LCGIDL,
   AdmsSensFadType & modelPar_WCGIDL,
   bool modelPar_given_WCGIDL,
   AdmsSensFadType & modelPar_PCGIDL,
   bool modelPar_given_PCGIDL,
   AdmsSensFadType & modelPar_EGIDL,
   bool modelPar_given_EGIDL,
   AdmsSensFadType & modelPar_LEGIDL,
   bool modelPar_given_LEGIDL,
   AdmsSensFadType & modelPar_WEGIDL,
   bool modelPar_given_WEGIDL,
   AdmsSensFadType & modelPar_PEGIDL,
   bool modelPar_given_PEGIDL,
   AdmsSensFadType & modelPar_AGISL,
   bool modelPar_given_AGISL,
   AdmsSensFadType & modelPar_AGISLL,
   bool modelPar_given_AGISLL,
   AdmsSensFadType & modelPar_AGISLW,
   bool modelPar_given_AGISLW,
   AdmsSensFadType & modelPar_LAGISL,
   bool modelPar_given_LAGISL,
   AdmsSensFadType & modelPar_WAGISL,
   bool modelPar_given_WAGISL,
   AdmsSensFadType & modelPar_PAGISL,
   bool modelPar_given_PAGISL,
   AdmsSensFadType & modelPar_BGISL,
   bool modelPar_given_BGISL,
   AdmsSensFadType & modelPar_LBGISL,
   bool modelPar_given_LBGISL,
   AdmsSensFadType & modelPar_WBGISL,
   bool modelPar_given_WBGISL,
   AdmsSensFadType & modelPar_PBGISL,
   bool modelPar_given_PBGISL,
   AdmsSensFadType & modelPar_CGISL,
   bool modelPar_given_CGISL,
   AdmsSensFadType & modelPar_LCGISL,
   bool modelPar_given_LCGISL,
   AdmsSensFadType & modelPar_WCGISL,
   bool modelPar_given_WCGISL,
   AdmsSensFadType & modelPar_PCGISL,
   bool modelPar_given_PCGISL,
   AdmsSensFadType & modelPar_EGISL,
   bool modelPar_given_EGISL,
   AdmsSensFadType & modelPar_LEGISL,
   bool modelPar_given_LEGISL,
   AdmsSensFadType & modelPar_WEGISL,
   bool modelPar_given_WEGISL,
   AdmsSensFadType & modelPar_PEGISL,
   bool modelPar_given_PEGISL,
   AdmsSensFadType & modelPar_CF,
   bool modelPar_given_CF,
   AdmsSensFadType & modelPar_LCF,
   bool modelPar_given_LCF,
   AdmsSensFadType & modelPar_WCF,
   bool modelPar_given_WCF,
   AdmsSensFadType & modelPar_PCF,
   bool modelPar_given_PCF,
   AdmsSensFadType & modelPar_CFRCOEFF,
   bool modelPar_given_CFRCOEFF,
   AdmsSensFadType & modelPar_CGSO,
   bool modelPar_given_CGSO,
   AdmsSensFadType & modelPar_CGDO,
   bool modelPar_given_CGDO,
   AdmsSensFadType & modelPar_CGBO,
   bool modelPar_given_CGBO,
   AdmsSensFadType & modelPar_CGSL,
   bool modelPar_given_CGSL,
   AdmsSensFadType & modelPar_LCGSL,
   bool modelPar_given_LCGSL,
   AdmsSensFadType & modelPar_WCGSL,
   bool modelPar_given_WCGSL,
   AdmsSensFadType & modelPar_PCGSL,
   bool modelPar_given_PCGSL,
   AdmsSensFadType & modelPar_CGDL,
   bool modelPar_given_CGDL,
   AdmsSensFadType & modelPar_LCGDL,
   bool modelPar_given_LCGDL,
   AdmsSensFadType & modelPar_WCGDL,
   bool modelPar_given_WCGDL,
   AdmsSensFadType & modelPar_PCGDL,
   bool modelPar_given_PCGDL,
   AdmsSensFadType & modelPar_CKAPPAS,
   bool modelPar_given_CKAPPAS,
   AdmsSensFadType & modelPar_LCKAPPAS,
   bool modelPar_given_LCKAPPAS,
   AdmsSensFadType & modelPar_WCKAPPAS,
   bool modelPar_given_WCKAPPAS,
   AdmsSensFadType & modelPar_PCKAPPAS,
   bool modelPar_given_PCKAPPAS,
   AdmsSensFadType & modelPar_CKAPPAD,
   bool modelPar_given_CKAPPAD,
   AdmsSensFadType & modelPar_LCKAPPAD,
   bool modelPar_given_LCKAPPAD,
   AdmsSensFadType & modelPar_WCKAPPAD,
   bool modelPar_given_WCKAPPAD,
   AdmsSensFadType & modelPar_PCKAPPAD,
   bool modelPar_given_PCKAPPAD,
   AdmsSensFadType & modelPar_DMCG,
   bool modelPar_given_DMCG,
   AdmsSensFadType & modelPar_DMCI,
   bool modelPar_given_DMCI,
   AdmsSensFadType & modelPar_DMDG,
   bool modelPar_given_DMDG,
   AdmsSensFadType & modelPar_DMCGT,
   bool modelPar_given_DMCGT,
   AdmsSensFadType & modelPar_XGL,
   bool modelPar_given_XGL,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_CJS,
   bool modelPar_given_CJS,
   AdmsSensFadType & modelPar_CJD,
   bool modelPar_given_CJD,
   AdmsSensFadType & modelPar_CJSWS,
   bool modelPar_given_CJSWS,
   AdmsSensFadType & modelPar_CJSWD,
   bool modelPar_given_CJSWD,
   AdmsSensFadType & modelPar_CJSWGS,
   bool modelPar_given_CJSWGS,
   AdmsSensFadType & modelPar_CJSWGD,
   bool modelPar_given_CJSWGD,
   AdmsSensFadType & modelPar_PBS,
   bool modelPar_given_PBS,
   AdmsSensFadType & modelPar_PBD,
   bool modelPar_given_PBD,
   AdmsSensFadType & modelPar_PBSWS,
   bool modelPar_given_PBSWS,
   AdmsSensFadType & modelPar_PBSWD,
   bool modelPar_given_PBSWD,
   AdmsSensFadType & modelPar_PBSWGS,
   bool modelPar_given_PBSWGS,
   AdmsSensFadType & modelPar_PBSWGD,
   bool modelPar_given_PBSWGD,
   AdmsSensFadType & modelPar_MJS,
   bool modelPar_given_MJS,
   AdmsSensFadType & modelPar_MJD,
   bool modelPar_given_MJD,
   AdmsSensFadType & modelPar_MJSWS,
   bool modelPar_given_MJSWS,
   AdmsSensFadType & modelPar_MJSWD,
   bool modelPar_given_MJSWD,
   AdmsSensFadType & modelPar_MJSWGS,
   bool modelPar_given_MJSWGS,
   AdmsSensFadType & modelPar_MJSWGD,
   bool modelPar_given_MJSWGD,
   AdmsSensFadType & modelPar_JSS,
   bool modelPar_given_JSS,
   AdmsSensFadType & modelPar_JSD,
   bool modelPar_given_JSD,
   AdmsSensFadType & modelPar_JSWS,
   bool modelPar_given_JSWS,
   AdmsSensFadType & modelPar_JSWD,
   bool modelPar_given_JSWD,
   AdmsSensFadType & modelPar_JSWGS,
   bool modelPar_given_JSWGS,
   AdmsSensFadType & modelPar_JSWGD,
   bool modelPar_given_JSWGD,
   AdmsSensFadType & modelPar_NJS,
   bool modelPar_given_NJS,
   AdmsSensFadType & modelPar_NJD,
   bool modelPar_given_NJD,
   AdmsSensFadType & modelPar_IJTHSFWD,
   bool modelPar_given_IJTHSFWD,
   AdmsSensFadType & modelPar_IJTHDFWD,
   bool modelPar_given_IJTHDFWD,
   AdmsSensFadType & modelPar_IJTHSREV,
   bool modelPar_given_IJTHSREV,
   AdmsSensFadType & modelPar_IJTHDREV,
   bool modelPar_given_IJTHDREV,
   AdmsSensFadType & modelPar_BVS,
   bool modelPar_given_BVS,
   AdmsSensFadType & modelPar_BVD,
   bool modelPar_given_BVD,
   AdmsSensFadType & modelPar_XJBVS,
   bool modelPar_given_XJBVS,
   AdmsSensFadType & modelPar_XJBVD,
   bool modelPar_given_XJBVD,
   AdmsSensFadType & modelPar_JTSS,
   bool modelPar_given_JTSS,
   AdmsSensFadType & modelPar_JTSD,
   bool modelPar_given_JTSD,
   AdmsSensFadType & modelPar_JTSSWS,
   bool modelPar_given_JTSSWS,
   AdmsSensFadType & modelPar_JTSSWD,
   bool modelPar_given_JTSSWD,
   AdmsSensFadType & modelPar_JTSSWGS,
   bool modelPar_given_JTSSWGS,
   AdmsSensFadType & modelPar_JTSSWGD,
   bool modelPar_given_JTSSWGD,
   AdmsSensFadType & modelPar_JTWEFF,
   bool modelPar_given_JTWEFF,
   AdmsSensFadType & modelPar_NJTS,
   bool modelPar_given_NJTS,
   AdmsSensFadType & modelPar_NJTSD,
   bool modelPar_given_NJTSD,
   AdmsSensFadType & modelPar_NJTSSW,
   bool modelPar_given_NJTSSW,
   AdmsSensFadType & modelPar_NJTSSWD,
   bool modelPar_given_NJTSSWD,
   AdmsSensFadType & modelPar_NJTSSWG,
   bool modelPar_given_NJTSSWG,
   AdmsSensFadType & modelPar_NJTSSWGD,
   bool modelPar_given_NJTSSWGD,
   AdmsSensFadType & modelPar_VTSS,
   bool modelPar_given_VTSS,
   AdmsSensFadType & modelPar_VTSD,
   bool modelPar_given_VTSD,
   AdmsSensFadType & modelPar_VTSSWS,
   bool modelPar_given_VTSSWS,
   AdmsSensFadType & modelPar_VTSSWD,
   bool modelPar_given_VTSSWD,
   AdmsSensFadType & modelPar_VTSSWGS,
   bool modelPar_given_VTSSWGS,
   AdmsSensFadType & modelPar_VTSSWGD,
   bool modelPar_given_VTSSWGD,
   AdmsSensFadType & modelPar_XRCRG1,
   bool modelPar_given_XRCRG1,
   AdmsSensFadType & modelPar_XRCRG2,
   bool modelPar_given_XRCRG2,
   AdmsSensFadType & modelPar_GBMIN,
   bool modelPar_given_GBMIN,
   AdmsSensFadType & modelPar_RBPS0,
   bool modelPar_given_RBPS0,
   AdmsSensFadType & modelPar_RBPSL,
   bool modelPar_given_RBPSL,
   AdmsSensFadType & modelPar_RBPSW,
   bool modelPar_given_RBPSW,
   AdmsSensFadType & modelPar_RBPSNF,
   bool modelPar_given_RBPSNF,
   AdmsSensFadType & modelPar_RBPD0,
   bool modelPar_given_RBPD0,
   AdmsSensFadType & modelPar_RBPDL,
   bool modelPar_given_RBPDL,
   AdmsSensFadType & modelPar_RBPDW,
   bool modelPar_given_RBPDW,
   AdmsSensFadType & modelPar_RBPDNF,
   bool modelPar_given_RBPDNF,
   AdmsSensFadType & modelPar_RBPBX0,
   bool modelPar_given_RBPBX0,
   AdmsSensFadType & modelPar_RBPBXL,
   bool modelPar_given_RBPBXL,
   AdmsSensFadType & modelPar_RBPBXW,
   bool modelPar_given_RBPBXW,
   AdmsSensFadType & modelPar_RBPBXNF,
   bool modelPar_given_RBPBXNF,
   AdmsSensFadType & modelPar_RBPBY0,
   bool modelPar_given_RBPBY0,
   AdmsSensFadType & modelPar_RBPBYL,
   bool modelPar_given_RBPBYL,
   AdmsSensFadType & modelPar_RBPBYW,
   bool modelPar_given_RBPBYW,
   AdmsSensFadType & modelPar_RBPBYNF,
   bool modelPar_given_RBPBYNF,
   AdmsSensFadType & modelPar_RBSBX0,
   bool modelPar_given_RBSBX0,
   AdmsSensFadType & modelPar_RBSBY0,
   bool modelPar_given_RBSBY0,
   AdmsSensFadType & modelPar_RBDBX0,
   bool modelPar_given_RBDBX0,
   AdmsSensFadType & modelPar_RBDBY0,
   bool modelPar_given_RBDBY0,
   AdmsSensFadType & modelPar_RBSDBXL,
   bool modelPar_given_RBSDBXL,
   AdmsSensFadType & modelPar_RBSDBXW,
   bool modelPar_given_RBSDBXW,
   AdmsSensFadType & modelPar_RBSDBXNF,
   bool modelPar_given_RBSDBXNF,
   AdmsSensFadType & modelPar_RBSDBYL,
   bool modelPar_given_RBSDBYL,
   AdmsSensFadType & modelPar_RBSDBYW,
   bool modelPar_given_RBSDBYW,
   AdmsSensFadType & modelPar_RBSDBYNF,
   bool modelPar_given_RBSDBYNF,
   AdmsSensFadType & modelPar_EF,
   bool modelPar_given_EF,
   AdmsSensFadType & modelPar_EM,
   bool modelPar_given_EM,
   AdmsSensFadType & modelPar_NOIA,
   bool modelPar_given_NOIA,
   AdmsSensFadType & modelPar_NOIB,
   bool modelPar_given_NOIB,
   AdmsSensFadType & modelPar_NOIC,
   bool modelPar_given_NOIC,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_NTNOI,
   bool modelPar_given_NTNOI,
   AdmsSensFadType & modelPar_RNOIA,
   bool modelPar_given_RNOIA,
   AdmsSensFadType & modelPar_RNOIB,
   bool modelPar_given_RNOIB,
   AdmsSensFadType & modelPar_RNOIC,
   bool modelPar_given_RNOIC,
   AdmsSensFadType & modelPar_TNOIA,
   bool modelPar_given_TNOIA,
   AdmsSensFadType & modelPar_TNOIB,
   bool modelPar_given_TNOIB,
   AdmsSensFadType & modelPar_TNOIC,
   bool modelPar_given_TNOIC,
   AdmsSensFadType & modelPar_DLBIN,
   bool modelPar_given_DLBIN,
   AdmsSensFadType & modelPar_DWBIN,
   bool modelPar_given_DWBIN,
   AdmsSensFadType & modelPar_TNOM,
   bool modelPar_given_TNOM,
   AdmsSensFadType & modelPar_DTEMP,
   bool modelPar_given_DTEMP,
   AdmsSensFadType & modelPar_TBGASUB,
   bool modelPar_given_TBGASUB,
   AdmsSensFadType & modelPar_TBGBSUB,
   bool modelPar_given_TBGBSUB,
   AdmsSensFadType & modelPar_TNFACTOR,
   bool modelPar_given_TNFACTOR,
   AdmsSensFadType & modelPar_UTE,
   bool modelPar_given_UTE,
   AdmsSensFadType & modelPar_LUTE,
   bool modelPar_given_LUTE,
   AdmsSensFadType & modelPar_WUTE,
   bool modelPar_given_WUTE,
   AdmsSensFadType & modelPar_PUTE,
   bool modelPar_given_PUTE,
   AdmsSensFadType & modelPar_UTEL,
   bool modelPar_given_UTEL,
   AdmsSensFadType & modelPar_UA1,
   bool modelPar_given_UA1,
   AdmsSensFadType & modelPar_LUA1,
   bool modelPar_given_LUA1,
   AdmsSensFadType & modelPar_WUA1,
   bool modelPar_given_WUA1,
   AdmsSensFadType & modelPar_PUA1,
   bool modelPar_given_PUA1,
   AdmsSensFadType & modelPar_UA1L,
   bool modelPar_given_UA1L,
   AdmsSensFadType & modelPar_UC1,
   bool modelPar_given_UC1,
   AdmsSensFadType & modelPar_LUC1,
   bool modelPar_given_LUC1,
   AdmsSensFadType & modelPar_WUC1,
   bool modelPar_given_WUC1,
   AdmsSensFadType & modelPar_PUC1,
   bool modelPar_given_PUC1,
   AdmsSensFadType & modelPar_UD1,
   bool modelPar_given_UD1,
   AdmsSensFadType & modelPar_LUD1,
   bool modelPar_given_LUD1,
   AdmsSensFadType & modelPar_WUD1,
   bool modelPar_given_WUD1,
   AdmsSensFadType & modelPar_PUD1,
   bool modelPar_given_PUD1,
   AdmsSensFadType & modelPar_UD1L,
   bool modelPar_given_UD1L,
   AdmsSensFadType & modelPar_UCSTE,
   bool modelPar_given_UCSTE,
   AdmsSensFadType & modelPar_LUCSTE,
   bool modelPar_given_LUCSTE,
   AdmsSensFadType & modelPar_WUCSTE,
   bool modelPar_given_WUCSTE,
   AdmsSensFadType & modelPar_PUCSTE,
   bool modelPar_given_PUCSTE,
   AdmsSensFadType & modelPar_TETA0,
   bool modelPar_given_TETA0,
   AdmsSensFadType & modelPar_PRT,
   bool modelPar_given_PRT,
   AdmsSensFadType & modelPar_LPRT,
   bool modelPar_given_LPRT,
   AdmsSensFadType & modelPar_WPRT,
   bool modelPar_given_WPRT,
   AdmsSensFadType & modelPar_PPRT,
   bool modelPar_given_PPRT,
   AdmsSensFadType & modelPar_AT,
   bool modelPar_given_AT,
   AdmsSensFadType & modelPar_LAT,
   bool modelPar_given_LAT,
   AdmsSensFadType & modelPar_WAT,
   bool modelPar_given_WAT,
   AdmsSensFadType & modelPar_PAT,
   bool modelPar_given_PAT,
   AdmsSensFadType & modelPar_ATL,
   bool modelPar_given_ATL,
   AdmsSensFadType & modelPar_TDELTA,
   bool modelPar_given_TDELTA,
   AdmsSensFadType & modelPar_PTWGT,
   bool modelPar_given_PTWGT,
   AdmsSensFadType & modelPar_LPTWGT,
   bool modelPar_given_LPTWGT,
   AdmsSensFadType & modelPar_WPTWGT,
   bool modelPar_given_WPTWGT,
   AdmsSensFadType & modelPar_PPTWGT,
   bool modelPar_given_PPTWGT,
   AdmsSensFadType & modelPar_PTWGTL,
   bool modelPar_given_PTWGTL,
   AdmsSensFadType & modelPar_KT1,
   bool modelPar_given_KT1,
   AdmsSensFadType & modelPar_KT1EXP,
   bool modelPar_given_KT1EXP,
   AdmsSensFadType & modelPar_KT1L,
   bool modelPar_given_KT1L,
   AdmsSensFadType & modelPar_LKT1,
   bool modelPar_given_LKT1,
   AdmsSensFadType & modelPar_WKT1,
   bool modelPar_given_WKT1,
   AdmsSensFadType & modelPar_PKT1,
   bool modelPar_given_PKT1,
   AdmsSensFadType & modelPar_KT2,
   bool modelPar_given_KT2,
   AdmsSensFadType & modelPar_LKT2,
   bool modelPar_given_LKT2,
   AdmsSensFadType & modelPar_WKT2,
   bool modelPar_given_WKT2,
   AdmsSensFadType & modelPar_PKT2,
   bool modelPar_given_PKT2,
   AdmsSensFadType & modelPar_IIT,
   bool modelPar_given_IIT,
   AdmsSensFadType & modelPar_LIIT,
   bool modelPar_given_LIIT,
   AdmsSensFadType & modelPar_WIIT,
   bool modelPar_given_WIIT,
   AdmsSensFadType & modelPar_PIIT,
   bool modelPar_given_PIIT,
   AdmsSensFadType & modelPar_IGT,
   bool modelPar_given_IGT,
   AdmsSensFadType & modelPar_LIGT,
   bool modelPar_given_LIGT,
   AdmsSensFadType & modelPar_WIGT,
   bool modelPar_given_WIGT,
   AdmsSensFadType & modelPar_PIGT,
   bool modelPar_given_PIGT,
   AdmsSensFadType & modelPar_TGIDL,
   bool modelPar_given_TGIDL,
   AdmsSensFadType & modelPar_LTGIDL,
   bool modelPar_given_LTGIDL,
   AdmsSensFadType & modelPar_WTGIDL,
   bool modelPar_given_WTGIDL,
   AdmsSensFadType & modelPar_PTGIDL,
   bool modelPar_given_PTGIDL,
   AdmsSensFadType & modelPar_TCJ,
   bool modelPar_given_TCJ,
   AdmsSensFadType & modelPar_TCJSW,
   bool modelPar_given_TCJSW,
   AdmsSensFadType & modelPar_TCJSWG,
   bool modelPar_given_TCJSWG,
   AdmsSensFadType & modelPar_TPB,
   bool modelPar_given_TPB,
   AdmsSensFadType & modelPar_TPBSW,
   bool modelPar_given_TPBSW,
   AdmsSensFadType & modelPar_TPBSWG,
   bool modelPar_given_TPBSWG,
   AdmsSensFadType & modelPar_XTIS,
   bool modelPar_given_XTIS,
   AdmsSensFadType & modelPar_XTID,
   bool modelPar_given_XTID,
   AdmsSensFadType & modelPar_XTSS,
   bool modelPar_given_XTSS,
   AdmsSensFadType & modelPar_XTSD,
   bool modelPar_given_XTSD,
   AdmsSensFadType & modelPar_XTSSWS,
   bool modelPar_given_XTSSWS,
   AdmsSensFadType & modelPar_XTSSWD,
   bool modelPar_given_XTSSWD,
   AdmsSensFadType & modelPar_XTSSWGS,
   bool modelPar_given_XTSSWGS,
   AdmsSensFadType & modelPar_XTSSWGD,
   bool modelPar_given_XTSSWGD,
   AdmsSensFadType & modelPar_TNJTS,
   bool modelPar_given_TNJTS,
   AdmsSensFadType & modelPar_TNJTSD,
   bool modelPar_given_TNJTSD,
   AdmsSensFadType & modelPar_TNJTSSW,
   bool modelPar_given_TNJTSSW,
   AdmsSensFadType & modelPar_TNJTSSWD,
   bool modelPar_given_TNJTSSWD,
   AdmsSensFadType & modelPar_TNJTSSWG,
   bool modelPar_given_TNJTSSWG,
   AdmsSensFadType & modelPar_TNJTSSWGD,
   bool modelPar_given_TNJTSSWGD,
   AdmsSensFadType & modelPar_RTH0,
   bool modelPar_given_RTH0,
   AdmsSensFadType & modelPar_CTH0,
   bool modelPar_given_CTH0,
   AdmsSensFadType & modelPar_WTH0,
   bool modelPar_given_WTH0,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KU0,
   bool modelPar_given_KU0,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKU0,
   bool modelPar_given_TKU0,
   AdmsSensFadType & modelPar_LKU0,
   bool modelPar_given_LKU0,
   AdmsSensFadType & modelPar_WKU0,
   bool modelPar_given_WKU0,
   AdmsSensFadType & modelPar_PKU0,
   bool modelPar_given_PKU0,
   AdmsSensFadType & modelPar_LLODKU0,
   bool modelPar_given_LLODKU0,
   AdmsSensFadType & modelPar_WLODKU0,
   bool modelPar_given_WLODKU0,
   AdmsSensFadType & modelPar_KVTH0,
   bool modelPar_given_KVTH0,
   AdmsSensFadType & modelPar_LKVTH0,
   bool modelPar_given_LKVTH0,
   AdmsSensFadType & modelPar_WKVTH0,
   bool modelPar_given_WKVTH0,
   AdmsSensFadType & modelPar_PKVTH0,
   bool modelPar_given_PKVTH0,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STK2,
   bool modelPar_given_STK2,
   AdmsSensFadType & modelPar_LODK2,
   bool modelPar_given_LODK2,
   AdmsSensFadType & modelPar_STETA0,
   bool modelPar_given_STETA0,
   AdmsSensFadType & modelPar_LODETA0,
   bool modelPar_given_LODETA0,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_KVTH0WE,
   bool modelPar_given_KVTH0WE,
   AdmsSensFadType & modelPar_LKVTH0WE,
   bool modelPar_given_LKVTH0WE,
   AdmsSensFadType & modelPar_WKVTH0WE,
   bool modelPar_given_WKVTH0WE,
   AdmsSensFadType & modelPar_PKVTH0WE,
   bool modelPar_given_PKVTH0WE,
   AdmsSensFadType & modelPar_K2WE,
   bool modelPar_given_K2WE,
   AdmsSensFadType & modelPar_LK2WE,
   bool modelPar_given_LK2WE,
   AdmsSensFadType & modelPar_WK2WE,
   bool modelPar_given_WK2WE,
   AdmsSensFadType & modelPar_PK2WE,
   bool modelPar_given_PK2WE,
   AdmsSensFadType & modelPar_KU0WE,
   bool modelPar_given_KU0WE,
   AdmsSensFadType & modelPar_LKU0WE,
   bool modelPar_given_LKU0WE,
   AdmsSensFadType & modelPar_WKU0WE,
   bool modelPar_given_WKU0WE,
   AdmsSensFadType & modelPar_PKU0WE,
   bool modelPar_given_PKU0WE,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEDGE,
   bool modelPar_given_WEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGE,
   bool modelPar_given_DGAMMAEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGEL,
   bool modelPar_given_DGAMMAEDGEL,
   AdmsSensFadType & modelPar_DGAMMAEDGELEXP,
   bool modelPar_given_DGAMMAEDGELEXP,
   AdmsSensFadType & modelPar_DVTEDGE,
   bool modelPar_given_DVTEDGE,
   AdmsSensFadType & modelPar_NFACTOREDGE,
   bool modelPar_given_NFACTOREDGE,
   AdmsSensFadType & modelPar_CITEDGE,
   bool modelPar_given_CITEDGE,
   AdmsSensFadType & modelPar_CDSCDEDGE,
   bool modelPar_given_CDSCDEDGE,
   AdmsSensFadType & modelPar_CDSCBEDGE,
   bool modelPar_given_CDSCBEDGE,
   AdmsSensFadType & modelPar_ETA0EDGE,
   bool modelPar_given_ETA0EDGE,
   AdmsSensFadType & modelPar_ETABEDGE,
   bool modelPar_given_ETABEDGE,
   AdmsSensFadType & modelPar_KT1EDGE,
   bool modelPar_given_KT1EDGE,
   AdmsSensFadType & modelPar_KT1LEDGE,
   bool modelPar_given_KT1LEDGE,
   AdmsSensFadType & modelPar_KT2EDGE,
   bool modelPar_given_KT2EDGE,
   AdmsSensFadType & modelPar_KT1EXPEDGE,
   bool modelPar_given_KT1EXPEDGE,
   AdmsSensFadType & modelPar_TNFACTOREDGE,
   bool modelPar_given_TNFACTOREDGE,
   AdmsSensFadType & modelPar_TETA0EDGE,
   bool modelPar_given_TETA0EDGE,
   AdmsSensFadType & modelPar_DVT0EDGE,
   bool modelPar_given_DVT0EDGE,
   AdmsSensFadType & modelPar_DVT1EDGE,
   bool modelPar_given_DVT1EDGE,
   AdmsSensFadType & modelPar_DVT2EDGE,
   bool modelPar_given_DVT2EDGE,
   // non-reals (including hidden)
   int modelPar_NGCON,
   bool modelPar_given_NGCON,
   int modelPar_RGATEMOD,
   bool modelPar_given_RGATEMOD,
   int modelPar_RBODYMOD,
   bool modelPar_given_RBODYMOD,
   int modelPar_GEOMOD,
   bool modelPar_given_GEOMOD,
   int modelPar_RGEOMOD,
   bool modelPar_given_RGEOMOD,
   int modelPar_SCA,
   bool modelPar_given_SCA,
   int modelPar_SCB,
   bool modelPar_given_SCB,
   int modelPar_SCC,
   bool modelPar_given_SCC,
   int modelPar_SC,
   bool modelPar_given_SC,
   int modelPar_TYPE,
   bool modelPar_given_TYPE,
   int modelPar_CVMOD,
   bool modelPar_given_CVMOD,
   int modelPar_COVMOD,
   bool modelPar_given_COVMOD,
   int modelPar_RDSMOD,
   bool modelPar_given_RDSMOD,
   int modelPar_WPEMOD,
   bool modelPar_given_WPEMOD,
   int modelPar_ASYMMOD,
   bool modelPar_given_ASYMMOD,
   int modelPar_GIDLMOD,
   bool modelPar_given_GIDLMOD,
   int modelPar_IGCMOD,
   bool modelPar_given_IGCMOD,
   int modelPar_IGBMOD,
   bool modelPar_given_IGBMOD,
   int modelPar_TNOIMOD,
   bool modelPar_given_TNOIMOD,
   int modelPar_SHMOD,
   bool modelPar_given_SHMOD,
   int modelPar_MOBSCALE,
   bool modelPar_given_MOBSCALE,
   int modelPar_PERMOD,
   bool modelPar_given_PERMOD,
   int modelPar_BINUNIT,
   bool modelPar_given_BINUNIT,
   int modelPar_EDGEFET,
   bool modelPar_given_EDGEFET,
   double admsTemperature,double adms_vt_nom, double ADMSgmin_arg, const Instance & theInstance);

void evaluateInitialModel(
   // model parameters
   // reals
   AdmsSensFadType & modelPar_L,
   bool modelPar_given_L,
   AdmsSensFadType & modelPar_VFBSDOFF,
   bool modelPar_given_VFBSDOFF,
   AdmsSensFadType & modelPar_XGW,
   bool modelPar_given_XGW,
   AdmsSensFadType & modelPar_RBPB,
   bool modelPar_given_RBPB,
   AdmsSensFadType & modelPar_RBPD,
   bool modelPar_given_RBPD,
   AdmsSensFadType & modelPar_RBPS,
   bool modelPar_given_RBPS,
   AdmsSensFadType & modelPar_RBDB,
   bool modelPar_given_RBDB,
   AdmsSensFadType & modelPar_RBSB,
   bool modelPar_given_RBSB,
   AdmsSensFadType & modelPar_SA,
   bool modelPar_given_SA,
   AdmsSensFadType & modelPar_SB,
   bool modelPar_given_SB,
   AdmsSensFadType & modelPar_SD,
   bool modelPar_given_SD,
   AdmsSensFadType & modelPar_LLONG,
   bool modelPar_given_LLONG,
   AdmsSensFadType & modelPar_LMLT,
   bool modelPar_given_LMLT,
   AdmsSensFadType & modelPar_WMLT,
   bool modelPar_given_WMLT,
   AdmsSensFadType & modelPar_XL,
   bool modelPar_given_XL,
   AdmsSensFadType & modelPar_WWIDE,
   bool modelPar_given_WWIDE,
   AdmsSensFadType & modelPar_XW,
   bool modelPar_given_XW,
   AdmsSensFadType & modelPar_LINT,
   bool modelPar_given_LINT,
   AdmsSensFadType & modelPar_LL,
   bool modelPar_given_LL,
   AdmsSensFadType & modelPar_LW,
   bool modelPar_given_LW,
   AdmsSensFadType & modelPar_LWL,
   bool modelPar_given_LWL,
   AdmsSensFadType & modelPar_LLN,
   bool modelPar_given_LLN,
   AdmsSensFadType & modelPar_LWN,
   bool modelPar_given_LWN,
   AdmsSensFadType & modelPar_WINT,
   bool modelPar_given_WINT,
   AdmsSensFadType & modelPar_WL,
   bool modelPar_given_WL,
   AdmsSensFadType & modelPar_WW,
   bool modelPar_given_WW,
   AdmsSensFadType & modelPar_WWL,
   bool modelPar_given_WWL,
   AdmsSensFadType & modelPar_WLN,
   bool modelPar_given_WLN,
   AdmsSensFadType & modelPar_WWN,
   bool modelPar_given_WWN,
   AdmsSensFadType & modelPar_DLC,
   bool modelPar_given_DLC,
   AdmsSensFadType & modelPar_LLC,
   bool modelPar_given_LLC,
   AdmsSensFadType & modelPar_LWC,
   bool modelPar_given_LWC,
   AdmsSensFadType & modelPar_LWLC,
   bool modelPar_given_LWLC,
   AdmsSensFadType & modelPar_DWC,
   bool modelPar_given_DWC,
   AdmsSensFadType & modelPar_WLC,
   bool modelPar_given_WLC,
   AdmsSensFadType & modelPar_WWC,
   bool modelPar_given_WWC,
   AdmsSensFadType & modelPar_WWLC,
   bool modelPar_given_WWLC,
   AdmsSensFadType & modelPar_TOXE,
   bool modelPar_given_TOXE,
   AdmsSensFadType & modelPar_TOXP,
   bool modelPar_given_TOXP,
   AdmsSensFadType & modelPar_DTOX,
   bool modelPar_given_DTOX,
   AdmsSensFadType & modelPar_NDEP,
   bool modelPar_given_NDEP,
   AdmsSensFadType & modelPar_NDEPL1,
   bool modelPar_given_NDEPL1,
   AdmsSensFadType & modelPar_NDEPLEXP1,
   bool modelPar_given_NDEPLEXP1,
   AdmsSensFadType & modelPar_NDEPL2,
   bool modelPar_given_NDEPL2,
   AdmsSensFadType & modelPar_NDEPLEXP2,
   bool modelPar_given_NDEPLEXP2,
   AdmsSensFadType & modelPar_NDEPW,
   bool modelPar_given_NDEPW,
   AdmsSensFadType & modelPar_NDEPWEXP,
   bool modelPar_given_NDEPWEXP,
   AdmsSensFadType & modelPar_NDEPWL,
   bool modelPar_given_NDEPWL,
   AdmsSensFadType & modelPar_NDEPWLEXP,
   bool modelPar_given_NDEPWLEXP,
   AdmsSensFadType & modelPar_LNDEP,
   bool modelPar_given_LNDEP,
   AdmsSensFadType & modelPar_WNDEP,
   bool modelPar_given_WNDEP,
   AdmsSensFadType & modelPar_PNDEP,
   bool modelPar_given_PNDEP,
   AdmsSensFadType & modelPar_NDEPCV,
   bool modelPar_given_NDEPCV,
   AdmsSensFadType & modelPar_NDEPCVL1,
   bool modelPar_given_NDEPCVL1,
   AdmsSensFadType & modelPar_NDEPCVLEXP1,
   bool modelPar_given_NDEPCVLEXP1,
   AdmsSensFadType & modelPar_NDEPCVL2,
   bool modelPar_given_NDEPCVL2,
   AdmsSensFadType & modelPar_NDEPCVLEXP2,
   bool modelPar_given_NDEPCVLEXP2,
   AdmsSensFadType & modelPar_NDEPCVW,
   bool modelPar_given_NDEPCVW,
   AdmsSensFadType & modelPar_NDEPCVWEXP,
   bool modelPar_given_NDEPCVWEXP,
   AdmsSensFadType & modelPar_NDEPCVWL,
   bool modelPar_given_NDEPCVWL,
   AdmsSensFadType & modelPar_NDEPCVWLEXP,
   bool modelPar_given_NDEPCVWLEXP,
   AdmsSensFadType & modelPar_LNDEPCV,
   bool modelPar_given_LNDEPCV,
   AdmsSensFadType & modelPar_WNDEPCV,
   bool modelPar_given_WNDEPCV,
   AdmsSensFadType & modelPar_PNDEPCV,
   bool modelPar_given_PNDEPCV,
   AdmsSensFadType & modelPar_NGATE,
   bool modelPar_given_NGATE,
   AdmsSensFadType & modelPar_LNGATE,
   bool modelPar_given_LNGATE,
   AdmsSensFadType & modelPar_WNGATE,
   bool modelPar_given_WNGATE,
   AdmsSensFadType & modelPar_PNGATE,
   bool modelPar_given_PNGATE,
   AdmsSensFadType & modelPar_EASUB,
   bool modelPar_given_EASUB,
   AdmsSensFadType & modelPar_NI0SUB,
   bool modelPar_given_NI0SUB,
   AdmsSensFadType & modelPar_BG0SUB,
   bool modelPar_given_BG0SUB,
   AdmsSensFadType & modelPar_EPSRSUB,
   bool modelPar_given_EPSRSUB,
   AdmsSensFadType & modelPar_EPSROX,
   bool modelPar_given_EPSROX,
   AdmsSensFadType & modelPar_XJ,
   bool modelPar_given_XJ,
   AdmsSensFadType & modelPar_LXJ,
   bool modelPar_given_LXJ,
   AdmsSensFadType & modelPar_WXJ,
   bool modelPar_given_WXJ,
   AdmsSensFadType & modelPar_PXJ,
   bool modelPar_given_PXJ,
   AdmsSensFadType & modelPar_LVFB,
   bool modelPar_given_LVFB,
   AdmsSensFadType & modelPar_WVFB,
   bool modelPar_given_WVFB,
   AdmsSensFadType & modelPar_PVFB,
   bool modelPar_given_PVFB,
   AdmsSensFadType & modelPar_VFB,
   bool modelPar_given_VFB,
   AdmsSensFadType & modelPar_VFBCV,
   bool modelPar_given_VFBCV,
   AdmsSensFadType & modelPar_LVFBCV,
   bool modelPar_given_LVFBCV,
   AdmsSensFadType & modelPar_WVFBCV,
   bool modelPar_given_WVFBCV,
   AdmsSensFadType & modelPar_PVFBCV,
   bool modelPar_given_PVFBCV,
   AdmsSensFadType & modelPar_VFBCVL,
   bool modelPar_given_VFBCVL,
   AdmsSensFadType & modelPar_VFBCVLEXP,
   bool modelPar_given_VFBCVLEXP,
   AdmsSensFadType & modelPar_VFBCVW,
   bool modelPar_given_VFBCVW,
   AdmsSensFadType & modelPar_VFBCVWEXP,
   bool modelPar_given_VFBCVWEXP,
   AdmsSensFadType & modelPar_VFBCVWL,
   bool modelPar_given_VFBCVWL,
   AdmsSensFadType & modelPar_VFBCVWLEXP,
   bool modelPar_given_VFBCVWLEXP,
   AdmsSensFadType & modelPar_DWJ,
   bool modelPar_given_DWJ,
   AdmsSensFadType & modelPar_NSD,
   bool modelPar_given_NSD,
   AdmsSensFadType & modelPar_LNSD,
   bool modelPar_given_LNSD,
   AdmsSensFadType & modelPar_WNSD,
   bool modelPar_given_WNSD,
   AdmsSensFadType & modelPar_PNSD,
   bool modelPar_given_PNSD,
   AdmsSensFadType & modelPar_DELVT0,
   bool modelPar_given_DELVT0,
   AdmsSensFadType & modelPar_DVTP0,
   bool modelPar_given_DVTP0,
   AdmsSensFadType & modelPar_LDVTP0,
   bool modelPar_given_LDVTP0,
   AdmsSensFadType & modelPar_WDVTP0,
   bool modelPar_given_WDVTP0,
   AdmsSensFadType & modelPar_PDVTP0,
   bool modelPar_given_PDVTP0,
   AdmsSensFadType & modelPar_DVTP1,
   bool modelPar_given_DVTP1,
   AdmsSensFadType & modelPar_LDVTP1,
   bool modelPar_given_LDVTP1,
   AdmsSensFadType & modelPar_WDVTP1,
   bool modelPar_given_WDVTP1,
   AdmsSensFadType & modelPar_PDVTP1,
   bool modelPar_given_PDVTP1,
   AdmsSensFadType & modelPar_DVTP2,
   bool modelPar_given_DVTP2,
   AdmsSensFadType & modelPar_LDVTP2,
   bool modelPar_given_LDVTP2,
   AdmsSensFadType & modelPar_WDVTP2,
   bool modelPar_given_WDVTP2,
   AdmsSensFadType & modelPar_PDVTP2,
   bool modelPar_given_PDVTP2,
   AdmsSensFadType & modelPar_DVTP3,
   bool modelPar_given_DVTP3,
   AdmsSensFadType & modelPar_LDVTP3,
   bool modelPar_given_LDVTP3,
   AdmsSensFadType & modelPar_WDVTP3,
   bool modelPar_given_WDVTP3,
   AdmsSensFadType & modelPar_PDVTP3,
   bool modelPar_given_PDVTP3,
   AdmsSensFadType & modelPar_DVTP4,
   bool modelPar_given_DVTP4,
   AdmsSensFadType & modelPar_LDVTP4,
   bool modelPar_given_LDVTP4,
   AdmsSensFadType & modelPar_WDVTP4,
   bool modelPar_given_WDVTP4,
   AdmsSensFadType & modelPar_PDVTP4,
   bool modelPar_given_PDVTP4,
   AdmsSensFadType & modelPar_DVTP5,
   bool modelPar_given_DVTP5,
   AdmsSensFadType & modelPar_LDVTP5,
   bool modelPar_given_LDVTP5,
   AdmsSensFadType & modelPar_WDVTP5,
   bool modelPar_given_WDVTP5,
   AdmsSensFadType & modelPar_PDVTP5,
   bool modelPar_given_PDVTP5,
   AdmsSensFadType & modelPar_PHIN,
   bool modelPar_given_PHIN,
   AdmsSensFadType & modelPar_LPHIN,
   bool modelPar_given_LPHIN,
   AdmsSensFadType & modelPar_WPHIN,
   bool modelPar_given_WPHIN,
   AdmsSensFadType & modelPar_PPHIN,
   bool modelPar_given_PPHIN,
   AdmsSensFadType & modelPar_ETA0,
   bool modelPar_given_ETA0,
   AdmsSensFadType & modelPar_LETA0,
   bool modelPar_given_LETA0,
   AdmsSensFadType & modelPar_WETA0,
   bool modelPar_given_WETA0,
   AdmsSensFadType & modelPar_PETA0,
   bool modelPar_given_PETA0,
   AdmsSensFadType & modelPar_ETA0R,
   bool modelPar_given_ETA0R,
   AdmsSensFadType & modelPar_LETA0R,
   bool modelPar_given_LETA0R,
   AdmsSensFadType & modelPar_WETA0R,
   bool modelPar_given_WETA0R,
   AdmsSensFadType & modelPar_PETA0R,
   bool modelPar_given_PETA0R,
   AdmsSensFadType & modelPar_DSUB,
   bool modelPar_given_DSUB,
   AdmsSensFadType & modelPar_ETAB,
   bool modelPar_given_ETAB,
   AdmsSensFadType & modelPar_ETABEXP,
   bool modelPar_given_ETABEXP,
   AdmsSensFadType & modelPar_LETAB,
   bool modelPar_given_LETAB,
   AdmsSensFadType & modelPar_WETAB,
   bool modelPar_given_WETAB,
   AdmsSensFadType & modelPar_PETAB,
   bool modelPar_given_PETAB,
   AdmsSensFadType & modelPar_K1,
   bool modelPar_given_K1,
   AdmsSensFadType & modelPar_K1L,
   bool modelPar_given_K1L,
   AdmsSensFadType & modelPar_K1LEXP,
   bool modelPar_given_K1LEXP,
   AdmsSensFadType & modelPar_K1W,
   bool modelPar_given_K1W,
   AdmsSensFadType & modelPar_K1WEXP,
   bool modelPar_given_K1WEXP,
   AdmsSensFadType & modelPar_K1WL,
   bool modelPar_given_K1WL,
   AdmsSensFadType & modelPar_K1WLEXP,
   bool modelPar_given_K1WLEXP,
   AdmsSensFadType & modelPar_LK1,
   bool modelPar_given_LK1,
   AdmsSensFadType & modelPar_WK1,
   bool modelPar_given_WK1,
   AdmsSensFadType & modelPar_PK1,
   bool modelPar_given_PK1,
   AdmsSensFadType & modelPar_K2,
   bool modelPar_given_K2,
   AdmsSensFadType & modelPar_K2L,
   bool modelPar_given_K2L,
   AdmsSensFadType & modelPar_K2LEXP,
   bool modelPar_given_K2LEXP,
   AdmsSensFadType & modelPar_K2W,
   bool modelPar_given_K2W,
   AdmsSensFadType & modelPar_K2WEXP,
   bool modelPar_given_K2WEXP,
   AdmsSensFadType & modelPar_K2WL,
   bool modelPar_given_K2WL,
   AdmsSensFadType & modelPar_K2WLEXP,
   bool modelPar_given_K2WLEXP,
   AdmsSensFadType & modelPar_LK2,
   bool modelPar_given_LK2,
   AdmsSensFadType & modelPar_WK2,
   bool modelPar_given_WK2,
   AdmsSensFadType & modelPar_PK2,
   bool modelPar_given_PK2,
   AdmsSensFadType & modelPar_ADOS,
   bool modelPar_given_ADOS,
   AdmsSensFadType & modelPar_BDOS,
   bool modelPar_given_BDOS,
   AdmsSensFadType & modelPar_QM0,
   bool modelPar_given_QM0,
   AdmsSensFadType & modelPar_ETAQM,
   bool modelPar_given_ETAQM,
   AdmsSensFadType & modelPar_CIT,
   bool modelPar_given_CIT,
   AdmsSensFadType & modelPar_LCIT,
   bool modelPar_given_LCIT,
   AdmsSensFadType & modelPar_WCIT,
   bool modelPar_given_WCIT,
   AdmsSensFadType & modelPar_PCIT,
   bool modelPar_given_PCIT,
   AdmsSensFadType & modelPar_NFACTOR,
   bool modelPar_given_NFACTOR,
   AdmsSensFadType & modelPar_NFACTORL,
   bool modelPar_given_NFACTORL,
   AdmsSensFadType & modelPar_NFACTORLEXP,
   bool modelPar_given_NFACTORLEXP,
   AdmsSensFadType & modelPar_NFACTORW,
   bool modelPar_given_NFACTORW,
   AdmsSensFadType & modelPar_NFACTORWEXP,
   bool modelPar_given_NFACTORWEXP,
   AdmsSensFadType & modelPar_NFACTORWL,
   bool modelPar_given_NFACTORWL,
   AdmsSensFadType & modelPar_NFACTORWLEXP,
   bool modelPar_given_NFACTORWLEXP,
   AdmsSensFadType & modelPar_LNFACTOR,
   bool modelPar_given_LNFACTOR,
   AdmsSensFadType & modelPar_WNFACTOR,
   bool modelPar_given_WNFACTOR,
   AdmsSensFadType & modelPar_PNFACTOR,
   bool modelPar_given_PNFACTOR,
   AdmsSensFadType & modelPar_CDSCD,
   bool modelPar_given_CDSCD,
   AdmsSensFadType & modelPar_CDSCDL,
   bool modelPar_given_CDSCDL,
   AdmsSensFadType & modelPar_CDSCDLEXP,
   bool modelPar_given_CDSCDLEXP,
   AdmsSensFadType & modelPar_LCDSCD,
   bool modelPar_given_LCDSCD,
   AdmsSensFadType & modelPar_WCDSCD,
   bool modelPar_given_WCDSCD,
   AdmsSensFadType & modelPar_PCDSCD,
   bool modelPar_given_PCDSCD,
   AdmsSensFadType & modelPar_CDSCDR,
   bool modelPar_given_CDSCDR,
   AdmsSensFadType & modelPar_CDSCDLR,
   bool modelPar_given_CDSCDLR,
   AdmsSensFadType & modelPar_LCDSCDR,
   bool modelPar_given_LCDSCDR,
   AdmsSensFadType & modelPar_WCDSCDR,
   bool modelPar_given_WCDSCDR,
   AdmsSensFadType & modelPar_PCDSCDR,
   bool modelPar_given_PCDSCDR,
   AdmsSensFadType & modelPar_CDSCB,
   bool modelPar_given_CDSCB,
   AdmsSensFadType & modelPar_CDSCBL,
   bool modelPar_given_CDSCBL,
   AdmsSensFadType & modelPar_CDSCBLEXP,
   bool modelPar_given_CDSCBLEXP,
   AdmsSensFadType & modelPar_LCDSCB,
   bool modelPar_given_LCDSCB,
   AdmsSensFadType & modelPar_WCDSCB,
   bool modelPar_given_WCDSCB,
   AdmsSensFadType & modelPar_PCDSCB,
   bool modelPar_given_PCDSCB,
   AdmsSensFadType & modelPar_VSAT,
   bool modelPar_given_VSAT,
   AdmsSensFadType & modelPar_LVSAT,
   bool modelPar_given_LVSAT,
   AdmsSensFadType & modelPar_WVSAT,
   bool modelPar_given_WVSAT,
   AdmsSensFadType & modelPar_PVSAT,
   bool modelPar_given_PVSAT,
   AdmsSensFadType & modelPar_VSATL,
   bool modelPar_given_VSATL,
   AdmsSensFadType & modelPar_VSATLEXP,
   bool modelPar_given_VSATLEXP,
   AdmsSensFadType & modelPar_VSATW,
   bool modelPar_given_VSATW,
   AdmsSensFadType & modelPar_VSATWEXP,
   bool modelPar_given_VSATWEXP,
   AdmsSensFadType & modelPar_VSATWL,
   bool modelPar_given_VSATWL,
   AdmsSensFadType & modelPar_VSATWLEXP,
   bool modelPar_given_VSATWLEXP,
   AdmsSensFadType & modelPar_VSATR,
   bool modelPar_given_VSATR,
   AdmsSensFadType & modelPar_LVSATR,
   bool modelPar_given_LVSATR,
   AdmsSensFadType & modelPar_WVSATR,
   bool modelPar_given_WVSATR,
   AdmsSensFadType & modelPar_PVSATR,
   bool modelPar_given_PVSATR,
   AdmsSensFadType & modelPar_DELTA,
   bool modelPar_given_DELTA,
   AdmsSensFadType & modelPar_LDELTA,
   bool modelPar_given_LDELTA,
   AdmsSensFadType & modelPar_WDELTA,
   bool modelPar_given_WDELTA,
   AdmsSensFadType & modelPar_PDELTA,
   bool modelPar_given_PDELTA,
   AdmsSensFadType & modelPar_DELTAL,
   bool modelPar_given_DELTAL,
   AdmsSensFadType & modelPar_DELTALEXP,
   bool modelPar_given_DELTALEXP,
   AdmsSensFadType & modelPar_VSATCV,
   bool modelPar_given_VSATCV,
   AdmsSensFadType & modelPar_LVSATCV,
   bool modelPar_given_LVSATCV,
   AdmsSensFadType & modelPar_WVSATCV,
   bool modelPar_given_WVSATCV,
   AdmsSensFadType & modelPar_PVSATCV,
   bool modelPar_given_PVSATCV,
   AdmsSensFadType & modelPar_VSATCVL,
   bool modelPar_given_VSATCVL,
   AdmsSensFadType & modelPar_VSATCVLEXP,
   bool modelPar_given_VSATCVLEXP,
   AdmsSensFadType & modelPar_VSATCVW,
   bool modelPar_given_VSATCVW,
   AdmsSensFadType & modelPar_VSATCVWEXP,
   bool modelPar_given_VSATCVWEXP,
   AdmsSensFadType & modelPar_VSATCVWL,
   bool modelPar_given_VSATCVWL,
   AdmsSensFadType & modelPar_VSATCVWLEXP,
   bool modelPar_given_VSATCVWLEXP,
   AdmsSensFadType & modelPar_UP1,
   bool modelPar_given_UP1,
   AdmsSensFadType & modelPar_LP1,
   bool modelPar_given_LP1,
   AdmsSensFadType & modelPar_UP2,
   bool modelPar_given_UP2,
   AdmsSensFadType & modelPar_LP2,
   bool modelPar_given_LP2,
   AdmsSensFadType & modelPar_MULU0,
   bool modelPar_given_MULU0,
   AdmsSensFadType & modelPar_U0,
   bool modelPar_given_U0,
   AdmsSensFadType & modelPar_U0L,
   bool modelPar_given_U0L,
   AdmsSensFadType & modelPar_U0LEXP,
   bool modelPar_given_U0LEXP,
   AdmsSensFadType & modelPar_LU0,
   bool modelPar_given_LU0,
   AdmsSensFadType & modelPar_WU0,
   bool modelPar_given_WU0,
   AdmsSensFadType & modelPar_PU0,
   bool modelPar_given_PU0,
   AdmsSensFadType & modelPar_U0R,
   bool modelPar_given_U0R,
   AdmsSensFadType & modelPar_LU0R,
   bool modelPar_given_LU0R,
   AdmsSensFadType & modelPar_WU0R,
   bool modelPar_given_WU0R,
   AdmsSensFadType & modelPar_PU0R,
   bool modelPar_given_PU0R,
   AdmsSensFadType & modelPar_ETAMOB,
   bool modelPar_given_ETAMOB,
   AdmsSensFadType & modelPar_UA,
   bool modelPar_given_UA,
   AdmsSensFadType & modelPar_UAL,
   bool modelPar_given_UAL,
   AdmsSensFadType & modelPar_UALEXP,
   bool modelPar_given_UALEXP,
   AdmsSensFadType & modelPar_UAW,
   bool modelPar_given_UAW,
   AdmsSensFadType & modelPar_UAWEXP,
   bool modelPar_given_UAWEXP,
   AdmsSensFadType & modelPar_UAWL,
   bool modelPar_given_UAWL,
   AdmsSensFadType & modelPar_UAWLEXP,
   bool modelPar_given_UAWLEXP,
   AdmsSensFadType & modelPar_LUA,
   bool modelPar_given_LUA,
   AdmsSensFadType & modelPar_WUA,
   bool modelPar_given_WUA,
   AdmsSensFadType & modelPar_PUA,
   bool modelPar_given_PUA,
   AdmsSensFadType & modelPar_UAR,
   bool modelPar_given_UAR,
   AdmsSensFadType & modelPar_LUAR,
   bool modelPar_given_LUAR,
   AdmsSensFadType & modelPar_WUAR,
   bool modelPar_given_WUAR,
   AdmsSensFadType & modelPar_PUAR,
   bool modelPar_given_PUAR,
   AdmsSensFadType & modelPar_EU,
   bool modelPar_given_EU,
   AdmsSensFadType & modelPar_LEU,
   bool modelPar_given_LEU,
   AdmsSensFadType & modelPar_WEU,
   bool modelPar_given_WEU,
   AdmsSensFadType & modelPar_PEU,
   bool modelPar_given_PEU,
   AdmsSensFadType & modelPar_EUL,
   bool modelPar_given_EUL,
   AdmsSensFadType & modelPar_EULEXP,
   bool modelPar_given_EULEXP,
   AdmsSensFadType & modelPar_EUW,
   bool modelPar_given_EUW,
   AdmsSensFadType & modelPar_EUWEXP,
   bool modelPar_given_EUWEXP,
   AdmsSensFadType & modelPar_EUWL,
   bool modelPar_given_EUWL,
   AdmsSensFadType & modelPar_EUWLEXP,
   bool modelPar_given_EUWLEXP,
   AdmsSensFadType & modelPar_UD,
   bool modelPar_given_UD,
   AdmsSensFadType & modelPar_UDL,
   bool modelPar_given_UDL,
   AdmsSensFadType & modelPar_UDLEXP,
   bool modelPar_given_UDLEXP,
   AdmsSensFadType & modelPar_LUD,
   bool modelPar_given_LUD,
   AdmsSensFadType & modelPar_WUD,
   bool modelPar_given_WUD,
   AdmsSensFadType & modelPar_PUD,
   bool modelPar_given_PUD,
   AdmsSensFadType & modelPar_UDR,
   bool modelPar_given_UDR,
   AdmsSensFadType & modelPar_LUDR,
   bool modelPar_given_LUDR,
   AdmsSensFadType & modelPar_WUDR,
   bool modelPar_given_WUDR,
   AdmsSensFadType & modelPar_PUDR,
   bool modelPar_given_PUDR,
   AdmsSensFadType & modelPar_UCS,
   bool modelPar_given_UCS,
   AdmsSensFadType & modelPar_LUCS,
   bool modelPar_given_LUCS,
   AdmsSensFadType & modelPar_WUCS,
   bool modelPar_given_WUCS,
   AdmsSensFadType & modelPar_PUCS,
   bool modelPar_given_PUCS,
   AdmsSensFadType & modelPar_UCSR,
   bool modelPar_given_UCSR,
   AdmsSensFadType & modelPar_LUCSR,
   bool modelPar_given_LUCSR,
   AdmsSensFadType & modelPar_WUCSR,
   bool modelPar_given_WUCSR,
   AdmsSensFadType & modelPar_PUCSR,
   bool modelPar_given_PUCSR,
   AdmsSensFadType & modelPar_UC,
   bool modelPar_given_UC,
   AdmsSensFadType & modelPar_UCL,
   bool modelPar_given_UCL,
   AdmsSensFadType & modelPar_UCLEXP,
   bool modelPar_given_UCLEXP,
   AdmsSensFadType & modelPar_UCW,
   bool modelPar_given_UCW,
   AdmsSensFadType & modelPar_UCWEXP,
   bool modelPar_given_UCWEXP,
   AdmsSensFadType & modelPar_UCWL,
   bool modelPar_given_UCWL,
   AdmsSensFadType & modelPar_UCWLEXP,
   bool modelPar_given_UCWLEXP,
   AdmsSensFadType & modelPar_LUC,
   bool modelPar_given_LUC,
   AdmsSensFadType & modelPar_WUC,
   bool modelPar_given_WUC,
   AdmsSensFadType & modelPar_PUC,
   bool modelPar_given_PUC,
   AdmsSensFadType & modelPar_UCR,
   bool modelPar_given_UCR,
   AdmsSensFadType & modelPar_LUCR,
   bool modelPar_given_LUCR,
   AdmsSensFadType & modelPar_WUCR,
   bool modelPar_given_WUCR,
   AdmsSensFadType & modelPar_PUCR,
   bool modelPar_given_PUCR,
   AdmsSensFadType & modelPar_PCLM,
   bool modelPar_given_PCLM,
   AdmsSensFadType & modelPar_PCLML,
   bool modelPar_given_PCLML,
   AdmsSensFadType & modelPar_PCLMLEXP,
   bool modelPar_given_PCLMLEXP,
   AdmsSensFadType & modelPar_LPCLM,
   bool modelPar_given_LPCLM,
   AdmsSensFadType & modelPar_WPCLM,
   bool modelPar_given_WPCLM,
   AdmsSensFadType & modelPar_PPCLM,
   bool modelPar_given_PPCLM,
   AdmsSensFadType & modelPar_PCLMR,
   bool modelPar_given_PCLMR,
   AdmsSensFadType & modelPar_LPCLMR,
   bool modelPar_given_LPCLMR,
   AdmsSensFadType & modelPar_WPCLMR,
   bool modelPar_given_WPCLMR,
   AdmsSensFadType & modelPar_PPCLMR,
   bool modelPar_given_PPCLMR,
   AdmsSensFadType & modelPar_PCLMG,
   bool modelPar_given_PCLMG,
   AdmsSensFadType & modelPar_PCLMCV,
   bool modelPar_given_PCLMCV,
   AdmsSensFadType & modelPar_PCLMCVL,
   bool modelPar_given_PCLMCVL,
   AdmsSensFadType & modelPar_PCLMCVLEXP,
   bool modelPar_given_PCLMCVLEXP,
   AdmsSensFadType & modelPar_LPCLMCV,
   bool modelPar_given_LPCLMCV,
   AdmsSensFadType & modelPar_WPCLMCV,
   bool modelPar_given_WPCLMCV,
   AdmsSensFadType & modelPar_PPCLMCV,
   bool modelPar_given_PPCLMCV,
   AdmsSensFadType & modelPar_PSCBE1,
   bool modelPar_given_PSCBE1,
   AdmsSensFadType & modelPar_LPSCBE1,
   bool modelPar_given_LPSCBE1,
   AdmsSensFadType & modelPar_WPSCBE1,
   bool modelPar_given_WPSCBE1,
   AdmsSensFadType & modelPar_PPSCBE1,
   bool modelPar_given_PPSCBE1,
   AdmsSensFadType & modelPar_PSCBE2,
   bool modelPar_given_PSCBE2,
   AdmsSensFadType & modelPar_LPSCBE2,
   bool modelPar_given_LPSCBE2,
   AdmsSensFadType & modelPar_WPSCBE2,
   bool modelPar_given_WPSCBE2,
   AdmsSensFadType & modelPar_PPSCBE2,
   bool modelPar_given_PPSCBE2,
   AdmsSensFadType & modelPar_PDITS,
   bool modelPar_given_PDITS,
   AdmsSensFadType & modelPar_LPDITS,
   bool modelPar_given_LPDITS,
   AdmsSensFadType & modelPar_WPDITS,
   bool modelPar_given_WPDITS,
   AdmsSensFadType & modelPar_PPDITS,
   bool modelPar_given_PPDITS,
   AdmsSensFadType & modelPar_PDITSL,
   bool modelPar_given_PDITSL,
   AdmsSensFadType & modelPar_PDITSD,
   bool modelPar_given_PDITSD,
   AdmsSensFadType & modelPar_LPDITSD,
   bool modelPar_given_LPDITSD,
   AdmsSensFadType & modelPar_WPDITSD,
   bool modelPar_given_WPDITSD,
   AdmsSensFadType & modelPar_PPDITSD,
   bool modelPar_given_PPDITSD,
   AdmsSensFadType & modelPar_RSH,
   bool modelPar_given_RSH,
   AdmsSensFadType & modelPar_PRWG,
   bool modelPar_given_PRWG,
   AdmsSensFadType & modelPar_LPRWG,
   bool modelPar_given_LPRWG,
   AdmsSensFadType & modelPar_WPRWG,
   bool modelPar_given_WPRWG,
   AdmsSensFadType & modelPar_PPRWG,
   bool modelPar_given_PPRWG,
   AdmsSensFadType & modelPar_PRWB,
   bool modelPar_given_PRWB,
   AdmsSensFadType & modelPar_LPRWB,
   bool modelPar_given_LPRWB,
   AdmsSensFadType & modelPar_WPRWB,
   bool modelPar_given_WPRWB,
   AdmsSensFadType & modelPar_PPRWB,
   bool modelPar_given_PPRWB,
   AdmsSensFadType & modelPar_PRWBL,
   bool modelPar_given_PRWBL,
   AdmsSensFadType & modelPar_PRWBLEXP,
   bool modelPar_given_PRWBLEXP,
   AdmsSensFadType & modelPar_WR,
   bool modelPar_given_WR,
   AdmsSensFadType & modelPar_LWR,
   bool modelPar_given_LWR,
   AdmsSensFadType & modelPar_WWR,
   bool modelPar_given_WWR,
   AdmsSensFadType & modelPar_PWR,
   bool modelPar_given_PWR,
   AdmsSensFadType & modelPar_RSWMIN,
   bool modelPar_given_RSWMIN,
   AdmsSensFadType & modelPar_LRSWMIN,
   bool modelPar_given_LRSWMIN,
   AdmsSensFadType & modelPar_WRSWMIN,
   bool modelPar_given_WRSWMIN,
   AdmsSensFadType & modelPar_PRSWMIN,
   bool modelPar_given_PRSWMIN,
   AdmsSensFadType & modelPar_RSW,
   bool modelPar_given_RSW,
   AdmsSensFadType & modelPar_LRSW,
   bool modelPar_given_LRSW,
   AdmsSensFadType & modelPar_WRSW,
   bool modelPar_given_WRSW,
   AdmsSensFadType & modelPar_PRSW,
   bool modelPar_given_PRSW,
   AdmsSensFadType & modelPar_RSWL,
   bool modelPar_given_RSWL,
   AdmsSensFadType & modelPar_RSWLEXP,
   bool modelPar_given_RSWLEXP,
   AdmsSensFadType & modelPar_RDWMIN,
   bool modelPar_given_RDWMIN,
   AdmsSensFadType & modelPar_LRDWMIN,
   bool modelPar_given_LRDWMIN,
   AdmsSensFadType & modelPar_WRDWMIN,
   bool modelPar_given_WRDWMIN,
   AdmsSensFadType & modelPar_PRDWMIN,
   bool modelPar_given_PRDWMIN,
   AdmsSensFadType & modelPar_RDW,
   bool modelPar_given_RDW,
   AdmsSensFadType & modelPar_LRDW,
   bool modelPar_given_LRDW,
   AdmsSensFadType & modelPar_WRDW,
   bool modelPar_given_WRDW,
   AdmsSensFadType & modelPar_PRDW,
   bool modelPar_given_PRDW,
   AdmsSensFadType & modelPar_RDWL,
   bool modelPar_given_RDWL,
   AdmsSensFadType & modelPar_RDWLEXP,
   bool modelPar_given_RDWLEXP,
   AdmsSensFadType & modelPar_RDSWMIN,
   bool modelPar_given_RDSWMIN,
   AdmsSensFadType & modelPar_LRDSWMIN,
   bool modelPar_given_LRDSWMIN,
   AdmsSensFadType & modelPar_WRDSWMIN,
   bool modelPar_given_WRDSWMIN,
   AdmsSensFadType & modelPar_PRDSWMIN,
   bool modelPar_given_PRDSWMIN,
   AdmsSensFadType & modelPar_RDSW,
   bool modelPar_given_RDSW,
   AdmsSensFadType & modelPar_RDSWL,
   bool modelPar_given_RDSWL,
   AdmsSensFadType & modelPar_RDSWLEXP,
   bool modelPar_given_RDSWLEXP,
   AdmsSensFadType & modelPar_LRDSW,
   bool modelPar_given_LRDSW,
   AdmsSensFadType & modelPar_WRDSW,
   bool modelPar_given_WRDSW,
   AdmsSensFadType & modelPar_PRDSW,
   bool modelPar_given_PRDSW,
   AdmsSensFadType & modelPar_PSAT,
   bool modelPar_given_PSAT,
   AdmsSensFadType & modelPar_LPSAT,
   bool modelPar_given_LPSAT,
   AdmsSensFadType & modelPar_WPSAT,
   bool modelPar_given_WPSAT,
   AdmsSensFadType & modelPar_PPSAT,
   bool modelPar_given_PPSAT,
   AdmsSensFadType & modelPar_PSATL,
   bool modelPar_given_PSATL,
   AdmsSensFadType & modelPar_PSATLEXP,
   bool modelPar_given_PSATLEXP,
   AdmsSensFadType & modelPar_PSATB,
   bool modelPar_given_PSATB,
   AdmsSensFadType & modelPar_PSATR,
   bool modelPar_given_PSATR,
   AdmsSensFadType & modelPar_LPSATR,
   bool modelPar_given_LPSATR,
   AdmsSensFadType & modelPar_WPSATR,
   bool modelPar_given_WPSATR,
   AdmsSensFadType & modelPar_PPSATR,
   bool modelPar_given_PPSATR,
   AdmsSensFadType & modelPar_LPSATB,
   bool modelPar_given_LPSATB,
   AdmsSensFadType & modelPar_WPSATB,
   bool modelPar_given_WPSATB,
   AdmsSensFadType & modelPar_PPSATB,
   bool modelPar_given_PPSATB,
   AdmsSensFadType & modelPar_PSATX,
   bool modelPar_given_PSATX,
   AdmsSensFadType & modelPar_PTWG,
   bool modelPar_given_PTWG,
   AdmsSensFadType & modelPar_LPTWG,
   bool modelPar_given_LPTWG,
   AdmsSensFadType & modelPar_WPTWG,
   bool modelPar_given_WPTWG,
   AdmsSensFadType & modelPar_PPTWG,
   bool modelPar_given_PPTWG,
   AdmsSensFadType & modelPar_PTWGL,
   bool modelPar_given_PTWGL,
   AdmsSensFadType & modelPar_PTWGLEXP,
   bool modelPar_given_PTWGLEXP,
   AdmsSensFadType & modelPar_PTWGR,
   bool modelPar_given_PTWGR,
   AdmsSensFadType & modelPar_LPTWGR,
   bool modelPar_given_LPTWGR,
   AdmsSensFadType & modelPar_WPTWGR,
   bool modelPar_given_WPTWGR,
   AdmsSensFadType & modelPar_PPTWGR,
   bool modelPar_given_PPTWGR,
   AdmsSensFadType & modelPar_PTWGLR,
   bool modelPar_given_PTWGLR,
   AdmsSensFadType & modelPar_PTWGLEXPR,
   bool modelPar_given_PTWGLEXPR,
   AdmsSensFadType & modelPar_A1,
   bool modelPar_given_A1,
   AdmsSensFadType & modelPar_LA1,
   bool modelPar_given_LA1,
   AdmsSensFadType & modelPar_WA1,
   bool modelPar_given_WA1,
   AdmsSensFadType & modelPar_PA1,
   bool modelPar_given_PA1,
   AdmsSensFadType & modelPar_A11,
   bool modelPar_given_A11,
   AdmsSensFadType & modelPar_LA11,
   bool modelPar_given_LA11,
   AdmsSensFadType & modelPar_WA11,
   bool modelPar_given_WA11,
   AdmsSensFadType & modelPar_PA11,
   bool modelPar_given_PA11,
   AdmsSensFadType & modelPar_A2,
   bool modelPar_given_A2,
   AdmsSensFadType & modelPar_LA2,
   bool modelPar_given_LA2,
   AdmsSensFadType & modelPar_WA2,
   bool modelPar_given_WA2,
   AdmsSensFadType & modelPar_PA2,
   bool modelPar_given_PA2,
   AdmsSensFadType & modelPar_A21,
   bool modelPar_given_A21,
   AdmsSensFadType & modelPar_LA21,
   bool modelPar_given_LA21,
   AdmsSensFadType & modelPar_WA21,
   bool modelPar_given_WA21,
   AdmsSensFadType & modelPar_PA21,
   bool modelPar_given_PA21,
   AdmsSensFadType & modelPar_PDIBLC,
   bool modelPar_given_PDIBLC,
   AdmsSensFadType & modelPar_PDIBLCL,
   bool modelPar_given_PDIBLCL,
   AdmsSensFadType & modelPar_PDIBLCLEXP,
   bool modelPar_given_PDIBLCLEXP,
   AdmsSensFadType & modelPar_LPDIBLC,
   bool modelPar_given_LPDIBLC,
   AdmsSensFadType & modelPar_WPDIBLC,
   bool modelPar_given_WPDIBLC,
   AdmsSensFadType & modelPar_PPDIBLC,
   bool modelPar_given_PPDIBLC,
   AdmsSensFadType & modelPar_PDIBLCR,
   bool modelPar_given_PDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCLR,
   bool modelPar_given_PDIBLCLR,
   AdmsSensFadType & modelPar_PDIBLCLEXPR,
   bool modelPar_given_PDIBLCLEXPR,
   AdmsSensFadType & modelPar_LPDIBLCR,
   bool modelPar_given_LPDIBLCR,
   AdmsSensFadType & modelPar_WPDIBLCR,
   bool modelPar_given_WPDIBLCR,
   AdmsSensFadType & modelPar_PPDIBLCR,
   bool modelPar_given_PPDIBLCR,
   AdmsSensFadType & modelPar_PDIBLCB,
   bool modelPar_given_PDIBLCB,
   AdmsSensFadType & modelPar_LPDIBLCB,
   bool modelPar_given_LPDIBLCB,
   AdmsSensFadType & modelPar_WPDIBLCB,
   bool modelPar_given_WPDIBLCB,
   AdmsSensFadType & modelPar_PPDIBLCB,
   bool modelPar_given_PPDIBLCB,
   AdmsSensFadType & modelPar_PVAG,
   bool modelPar_given_PVAG,
   AdmsSensFadType & modelPar_LPVAG,
   bool modelPar_given_LPVAG,
   AdmsSensFadType & modelPar_WPVAG,
   bool modelPar_given_WPVAG,
   AdmsSensFadType & modelPar_PPVAG,
   bool modelPar_given_PPVAG,
   AdmsSensFadType & modelPar_FPROUT,
   bool modelPar_given_FPROUT,
   AdmsSensFadType & modelPar_FPROUTL,
   bool modelPar_given_FPROUTL,
   AdmsSensFadType & modelPar_FPROUTLEXP,
   bool modelPar_given_FPROUTLEXP,
   AdmsSensFadType & modelPar_LFPROUT,
   bool modelPar_given_LFPROUT,
   AdmsSensFadType & modelPar_WFPROUT,
   bool modelPar_given_WFPROUT,
   AdmsSensFadType & modelPar_PFPROUT,
   bool modelPar_given_PFPROUT,
   AdmsSensFadType & modelPar_ALPHA0,
   bool modelPar_given_ALPHA0,
   AdmsSensFadType & modelPar_ALPHA0L,
   bool modelPar_given_ALPHA0L,
   AdmsSensFadType & modelPar_ALPHA0LEXP,
   bool modelPar_given_ALPHA0LEXP,
   AdmsSensFadType & modelPar_LALPHA0,
   bool modelPar_given_LALPHA0,
   AdmsSensFadType & modelPar_WALPHA0,
   bool modelPar_given_WALPHA0,
   AdmsSensFadType & modelPar_PALPHA0,
   bool modelPar_given_PALPHA0,
   AdmsSensFadType & modelPar_BETA0,
   bool modelPar_given_BETA0,
   AdmsSensFadType & modelPar_LBETA0,
   bool modelPar_given_LBETA0,
   AdmsSensFadType & modelPar_WBETA0,
   bool modelPar_given_WBETA0,
   AdmsSensFadType & modelPar_PBETA0,
   bool modelPar_given_PBETA0,
   AdmsSensFadType & modelPar_AIGBACC,
   bool modelPar_given_AIGBACC,
   AdmsSensFadType & modelPar_BIGBACC,
   bool modelPar_given_BIGBACC,
   AdmsSensFadType & modelPar_CIGBACC,
   bool modelPar_given_CIGBACC,
   AdmsSensFadType & modelPar_NIGBACC,
   bool modelPar_given_NIGBACC,
   AdmsSensFadType & modelPar_AIGBINV,
   bool modelPar_given_AIGBINV,
   AdmsSensFadType & modelPar_BIGBINV,
   bool modelPar_given_BIGBINV,
   AdmsSensFadType & modelPar_CIGBINV,
   bool modelPar_given_CIGBINV,
   AdmsSensFadType & modelPar_EIGBINV,
   bool modelPar_given_EIGBINV,
   AdmsSensFadType & modelPar_NIGBINV,
   bool modelPar_given_NIGBINV,
   AdmsSensFadType & modelPar_AIGC,
   bool modelPar_given_AIGC,
   AdmsSensFadType & modelPar_BIGC,
   bool modelPar_given_BIGC,
   AdmsSensFadType & modelPar_CIGC,
   bool modelPar_given_CIGC,
   AdmsSensFadType & modelPar_AIGS,
   bool modelPar_given_AIGS,
   AdmsSensFadType & modelPar_BIGS,
   bool modelPar_given_BIGS,
   AdmsSensFadType & modelPar_CIGS,
   bool modelPar_given_CIGS,
   AdmsSensFadType & modelPar_AIGD,
   bool modelPar_given_AIGD,
   AdmsSensFadType & modelPar_BIGD,
   bool modelPar_given_BIGD,
   AdmsSensFadType & modelPar_CIGD,
   bool modelPar_given_CIGD,
   AdmsSensFadType & modelPar_DLCIG,
   bool modelPar_given_DLCIG,
   AdmsSensFadType & modelPar_DLCIGD,
   bool modelPar_given_DLCIGD,
   AdmsSensFadType & modelPar_POXEDGE,
   bool modelPar_given_POXEDGE,
   AdmsSensFadType & modelPar_NTOX,
   bool modelPar_given_NTOX,
   AdmsSensFadType & modelPar_TOXREF,
   bool modelPar_given_TOXREF,
   AdmsSensFadType & modelPar_PIGCD,
   bool modelPar_given_PIGCD,
   AdmsSensFadType & modelPar_AIGCL,
   bool modelPar_given_AIGCL,
   AdmsSensFadType & modelPar_AIGCW,
   bool modelPar_given_AIGCW,
   AdmsSensFadType & modelPar_AIGSL,
   bool modelPar_given_AIGSL,
   AdmsSensFadType & modelPar_AIGSW,
   bool modelPar_given_AIGSW,
   AdmsSensFadType & modelPar_AIGDL,
   bool modelPar_given_AIGDL,
   AdmsSensFadType & modelPar_AIGDW,
   bool modelPar_given_AIGDW,
   AdmsSensFadType & modelPar_PIGCDL,
   bool modelPar_given_PIGCDL,
   AdmsSensFadType & modelPar_LAIGBINV,
   bool modelPar_given_LAIGBINV,
   AdmsSensFadType & modelPar_WAIGBINV,
   bool modelPar_given_WAIGBINV,
   AdmsSensFadType & modelPar_PAIGBINV,
   bool modelPar_given_PAIGBINV,
   AdmsSensFadType & modelPar_LBIGBINV,
   bool modelPar_given_LBIGBINV,
   AdmsSensFadType & modelPar_WBIGBINV,
   bool modelPar_given_WBIGBINV,
   AdmsSensFadType & modelPar_PBIGBINV,
   bool modelPar_given_PBIGBINV,
   AdmsSensFadType & modelPar_LCIGBINV,
   bool modelPar_given_LCIGBINV,
   AdmsSensFadType & modelPar_WCIGBINV,
   bool modelPar_given_WCIGBINV,
   AdmsSensFadType & modelPar_PCIGBINV,
   bool modelPar_given_PCIGBINV,
   AdmsSensFadType & modelPar_LEIGBINV,
   bool modelPar_given_LEIGBINV,
   AdmsSensFadType & modelPar_WEIGBINV,
   bool modelPar_given_WEIGBINV,
   AdmsSensFadType & modelPar_PEIGBINV,
   bool modelPar_given_PEIGBINV,
   AdmsSensFadType & modelPar_LNIGBINV,
   bool modelPar_given_LNIGBINV,
   AdmsSensFadType & modelPar_WNIGBINV,
   bool modelPar_given_WNIGBINV,
   AdmsSensFadType & modelPar_PNIGBINV,
   bool modelPar_given_PNIGBINV,
   AdmsSensFadType & modelPar_LAIGBACC,
   bool modelPar_given_LAIGBACC,
   AdmsSensFadType & modelPar_WAIGBACC,
   bool modelPar_given_WAIGBACC,
   AdmsSensFadType & modelPar_PAIGBACC,
   bool modelPar_given_PAIGBACC,
   AdmsSensFadType & modelPar_LBIGBACC,
   bool modelPar_given_LBIGBACC,
   AdmsSensFadType & modelPar_WBIGBACC,
   bool modelPar_given_WBIGBACC,
   AdmsSensFadType & modelPar_PBIGBACC,
   bool modelPar_given_PBIGBACC,
   AdmsSensFadType & modelPar_LCIGBACC,
   bool modelPar_given_LCIGBACC,
   AdmsSensFadType & modelPar_WCIGBACC,
   bool modelPar_given_WCIGBACC,
   AdmsSensFadType & modelPar_PCIGBACC,
   bool modelPar_given_PCIGBACC,
   AdmsSensFadType & modelPar_LNIGBACC,
   bool modelPar_given_LNIGBACC,
   AdmsSensFadType & modelPar_WNIGBACC,
   bool modelPar_given_WNIGBACC,
   AdmsSensFadType & modelPar_PNIGBACC,
   bool modelPar_given_PNIGBACC,
   AdmsSensFadType & modelPar_LAIGC,
   bool modelPar_given_LAIGC,
   AdmsSensFadType & modelPar_WAIGC,
   bool modelPar_given_WAIGC,
   AdmsSensFadType & modelPar_PAIGC,
   bool modelPar_given_PAIGC,
   AdmsSensFadType & modelPar_LBIGC,
   bool modelPar_given_LBIGC,
   AdmsSensFadType & modelPar_WBIGC,
   bool modelPar_given_WBIGC,
   AdmsSensFadType & modelPar_PBIGC,
   bool modelPar_given_PBIGC,
   AdmsSensFadType & modelPar_LCIGC,
   bool modelPar_given_LCIGC,
   AdmsSensFadType & modelPar_WCIGC,
   bool modelPar_given_WCIGC,
   AdmsSensFadType & modelPar_PCIGC,
   bool modelPar_given_PCIGC,
   AdmsSensFadType & modelPar_LAIGS,
   bool modelPar_given_LAIGS,
   AdmsSensFadType & modelPar_WAIGS,
   bool modelPar_given_WAIGS,
   AdmsSensFadType & modelPar_PAIGS,
   bool modelPar_given_PAIGS,
   AdmsSensFadType & modelPar_LBIGS,
   bool modelPar_given_LBIGS,
   AdmsSensFadType & modelPar_WBIGS,
   bool modelPar_given_WBIGS,
   AdmsSensFadType & modelPar_PBIGS,
   bool modelPar_given_PBIGS,
   AdmsSensFadType & modelPar_LCIGS,
   bool modelPar_given_LCIGS,
   AdmsSensFadType & modelPar_WCIGS,
   bool modelPar_given_WCIGS,
   AdmsSensFadType & modelPar_PCIGS,
   bool modelPar_given_PCIGS,
   AdmsSensFadType & modelPar_LAIGD,
   bool modelPar_given_LAIGD,
   AdmsSensFadType & modelPar_WAIGD,
   bool modelPar_given_WAIGD,
   AdmsSensFadType & modelPar_PAIGD,
   bool modelPar_given_PAIGD,
   AdmsSensFadType & modelPar_LBIGD,
   bool modelPar_given_LBIGD,
   AdmsSensFadType & modelPar_WBIGD,
   bool modelPar_given_WBIGD,
   AdmsSensFadType & modelPar_PBIGD,
   bool modelPar_given_PBIGD,
   AdmsSensFadType & modelPar_LCIGD,
   bool modelPar_given_LCIGD,
   AdmsSensFadType & modelPar_WCIGD,
   bool modelPar_given_WCIGD,
   AdmsSensFadType & modelPar_PCIGD,
   bool modelPar_given_PCIGD,
   AdmsSensFadType & modelPar_LPOXEDGE,
   bool modelPar_given_LPOXEDGE,
   AdmsSensFadType & modelPar_WPOXEDGE,
   bool modelPar_given_WPOXEDGE,
   AdmsSensFadType & modelPar_PPOXEDGE,
   bool modelPar_given_PPOXEDGE,
   AdmsSensFadType & modelPar_LDLCIG,
   bool modelPar_given_LDLCIG,
   AdmsSensFadType & modelPar_WDLCIG,
   bool modelPar_given_WDLCIG,
   AdmsSensFadType & modelPar_PDLCIG,
   bool modelPar_given_PDLCIG,
   AdmsSensFadType & modelPar_LDLCIGD,
   bool modelPar_given_LDLCIGD,
   AdmsSensFadType & modelPar_WDLCIGD,
   bool modelPar_given_WDLCIGD,
   AdmsSensFadType & modelPar_PDLCIGD,
   bool modelPar_given_PDLCIGD,
   AdmsSensFadType & modelPar_LNTOX,
   bool modelPar_given_LNTOX,
   AdmsSensFadType & modelPar_WNTOX,
   bool modelPar_given_WNTOX,
   AdmsSensFadType & modelPar_PNTOX,
   bool modelPar_given_PNTOX,
   AdmsSensFadType & modelPar_AGIDL,
   bool modelPar_given_AGIDL,
   AdmsSensFadType & modelPar_AGIDLL,
   bool modelPar_given_AGIDLL,
   AdmsSensFadType & modelPar_AGIDLW,
   bool modelPar_given_AGIDLW,
   AdmsSensFadType & modelPar_LAGIDL,
   bool modelPar_given_LAGIDL,
   AdmsSensFadType & modelPar_WAGIDL,
   bool modelPar_given_WAGIDL,
   AdmsSensFadType & modelPar_PAGIDL,
   bool modelPar_given_PAGIDL,
   AdmsSensFadType & modelPar_BGIDL,
   bool modelPar_given_BGIDL,
   AdmsSensFadType & modelPar_LBGIDL,
   bool modelPar_given_LBGIDL,
   AdmsSensFadType & modelPar_WBGIDL,
   bool modelPar_given_WBGIDL,
   AdmsSensFadType & modelPar_PBGIDL,
   bool modelPar_given_PBGIDL,
   AdmsSensFadType & modelPar_CGIDL,
   bool modelPar_given_CGIDL,
   AdmsSensFadType & modelPar_LCGIDL,
   bool modelPar_given_LCGIDL,
   AdmsSensFadType & modelPar_WCGIDL,
   bool modelPar_given_WCGIDL,
   AdmsSensFadType & modelPar_PCGIDL,
   bool modelPar_given_PCGIDL,
   AdmsSensFadType & modelPar_EGIDL,
   bool modelPar_given_EGIDL,
   AdmsSensFadType & modelPar_LEGIDL,
   bool modelPar_given_LEGIDL,
   AdmsSensFadType & modelPar_WEGIDL,
   bool modelPar_given_WEGIDL,
   AdmsSensFadType & modelPar_PEGIDL,
   bool modelPar_given_PEGIDL,
   AdmsSensFadType & modelPar_AGISL,
   bool modelPar_given_AGISL,
   AdmsSensFadType & modelPar_AGISLL,
   bool modelPar_given_AGISLL,
   AdmsSensFadType & modelPar_AGISLW,
   bool modelPar_given_AGISLW,
   AdmsSensFadType & modelPar_LAGISL,
   bool modelPar_given_LAGISL,
   AdmsSensFadType & modelPar_WAGISL,
   bool modelPar_given_WAGISL,
   AdmsSensFadType & modelPar_PAGISL,
   bool modelPar_given_PAGISL,
   AdmsSensFadType & modelPar_BGISL,
   bool modelPar_given_BGISL,
   AdmsSensFadType & modelPar_LBGISL,
   bool modelPar_given_LBGISL,
   AdmsSensFadType & modelPar_WBGISL,
   bool modelPar_given_WBGISL,
   AdmsSensFadType & modelPar_PBGISL,
   bool modelPar_given_PBGISL,
   AdmsSensFadType & modelPar_CGISL,
   bool modelPar_given_CGISL,
   AdmsSensFadType & modelPar_LCGISL,
   bool modelPar_given_LCGISL,
   AdmsSensFadType & modelPar_WCGISL,
   bool modelPar_given_WCGISL,
   AdmsSensFadType & modelPar_PCGISL,
   bool modelPar_given_PCGISL,
   AdmsSensFadType & modelPar_EGISL,
   bool modelPar_given_EGISL,
   AdmsSensFadType & modelPar_LEGISL,
   bool modelPar_given_LEGISL,
   AdmsSensFadType & modelPar_WEGISL,
   bool modelPar_given_WEGISL,
   AdmsSensFadType & modelPar_PEGISL,
   bool modelPar_given_PEGISL,
   AdmsSensFadType & modelPar_CF,
   bool modelPar_given_CF,
   AdmsSensFadType & modelPar_LCF,
   bool modelPar_given_LCF,
   AdmsSensFadType & modelPar_WCF,
   bool modelPar_given_WCF,
   AdmsSensFadType & modelPar_PCF,
   bool modelPar_given_PCF,
   AdmsSensFadType & modelPar_CFRCOEFF,
   bool modelPar_given_CFRCOEFF,
   AdmsSensFadType & modelPar_CGSO,
   bool modelPar_given_CGSO,
   AdmsSensFadType & modelPar_CGDO,
   bool modelPar_given_CGDO,
   AdmsSensFadType & modelPar_CGBO,
   bool modelPar_given_CGBO,
   AdmsSensFadType & modelPar_CGSL,
   bool modelPar_given_CGSL,
   AdmsSensFadType & modelPar_LCGSL,
   bool modelPar_given_LCGSL,
   AdmsSensFadType & modelPar_WCGSL,
   bool modelPar_given_WCGSL,
   AdmsSensFadType & modelPar_PCGSL,
   bool modelPar_given_PCGSL,
   AdmsSensFadType & modelPar_CGDL,
   bool modelPar_given_CGDL,
   AdmsSensFadType & modelPar_LCGDL,
   bool modelPar_given_LCGDL,
   AdmsSensFadType & modelPar_WCGDL,
   bool modelPar_given_WCGDL,
   AdmsSensFadType & modelPar_PCGDL,
   bool modelPar_given_PCGDL,
   AdmsSensFadType & modelPar_CKAPPAS,
   bool modelPar_given_CKAPPAS,
   AdmsSensFadType & modelPar_LCKAPPAS,
   bool modelPar_given_LCKAPPAS,
   AdmsSensFadType & modelPar_WCKAPPAS,
   bool modelPar_given_WCKAPPAS,
   AdmsSensFadType & modelPar_PCKAPPAS,
   bool modelPar_given_PCKAPPAS,
   AdmsSensFadType & modelPar_CKAPPAD,
   bool modelPar_given_CKAPPAD,
   AdmsSensFadType & modelPar_LCKAPPAD,
   bool modelPar_given_LCKAPPAD,
   AdmsSensFadType & modelPar_WCKAPPAD,
   bool modelPar_given_WCKAPPAD,
   AdmsSensFadType & modelPar_PCKAPPAD,
   bool modelPar_given_PCKAPPAD,
   AdmsSensFadType & modelPar_DMCG,
   bool modelPar_given_DMCG,
   AdmsSensFadType & modelPar_DMCI,
   bool modelPar_given_DMCI,
   AdmsSensFadType & modelPar_DMDG,
   bool modelPar_given_DMDG,
   AdmsSensFadType & modelPar_DMCGT,
   bool modelPar_given_DMCGT,
   AdmsSensFadType & modelPar_XGL,
   bool modelPar_given_XGL,
   AdmsSensFadType & modelPar_RSHG,
   bool modelPar_given_RSHG,
   AdmsSensFadType & modelPar_CJS,
   bool modelPar_given_CJS,
   AdmsSensFadType & modelPar_CJD,
   bool modelPar_given_CJD,
   AdmsSensFadType & modelPar_CJSWS,
   bool modelPar_given_CJSWS,
   AdmsSensFadType & modelPar_CJSWD,
   bool modelPar_given_CJSWD,
   AdmsSensFadType & modelPar_CJSWGS,
   bool modelPar_given_CJSWGS,
   AdmsSensFadType & modelPar_CJSWGD,
   bool modelPar_given_CJSWGD,
   AdmsSensFadType & modelPar_PBS,
   bool modelPar_given_PBS,
   AdmsSensFadType & modelPar_PBD,
   bool modelPar_given_PBD,
   AdmsSensFadType & modelPar_PBSWS,
   bool modelPar_given_PBSWS,
   AdmsSensFadType & modelPar_PBSWD,
   bool modelPar_given_PBSWD,
   AdmsSensFadType & modelPar_PBSWGS,
   bool modelPar_given_PBSWGS,
   AdmsSensFadType & modelPar_PBSWGD,
   bool modelPar_given_PBSWGD,
   AdmsSensFadType & modelPar_MJS,
   bool modelPar_given_MJS,
   AdmsSensFadType & modelPar_MJD,
   bool modelPar_given_MJD,
   AdmsSensFadType & modelPar_MJSWS,
   bool modelPar_given_MJSWS,
   AdmsSensFadType & modelPar_MJSWD,
   bool modelPar_given_MJSWD,
   AdmsSensFadType & modelPar_MJSWGS,
   bool modelPar_given_MJSWGS,
   AdmsSensFadType & modelPar_MJSWGD,
   bool modelPar_given_MJSWGD,
   AdmsSensFadType & modelPar_JSS,
   bool modelPar_given_JSS,
   AdmsSensFadType & modelPar_JSD,
   bool modelPar_given_JSD,
   AdmsSensFadType & modelPar_JSWS,
   bool modelPar_given_JSWS,
   AdmsSensFadType & modelPar_JSWD,
   bool modelPar_given_JSWD,
   AdmsSensFadType & modelPar_JSWGS,
   bool modelPar_given_JSWGS,
   AdmsSensFadType & modelPar_JSWGD,
   bool modelPar_given_JSWGD,
   AdmsSensFadType & modelPar_NJS,
   bool modelPar_given_NJS,
   AdmsSensFadType & modelPar_NJD,
   bool modelPar_given_NJD,
   AdmsSensFadType & modelPar_IJTHSFWD,
   bool modelPar_given_IJTHSFWD,
   AdmsSensFadType & modelPar_IJTHDFWD,
   bool modelPar_given_IJTHDFWD,
   AdmsSensFadType & modelPar_IJTHSREV,
   bool modelPar_given_IJTHSREV,
   AdmsSensFadType & modelPar_IJTHDREV,
   bool modelPar_given_IJTHDREV,
   AdmsSensFadType & modelPar_BVS,
   bool modelPar_given_BVS,
   AdmsSensFadType & modelPar_BVD,
   bool modelPar_given_BVD,
   AdmsSensFadType & modelPar_XJBVS,
   bool modelPar_given_XJBVS,
   AdmsSensFadType & modelPar_XJBVD,
   bool modelPar_given_XJBVD,
   AdmsSensFadType & modelPar_JTSS,
   bool modelPar_given_JTSS,
   AdmsSensFadType & modelPar_JTSD,
   bool modelPar_given_JTSD,
   AdmsSensFadType & modelPar_JTSSWS,
   bool modelPar_given_JTSSWS,
   AdmsSensFadType & modelPar_JTSSWD,
   bool modelPar_given_JTSSWD,
   AdmsSensFadType & modelPar_JTSSWGS,
   bool modelPar_given_JTSSWGS,
   AdmsSensFadType & modelPar_JTSSWGD,
   bool modelPar_given_JTSSWGD,
   AdmsSensFadType & modelPar_JTWEFF,
   bool modelPar_given_JTWEFF,
   AdmsSensFadType & modelPar_NJTS,
   bool modelPar_given_NJTS,
   AdmsSensFadType & modelPar_NJTSD,
   bool modelPar_given_NJTSD,
   AdmsSensFadType & modelPar_NJTSSW,
   bool modelPar_given_NJTSSW,
   AdmsSensFadType & modelPar_NJTSSWD,
   bool modelPar_given_NJTSSWD,
   AdmsSensFadType & modelPar_NJTSSWG,
   bool modelPar_given_NJTSSWG,
   AdmsSensFadType & modelPar_NJTSSWGD,
   bool modelPar_given_NJTSSWGD,
   AdmsSensFadType & modelPar_VTSS,
   bool modelPar_given_VTSS,
   AdmsSensFadType & modelPar_VTSD,
   bool modelPar_given_VTSD,
   AdmsSensFadType & modelPar_VTSSWS,
   bool modelPar_given_VTSSWS,
   AdmsSensFadType & modelPar_VTSSWD,
   bool modelPar_given_VTSSWD,
   AdmsSensFadType & modelPar_VTSSWGS,
   bool modelPar_given_VTSSWGS,
   AdmsSensFadType & modelPar_VTSSWGD,
   bool modelPar_given_VTSSWGD,
   AdmsSensFadType & modelPar_XRCRG1,
   bool modelPar_given_XRCRG1,
   AdmsSensFadType & modelPar_XRCRG2,
   bool modelPar_given_XRCRG2,
   AdmsSensFadType & modelPar_GBMIN,
   bool modelPar_given_GBMIN,
   AdmsSensFadType & modelPar_RBPS0,
   bool modelPar_given_RBPS0,
   AdmsSensFadType & modelPar_RBPSL,
   bool modelPar_given_RBPSL,
   AdmsSensFadType & modelPar_RBPSW,
   bool modelPar_given_RBPSW,
   AdmsSensFadType & modelPar_RBPSNF,
   bool modelPar_given_RBPSNF,
   AdmsSensFadType & modelPar_RBPD0,
   bool modelPar_given_RBPD0,
   AdmsSensFadType & modelPar_RBPDL,
   bool modelPar_given_RBPDL,
   AdmsSensFadType & modelPar_RBPDW,
   bool modelPar_given_RBPDW,
   AdmsSensFadType & modelPar_RBPDNF,
   bool modelPar_given_RBPDNF,
   AdmsSensFadType & modelPar_RBPBX0,
   bool modelPar_given_RBPBX0,
   AdmsSensFadType & modelPar_RBPBXL,
   bool modelPar_given_RBPBXL,
   AdmsSensFadType & modelPar_RBPBXW,
   bool modelPar_given_RBPBXW,
   AdmsSensFadType & modelPar_RBPBXNF,
   bool modelPar_given_RBPBXNF,
   AdmsSensFadType & modelPar_RBPBY0,
   bool modelPar_given_RBPBY0,
   AdmsSensFadType & modelPar_RBPBYL,
   bool modelPar_given_RBPBYL,
   AdmsSensFadType & modelPar_RBPBYW,
   bool modelPar_given_RBPBYW,
   AdmsSensFadType & modelPar_RBPBYNF,
   bool modelPar_given_RBPBYNF,
   AdmsSensFadType & modelPar_RBSBX0,
   bool modelPar_given_RBSBX0,
   AdmsSensFadType & modelPar_RBSBY0,
   bool modelPar_given_RBSBY0,
   AdmsSensFadType & modelPar_RBDBX0,
   bool modelPar_given_RBDBX0,
   AdmsSensFadType & modelPar_RBDBY0,
   bool modelPar_given_RBDBY0,
   AdmsSensFadType & modelPar_RBSDBXL,
   bool modelPar_given_RBSDBXL,
   AdmsSensFadType & modelPar_RBSDBXW,
   bool modelPar_given_RBSDBXW,
   AdmsSensFadType & modelPar_RBSDBXNF,
   bool modelPar_given_RBSDBXNF,
   AdmsSensFadType & modelPar_RBSDBYL,
   bool modelPar_given_RBSDBYL,
   AdmsSensFadType & modelPar_RBSDBYW,
   bool modelPar_given_RBSDBYW,
   AdmsSensFadType & modelPar_RBSDBYNF,
   bool modelPar_given_RBSDBYNF,
   AdmsSensFadType & modelPar_EF,
   bool modelPar_given_EF,
   AdmsSensFadType & modelPar_EM,
   bool modelPar_given_EM,
   AdmsSensFadType & modelPar_NOIA,
   bool modelPar_given_NOIA,
   AdmsSensFadType & modelPar_NOIB,
   bool modelPar_given_NOIB,
   AdmsSensFadType & modelPar_NOIC,
   bool modelPar_given_NOIC,
   AdmsSensFadType & modelPar_LINTNOI,
   bool modelPar_given_LINTNOI,
   AdmsSensFadType & modelPar_NTNOI,
   bool modelPar_given_NTNOI,
   AdmsSensFadType & modelPar_RNOIA,
   bool modelPar_given_RNOIA,
   AdmsSensFadType & modelPar_RNOIB,
   bool modelPar_given_RNOIB,
   AdmsSensFadType & modelPar_RNOIC,
   bool modelPar_given_RNOIC,
   AdmsSensFadType & modelPar_TNOIA,
   bool modelPar_given_TNOIA,
   AdmsSensFadType & modelPar_TNOIB,
   bool modelPar_given_TNOIB,
   AdmsSensFadType & modelPar_TNOIC,
   bool modelPar_given_TNOIC,
   AdmsSensFadType & modelPar_DLBIN,
   bool modelPar_given_DLBIN,
   AdmsSensFadType & modelPar_DWBIN,
   bool modelPar_given_DWBIN,
   AdmsSensFadType & modelPar_TNOM,
   bool modelPar_given_TNOM,
   AdmsSensFadType & modelPar_DTEMP,
   bool modelPar_given_DTEMP,
   AdmsSensFadType & modelPar_TBGASUB,
   bool modelPar_given_TBGASUB,
   AdmsSensFadType & modelPar_TBGBSUB,
   bool modelPar_given_TBGBSUB,
   AdmsSensFadType & modelPar_TNFACTOR,
   bool modelPar_given_TNFACTOR,
   AdmsSensFadType & modelPar_UTE,
   bool modelPar_given_UTE,
   AdmsSensFadType & modelPar_LUTE,
   bool modelPar_given_LUTE,
   AdmsSensFadType & modelPar_WUTE,
   bool modelPar_given_WUTE,
   AdmsSensFadType & modelPar_PUTE,
   bool modelPar_given_PUTE,
   AdmsSensFadType & modelPar_UTEL,
   bool modelPar_given_UTEL,
   AdmsSensFadType & modelPar_UA1,
   bool modelPar_given_UA1,
   AdmsSensFadType & modelPar_LUA1,
   bool modelPar_given_LUA1,
   AdmsSensFadType & modelPar_WUA1,
   bool modelPar_given_WUA1,
   AdmsSensFadType & modelPar_PUA1,
   bool modelPar_given_PUA1,
   AdmsSensFadType & modelPar_UA1L,
   bool modelPar_given_UA1L,
   AdmsSensFadType & modelPar_UC1,
   bool modelPar_given_UC1,
   AdmsSensFadType & modelPar_LUC1,
   bool modelPar_given_LUC1,
   AdmsSensFadType & modelPar_WUC1,
   bool modelPar_given_WUC1,
   AdmsSensFadType & modelPar_PUC1,
   bool modelPar_given_PUC1,
   AdmsSensFadType & modelPar_UD1,
   bool modelPar_given_UD1,
   AdmsSensFadType & modelPar_LUD1,
   bool modelPar_given_LUD1,
   AdmsSensFadType & modelPar_WUD1,
   bool modelPar_given_WUD1,
   AdmsSensFadType & modelPar_PUD1,
   bool modelPar_given_PUD1,
   AdmsSensFadType & modelPar_UD1L,
   bool modelPar_given_UD1L,
   AdmsSensFadType & modelPar_UCSTE,
   bool modelPar_given_UCSTE,
   AdmsSensFadType & modelPar_LUCSTE,
   bool modelPar_given_LUCSTE,
   AdmsSensFadType & modelPar_WUCSTE,
   bool modelPar_given_WUCSTE,
   AdmsSensFadType & modelPar_PUCSTE,
   bool modelPar_given_PUCSTE,
   AdmsSensFadType & modelPar_TETA0,
   bool modelPar_given_TETA0,
   AdmsSensFadType & modelPar_PRT,
   bool modelPar_given_PRT,
   AdmsSensFadType & modelPar_LPRT,
   bool modelPar_given_LPRT,
   AdmsSensFadType & modelPar_WPRT,
   bool modelPar_given_WPRT,
   AdmsSensFadType & modelPar_PPRT,
   bool modelPar_given_PPRT,
   AdmsSensFadType & modelPar_AT,
   bool modelPar_given_AT,
   AdmsSensFadType & modelPar_LAT,
   bool modelPar_given_LAT,
   AdmsSensFadType & modelPar_WAT,
   bool modelPar_given_WAT,
   AdmsSensFadType & modelPar_PAT,
   bool modelPar_given_PAT,
   AdmsSensFadType & modelPar_ATL,
   bool modelPar_given_ATL,
   AdmsSensFadType & modelPar_TDELTA,
   bool modelPar_given_TDELTA,
   AdmsSensFadType & modelPar_PTWGT,
   bool modelPar_given_PTWGT,
   AdmsSensFadType & modelPar_LPTWGT,
   bool modelPar_given_LPTWGT,
   AdmsSensFadType & modelPar_WPTWGT,
   bool modelPar_given_WPTWGT,
   AdmsSensFadType & modelPar_PPTWGT,
   bool modelPar_given_PPTWGT,
   AdmsSensFadType & modelPar_PTWGTL,
   bool modelPar_given_PTWGTL,
   AdmsSensFadType & modelPar_KT1,
   bool modelPar_given_KT1,
   AdmsSensFadType & modelPar_KT1EXP,
   bool modelPar_given_KT1EXP,
   AdmsSensFadType & modelPar_KT1L,
   bool modelPar_given_KT1L,
   AdmsSensFadType & modelPar_LKT1,
   bool modelPar_given_LKT1,
   AdmsSensFadType & modelPar_WKT1,
   bool modelPar_given_WKT1,
   AdmsSensFadType & modelPar_PKT1,
   bool modelPar_given_PKT1,
   AdmsSensFadType & modelPar_KT2,
   bool modelPar_given_KT2,
   AdmsSensFadType & modelPar_LKT2,
   bool modelPar_given_LKT2,
   AdmsSensFadType & modelPar_WKT2,
   bool modelPar_given_WKT2,
   AdmsSensFadType & modelPar_PKT2,
   bool modelPar_given_PKT2,
   AdmsSensFadType & modelPar_IIT,
   bool modelPar_given_IIT,
   AdmsSensFadType & modelPar_LIIT,
   bool modelPar_given_LIIT,
   AdmsSensFadType & modelPar_WIIT,
   bool modelPar_given_WIIT,
   AdmsSensFadType & modelPar_PIIT,
   bool modelPar_given_PIIT,
   AdmsSensFadType & modelPar_IGT,
   bool modelPar_given_IGT,
   AdmsSensFadType & modelPar_LIGT,
   bool modelPar_given_LIGT,
   AdmsSensFadType & modelPar_WIGT,
   bool modelPar_given_WIGT,
   AdmsSensFadType & modelPar_PIGT,
   bool modelPar_given_PIGT,
   AdmsSensFadType & modelPar_TGIDL,
   bool modelPar_given_TGIDL,
   AdmsSensFadType & modelPar_LTGIDL,
   bool modelPar_given_LTGIDL,
   AdmsSensFadType & modelPar_WTGIDL,
   bool modelPar_given_WTGIDL,
   AdmsSensFadType & modelPar_PTGIDL,
   bool modelPar_given_PTGIDL,
   AdmsSensFadType & modelPar_TCJ,
   bool modelPar_given_TCJ,
   AdmsSensFadType & modelPar_TCJSW,
   bool modelPar_given_TCJSW,
   AdmsSensFadType & modelPar_TCJSWG,
   bool modelPar_given_TCJSWG,
   AdmsSensFadType & modelPar_TPB,
   bool modelPar_given_TPB,
   AdmsSensFadType & modelPar_TPBSW,
   bool modelPar_given_TPBSW,
   AdmsSensFadType & modelPar_TPBSWG,
   bool modelPar_given_TPBSWG,
   AdmsSensFadType & modelPar_XTIS,
   bool modelPar_given_XTIS,
   AdmsSensFadType & modelPar_XTID,
   bool modelPar_given_XTID,
   AdmsSensFadType & modelPar_XTSS,
   bool modelPar_given_XTSS,
   AdmsSensFadType & modelPar_XTSD,
   bool modelPar_given_XTSD,
   AdmsSensFadType & modelPar_XTSSWS,
   bool modelPar_given_XTSSWS,
   AdmsSensFadType & modelPar_XTSSWD,
   bool modelPar_given_XTSSWD,
   AdmsSensFadType & modelPar_XTSSWGS,
   bool modelPar_given_XTSSWGS,
   AdmsSensFadType & modelPar_XTSSWGD,
   bool modelPar_given_XTSSWGD,
   AdmsSensFadType & modelPar_TNJTS,
   bool modelPar_given_TNJTS,
   AdmsSensFadType & modelPar_TNJTSD,
   bool modelPar_given_TNJTSD,
   AdmsSensFadType & modelPar_TNJTSSW,
   bool modelPar_given_TNJTSSW,
   AdmsSensFadType & modelPar_TNJTSSWD,
   bool modelPar_given_TNJTSSWD,
   AdmsSensFadType & modelPar_TNJTSSWG,
   bool modelPar_given_TNJTSSWG,
   AdmsSensFadType & modelPar_TNJTSSWGD,
   bool modelPar_given_TNJTSSWGD,
   AdmsSensFadType & modelPar_RTH0,
   bool modelPar_given_RTH0,
   AdmsSensFadType & modelPar_CTH0,
   bool modelPar_given_CTH0,
   AdmsSensFadType & modelPar_WTH0,
   bool modelPar_given_WTH0,
   AdmsSensFadType & modelPar_SAREF,
   bool modelPar_given_SAREF,
   AdmsSensFadType & modelPar_SBREF,
   bool modelPar_given_SBREF,
   AdmsSensFadType & modelPar_WLOD,
   bool modelPar_given_WLOD,
   AdmsSensFadType & modelPar_KU0,
   bool modelPar_given_KU0,
   AdmsSensFadType & modelPar_KVSAT,
   bool modelPar_given_KVSAT,
   AdmsSensFadType & modelPar_TKU0,
   bool modelPar_given_TKU0,
   AdmsSensFadType & modelPar_LKU0,
   bool modelPar_given_LKU0,
   AdmsSensFadType & modelPar_WKU0,
   bool modelPar_given_WKU0,
   AdmsSensFadType & modelPar_PKU0,
   bool modelPar_given_PKU0,
   AdmsSensFadType & modelPar_LLODKU0,
   bool modelPar_given_LLODKU0,
   AdmsSensFadType & modelPar_WLODKU0,
   bool modelPar_given_WLODKU0,
   AdmsSensFadType & modelPar_KVTH0,
   bool modelPar_given_KVTH0,
   AdmsSensFadType & modelPar_LKVTH0,
   bool modelPar_given_LKVTH0,
   AdmsSensFadType & modelPar_WKVTH0,
   bool modelPar_given_WKVTH0,
   AdmsSensFadType & modelPar_PKVTH0,
   bool modelPar_given_PKVTH0,
   AdmsSensFadType & modelPar_LLODVTH,
   bool modelPar_given_LLODVTH,
   AdmsSensFadType & modelPar_WLODVTH,
   bool modelPar_given_WLODVTH,
   AdmsSensFadType & modelPar_STK2,
   bool modelPar_given_STK2,
   AdmsSensFadType & modelPar_LODK2,
   bool modelPar_given_LODK2,
   AdmsSensFadType & modelPar_STETA0,
   bool modelPar_given_STETA0,
   AdmsSensFadType & modelPar_LODETA0,
   bool modelPar_given_LODETA0,
   AdmsSensFadType & modelPar_WEB,
   bool modelPar_given_WEB,
   AdmsSensFadType & modelPar_WEC,
   bool modelPar_given_WEC,
   AdmsSensFadType & modelPar_KVTH0WE,
   bool modelPar_given_KVTH0WE,
   AdmsSensFadType & modelPar_LKVTH0WE,
   bool modelPar_given_LKVTH0WE,
   AdmsSensFadType & modelPar_WKVTH0WE,
   bool modelPar_given_WKVTH0WE,
   AdmsSensFadType & modelPar_PKVTH0WE,
   bool modelPar_given_PKVTH0WE,
   AdmsSensFadType & modelPar_K2WE,
   bool modelPar_given_K2WE,
   AdmsSensFadType & modelPar_LK2WE,
   bool modelPar_given_LK2WE,
   AdmsSensFadType & modelPar_WK2WE,
   bool modelPar_given_WK2WE,
   AdmsSensFadType & modelPar_PK2WE,
   bool modelPar_given_PK2WE,
   AdmsSensFadType & modelPar_KU0WE,
   bool modelPar_given_KU0WE,
   AdmsSensFadType & modelPar_LKU0WE,
   bool modelPar_given_LKU0WE,
   AdmsSensFadType & modelPar_WKU0WE,
   bool modelPar_given_WKU0WE,
   AdmsSensFadType & modelPar_PKU0WE,
   bool modelPar_given_PKU0WE,
   AdmsSensFadType & modelPar_SCREF,
   bool modelPar_given_SCREF,
   AdmsSensFadType & modelPar_WEDGE,
   bool modelPar_given_WEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGE,
   bool modelPar_given_DGAMMAEDGE,
   AdmsSensFadType & modelPar_DGAMMAEDGEL,
   bool modelPar_given_DGAMMAEDGEL,
   AdmsSensFadType & modelPar_DGAMMAEDGELEXP,
   bool modelPar_given_DGAMMAEDGELEXP,
   AdmsSensFadType & modelPar_DVTEDGE,
   bool modelPar_given_DVTEDGE,
   AdmsSensFadType & modelPar_NFACTOREDGE,
   bool modelPar_given_NFACTOREDGE,
   AdmsSensFadType & modelPar_CITEDGE,
   bool modelPar_given_CITEDGE,
   AdmsSensFadType & modelPar_CDSCDEDGE,
   bool modelPar_given_CDSCDEDGE,
   AdmsSensFadType & modelPar_CDSCBEDGE,
   bool modelPar_given_CDSCBEDGE,
   AdmsSensFadType & modelPar_ETA0EDGE,
   bool modelPar_given_ETA0EDGE,
   AdmsSensFadType & modelPar_ETABEDGE,
   bool modelPar_given_ETABEDGE,
   AdmsSensFadType & modelPar_KT1EDGE,
   bool modelPar_given_KT1EDGE,
   AdmsSensFadType & modelPar_KT1LEDGE,
   bool modelPar_given_KT1LEDGE,
   AdmsSensFadType & modelPar_KT2EDGE,
   bool modelPar_given_KT2EDGE,
   AdmsSensFadType & modelPar_KT1EXPEDGE,
   bool modelPar_given_KT1EXPEDGE,
   AdmsSensFadType & modelPar_TNFACTOREDGE,
   bool modelPar_given_TNFACTOREDGE,
   AdmsSensFadType & modelPar_TETA0EDGE,
   bool modelPar_given_TETA0EDGE,
   AdmsSensFadType & modelPar_DVT0EDGE,
   bool modelPar_given_DVT0EDGE,
   AdmsSensFadType & modelPar_DVT1EDGE,
   bool modelPar_given_DVT1EDGE,
   AdmsSensFadType & modelPar_DVT2EDGE,
   bool modelPar_given_DVT2EDGE,
   // non-reals (including hidden)
   int modelPar_NGCON,
   bool modelPar_given_NGCON,
   int modelPar_RGATEMOD,
   bool modelPar_given_RGATEMOD,
   int modelPar_RBODYMOD,
   bool modelPar_given_RBODYMOD,
   int modelPar_GEOMOD,
   bool modelPar_given_GEOMOD,
   int modelPar_RGEOMOD,
   bool modelPar_given_RGEOMOD,
   int modelPar_SCA,
   bool modelPar_given_SCA,
   int modelPar_SCB,
   bool modelPar_given_SCB,
   int modelPar_SCC,
   bool modelPar_given_SCC,
   int modelPar_SC,
   bool modelPar_given_SC,
   int modelPar_TYPE,
   bool modelPar_given_TYPE,
   int modelPar_CVMOD,
   bool modelPar_given_CVMOD,
   int modelPar_COVMOD,
   bool modelPar_given_COVMOD,
   int modelPar_RDSMOD,
   bool modelPar_given_RDSMOD,
   int modelPar_WPEMOD,
   bool modelPar_given_WPEMOD,
   int modelPar_ASYMMOD,
   bool modelPar_given_ASYMMOD,
   int modelPar_GIDLMOD,
   bool modelPar_given_GIDLMOD,
   int modelPar_IGCMOD,
   bool modelPar_given_IGCMOD,
   int modelPar_IGBMOD,
   bool modelPar_given_IGBMOD,
   int modelPar_TNOIMOD,
   bool modelPar_given_TNOIMOD,
   int modelPar_SHMOD,
   bool modelPar_given_SHMOD,
   int modelPar_MOBSCALE,
   bool modelPar_given_MOBSCALE,
   int modelPar_PERMOD,
   bool modelPar_given_PERMOD,
   int modelPar_BINUNIT,
   bool modelPar_given_BINUNIT,
   int modelPar_EDGEFET,
   bool modelPar_given_EDGEFET,
   double admsTemperature, double ADMSgmin_arg, const Instance & theInstance);

#endif // Xyce_ADMS_SENSITIVITIES


// Limited exponential --- NOT what verilog LRM says, but what qucs,
// ng-spice, and zspice do.

template <typename T>
T limexp(const T &x)
{
  if ((x) < 80.0)
    return (exp(x));
  else
    return (exp(80.0)*(x-79.0));
}

// Maybe this will do as a substitute for a sacado-provided "floor" and "ceil"?
// It is wrong to try to propagate the Sacado derivatives to the return
// value, as the floor and ceil functions have zero derivative w.r.t. any
// of the variables in the argument, except when X is an integer at which
// point the derivative is undefined.
template <typename T>
double ADMSfloor(const T & x)
{
  return (floor(x.val()));
}

template <typename T>
double ADMSceil(const T & x)
{
  return (ceil(x.val()));
}


struct Traits: public DeviceTraits<Model, Instance, MOSFET1::Traits>
{
  static const char *name() {return "BSIM6";}
  static const char *deviceTypeName() {return "m level 77";}

  static int numNodes() {return 4;}
  static bool modelRequired() {return true;}
  static bool isLinearDevice() {return false;}

  static Device *factory(const Configuration &configuration, const FactoryBlock &factory_block);
  static void loadModelParameters(ParametricData<Model> &model_parameters);
  static void loadInstanceParameters(ParametricData<Instance> &instance_parameters);
};

//-----------------------------------------------------------------------------
// Class         : Instance

//
// Purpose       : This class represents a single instance  of the
//                 device.  It mainly contains indices and pointers into
//                 the matrix equation (see the resistor instance class for
//                 more details).
//
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class Instance : public DeviceInstance
{
  friend class ParametricData<Instance>;
  friend class Model;
#ifdef Xyce_ADMS_SENSITIVITIES
  friend class InstanceSensitivity;
  friend class ModelSensitivity;
#endif // Xyce_ADMS_SENSITIVITIES
  friend struct Traits;

public:
  Instance(
     const Configuration &       configuration,
     const InstanceBlock &       instance_block,
     Model &                     model,
     const FactoryBlock &        factory_block);

  ~Instance();

private:
  Instance(const Instance &);
  Instance &operator=(const Instance &);

public:
  void registerLIDs( const LocalIdVector & intLIDVecRef,
                     const LocalIdVector & extLIDVecRef );
  void registerStoreLIDs( const LocalIdVector & stoLIDVecRef );
  void setupPointers();

  void loadNodeSymbols(Util::SymbolTable &symbol_table) const;

  const JacobianStamp & jacobianStamp() const;
  void registerJacLIDs( const JacobianStamp & jacLIDVec );

  void registerBranchDataLIDs(const std::vector<int> & branchLIDVecRef);

  bool processParams();
  bool updateTemperature ( const double & temp = -999.0 );
  bool updateIntermediateVars ();
  bool updatePrimaryState ();
  bool updateSecondaryState ();

  // load functions, residual:
  bool loadDAEQVector ();
  bool loadDAEFVector ();

  // load functions, Jacobian:
  bool loadDAEdQdx ();
  bool loadDAEdFdx ();

  void collapseNodes();
  int getNumNoiseSources () const;  // 16
  void setupNoiseSources (Xyce::Analysis::NoiseData & noiseData);
  void getNoiseSources (Xyce::Analysis::NoiseData & noiseData);

private:

public:
  // iterator reference to the bsim6 model which owns this instance.
  // Getters and setters
  Model &getModel()
  {
    return model_;
  }

private:

  Model & model_;   //< Owning Model
  // Begin verilog Instance Variables
  //   Instance Parameters
  double L;
  double W;
  int NF;
  double NRS;
  double NRD;
  double VFBSDOFF;
  int MINZ;
  double XGW;
  int NGCON;
  int RGATEMOD;
  int RBODYMOD;
  int GEOMOD;
  int RGEOMOD;
  double RBPB;
  double RBPD;
  double RBPS;
  double RBDB;
  double RBSB;
  double SA;
  double SB;
  double SD;
  int SCA;
  int SCB;
  int SCC;
  int SC;
  double AS;
  double AD;
  double PS;
  double PD;
  //  Variables of global_instance scope
  AdmsFadType T0;
  AdmsFadType T1;
  AdmsFadType T2;
  double epssi;
  double epsox;
  double Weff;
  double Leff;
  double Wact;
  double Lact;
  double Weffcj;
  double Cox;
  double epsratio;
  double Lnew;
  double L_mult;
  double Wnew;
  double DVTP0_i;
  double DVTP1_i;
  double DVTP2_i;
  double DVTP3_i;
  double DVTP4_i;
  double DVTP5_i;
  double A1_i;
  double A11_i;
  double A2_i;
  double A21_i;
  double PSAT_i;
  double PSATR_i;
  int devsign;
  double BSIM6TOXP;
  double DMCGeff;
  double DMDGeff;
  double DMCIeff;
  double RSourceGeo;
  double RDrainGeo;
  double Grgeltd;
  double Grbsb;
  double Grbdb;
  double Grbpb;
  double Grbps;
  double Grbpd;
  double Aechvb;
  double Bechvb;
  double AechvbEdge;
  double BechvbEdge;
  double ToxRatio;
  AdmsFadType U0R_t;
  AdmsFadType UAR_t;
  AdmsFadType UCR_t;
  AdmsFadType UDR_t;
  AdmsFadType UCSR_t;
  AdmsFadType VSATR_t;
  AdmsFadType PTWGR_t;
  AdmsFadType ETA0R_t;
  double PSATB_i;
  double KT1_i;
  double KT2_i;
  double NGATE_i;
  double NSD_i;
  double NDEP_i;
  double VFB_i;
  double CIT_i;
  double CDSCD_i;
  double CDSCDR_i;
  double CDSCB_i;
  double NFACTOR_i;
  double PHIN_i;
  double ETA0_i;
  double ETA0R_i;
  double ETAB_i;
  AdmsFadType K2_i;
  double K1_i;
  double DELTA_i;
  double U0_i;
  double U0R_i;
  double VSAT_i;
  double VSATR_i;
  double VSATCV_i;
  double UA_i;
  double UAR_i;
  double EU_i;
  double UD_i;
  double UDR_i;
  AdmsFadType UD_a;
  double UCS_i;
  double UCSR_i;
  double UC_i;
  double UCR_i;
  double PDIBLC_i;
  double PDIBLCR_i;
  double PDIBLCB_i;
  double PSCBE1_i;
  double PSCBE2_i;
  double PDITS_i;
  double PDITSD_i;
  double FPROUT_i;
  double PRWG_i;
  double WR_i;
  double RDWMIN_i;
  double RSWMIN_i;
  double RDW_i;
  double RSW_i;
  double RDSW_i;
  double RDSWMIN_i;
  double PTWG_i;
  double PTWGR_i;
  double PVAG_i;
  double XJ_i;
  double PCLM_i;
  double PCLMR_i;
  double PCLMCV_i;
  double PRWB_i;
  double CF_i;
  double CGSL_i;
  double CGDL_i;
  double CKAPPAS_i;
  double CKAPPAD_i;
  double VFBCV_i;
  double NDEPCV_i;
  double ALPHA0_i;
  double BETA0_i;
  double AGIDL_i;
  double BGIDL_i;
  double CGIDL_i;
  double EGIDL_i;
  double AGISL_i;
  double BGISL_i;
  double CGISL_i;
  double EGISL_i;
  double UTE_i;
  double UA1_i;
  double UC1_i;
  double UD1_i;
  double UCSTE_i;
  double PRT_i;
  double AT_i;
  double PTWGT_i;
  double IIT_i;
  double TGIDL_i;
  double KVTH0WE_i;
  double K2WE_i;
  double KU0WE_i;
  double AIGBINV_i;
  double BIGBINV_i;
  double CIGBINV_i;
  double EIGBINV_i;
  double NIGBINV_i;
  double AIGBACC_i;
  double BIGBACC_i;
  double CIGBACC_i;
  double NIGBACC_i;
  double AIGC_i;
  double BIGC_i;
  double CIGC_i;
  double AIGS_i;
  double BIGS_i;
  double CIGS_i;
  double AIGD_i;
  double BIGD_i;
  double CIGD_i;
  double PIGCD_i;
  double DLCIG_i;
  double DLCIGD_i;
  double IGT_i;
  double nuIntD;
  double nuEndD;
  double nuIntS;
  double nuEndS;
  double gth;
  double cth;
  // end verilog Instance Variables=====
  // Nodal LID Variables
  int li_d;
  int li_g;
  int li_s;
  int li_b;
  int li_di;
  int li_si;
  int li_gi;
  int li_gm;
  int li_bi;
  int li_sbulk;
  int li_dbulk;
  int li_N1;
  int li_N2;
  int li_t;
  // end Nodal LID Variables
  // Branch LID Variables
  // end Branch LID Variables
  // Lead (branch) LID Variables
  int li_branch_id;
  int li_branch_ig;
  int li_branch_is;
  int li_branch_ib;
  // end Lead (branch) LID Variables
  // Jacobian  pointers
  double * f_di_Equ_t_Node_Ptr;
  double * f_si_Equ_t_Node_Ptr;
  double * f_di_Equ_si_Node_Ptr;
  double * f_di_Equ_bi_Node_Ptr;
  double * f_si_Equ_si_Node_Ptr;
  double * f_si_Equ_bi_Node_Ptr;
  double * f_di_Equ_di_Node_Ptr;
  double * f_si_Equ_di_Node_Ptr;
  double * f_di_Equ_gi_Node_Ptr;
  double * f_si_Equ_gi_Node_Ptr;
  double * f_di_Equ_sbulk_Node_Ptr;
  double * f_si_Equ_sbulk_Node_Ptr;
  double * f_di_Equ_dbulk_Node_Ptr;
  double * f_si_Equ_dbulk_Node_Ptr;
  double * f_N2_Equ_sbulk_Node_Ptr;
  double * f_N2_Equ_si_Node_Ptr;
  double * f_N2_Equ_dbulk_Node_Ptr;
  double * f_N2_Equ_di_Node_Ptr;
  double * f_N2_Equ_t_Node_Ptr;
  double * f_N2_Equ_bi_Node_Ptr;
  double * f_N2_Equ_gi_Node_Ptr;
  double * f_N1_Equ_sbulk_Node_Ptr;
  double * f_N1_Equ_si_Node_Ptr;
  double * f_N1_Equ_dbulk_Node_Ptr;
  double * f_N1_Equ_di_Node_Ptr;
  double * f_N1_Equ_bi_Node_Ptr;
  double * f_N1_Equ_gi_Node_Ptr;
  double * f_N1_Equ_t_Node_Ptr;
  double * f_N1_Equ_N2_Node_Ptr;
  double * f_N1_Equ_N1_Node_Ptr;
  double * f_di_Equ_N2_Node_Ptr;
  double * f_si_Equ_N2_Node_Ptr;
  double * f_gi_Equ_N1_Node_Ptr;
  double * f_si_Equ_N1_Node_Ptr;
  double * f_gi_Equ_sbulk_Node_Ptr;
  double * f_gi_Equ_si_Node_Ptr;
  double * f_gi_Equ_dbulk_Node_Ptr;
  double * f_gi_Equ_di_Node_Ptr;
  double * f_gi_Equ_t_Node_Ptr;
  double * f_gi_Equ_bi_Node_Ptr;
  double * f_gi_Equ_gi_Node_Ptr;
  double * f_di_Equ_N1_Node_Ptr;
  double * f_N2_Equ_N2_Node_Ptr;
  double * f_bi_Equ_t_Node_Ptr;
  double * f_bi_Equ_si_Node_Ptr;
  double * f_bi_Equ_bi_Node_Ptr;
  double * f_bi_Equ_di_Node_Ptr;
  double * f_bi_Equ_gi_Node_Ptr;
  double * f_bi_Equ_sbulk_Node_Ptr;
  double * f_bi_Equ_dbulk_Node_Ptr;
  double * f_gm_Equ_gm_Node_Ptr;
  double * f_gm_Equ_si_Node_Ptr;
  double * f_si_Equ_gm_Node_Ptr;
  double * f_gm_Equ_bi_Node_Ptr;
  double * f_gm_Equ_di_Node_Ptr;
  double * f_gm_Equ_gi_Node_Ptr;
  double * f_gm_Equ_sbulk_Node_Ptr;
  double * f_gm_Equ_dbulk_Node_Ptr;
  double * f_gm_Equ_t_Node_Ptr;
  double * f_di_Equ_gm_Node_Ptr;
  double * f_bi_Equ_gm_Node_Ptr;
  double * f_d_Equ_si_Node_Ptr;
  double * f_d_Equ_bi_Node_Ptr;
  double * f_d_Equ_di_Node_Ptr;
  double * f_d_Equ_gi_Node_Ptr;
  double * f_d_Equ_t_Node_Ptr;
  double * f_d_Equ_d_Node_Ptr;
  double * f_di_Equ_d_Node_Ptr;
  double * f_s_Equ_si_Node_Ptr;
  double * f_s_Equ_bi_Node_Ptr;
  double * f_s_Equ_di_Node_Ptr;
  double * f_s_Equ_gi_Node_Ptr;
  double * f_s_Equ_t_Node_Ptr;
  double * f_s_Equ_s_Node_Ptr;
  double * f_si_Equ_s_Node_Ptr;
  double * f_g_Equ_t_Node_Ptr;
  double * f_g_Equ_si_Node_Ptr;
  double * f_g_Equ_bi_Node_Ptr;
  double * f_g_Equ_di_Node_Ptr;
  double * f_g_Equ_gi_Node_Ptr;
  double * f_g_Equ_g_Node_Ptr;
  double * f_g_Equ_gm_Node_Ptr;
  double * f_gm_Equ_g_Node_Ptr;
  double * f_gi_Equ_gm_Node_Ptr;
  double * f_t_Equ_s_Node_Ptr;
  double * f_t_Equ_si_Node_Ptr;
  double * f_t_Equ_d_Node_Ptr;
  double * f_t_Equ_di_Node_Ptr;
  double * f_t_Equ_bi_Node_Ptr;
  double * f_t_Equ_t_Node_Ptr;
  double * f_t_Equ_gi_Node_Ptr;
  double * f_t_Equ_sbulk_Node_Ptr;
  double * f_t_Equ_dbulk_Node_Ptr;
  double * f_t_Equ_gm_Node_Ptr;
  double * f_sbulk_Equ_bi_Node_Ptr;
  double * f_sbulk_Equ_sbulk_Node_Ptr;
  double * f_b_Equ_b_Node_Ptr;
  double * f_b_Equ_sbulk_Node_Ptr;
  double * f_sbulk_Equ_b_Node_Ptr;
  double * f_b_Equ_bi_Node_Ptr;
  double * f_bi_Equ_b_Node_Ptr;
  double * f_b_Equ_dbulk_Node_Ptr;
  double * f_dbulk_Equ_b_Node_Ptr;
  double * f_dbulk_Equ_dbulk_Node_Ptr;
  double * f_dbulk_Equ_bi_Node_Ptr;
  double * f_sbulk_Equ_t_Node_Ptr;
  double * f_b_Equ_t_Node_Ptr;
  double * f_dbulk_Equ_t_Node_Ptr;
  double * f_sbulk_Equ_si_Node_Ptr;
  double * f_sbulk_Equ_di_Node_Ptr;
  double * f_sbulk_Equ_gi_Node_Ptr;
  double * f_dbulk_Equ_si_Node_Ptr;
  double * f_dbulk_Equ_di_Node_Ptr;
  double * f_dbulk_Equ_gi_Node_Ptr;
  double * f_dbulk_Equ_sbulk_Node_Ptr;
  double * f_sbulk_Equ_dbulk_Node_Ptr;
  double * q_di_Equ_t_Node_Ptr;
  double * q_si_Equ_t_Node_Ptr;
  double * q_di_Equ_si_Node_Ptr;
  double * q_di_Equ_bi_Node_Ptr;
  double * q_si_Equ_si_Node_Ptr;
  double * q_si_Equ_bi_Node_Ptr;
  double * q_di_Equ_di_Node_Ptr;
  double * q_si_Equ_di_Node_Ptr;
  double * q_di_Equ_gi_Node_Ptr;
  double * q_si_Equ_gi_Node_Ptr;
  double * q_di_Equ_sbulk_Node_Ptr;
  double * q_si_Equ_sbulk_Node_Ptr;
  double * q_di_Equ_dbulk_Node_Ptr;
  double * q_si_Equ_dbulk_Node_Ptr;
  double * q_N2_Equ_sbulk_Node_Ptr;
  double * q_N2_Equ_si_Node_Ptr;
  double * q_N2_Equ_dbulk_Node_Ptr;
  double * q_N2_Equ_di_Node_Ptr;
  double * q_N2_Equ_t_Node_Ptr;
  double * q_N2_Equ_bi_Node_Ptr;
  double * q_N2_Equ_gi_Node_Ptr;
  double * q_N1_Equ_sbulk_Node_Ptr;
  double * q_N1_Equ_si_Node_Ptr;
  double * q_N1_Equ_dbulk_Node_Ptr;
  double * q_N1_Equ_di_Node_Ptr;
  double * q_N1_Equ_bi_Node_Ptr;
  double * q_N1_Equ_gi_Node_Ptr;
  double * q_N1_Equ_t_Node_Ptr;
  double * q_N1_Equ_N2_Node_Ptr;
  double * q_N1_Equ_N1_Node_Ptr;
  double * q_di_Equ_N2_Node_Ptr;
  double * q_si_Equ_N2_Node_Ptr;
  double * q_gi_Equ_N1_Node_Ptr;
  double * q_si_Equ_N1_Node_Ptr;
  double * q_gi_Equ_sbulk_Node_Ptr;
  double * q_gi_Equ_si_Node_Ptr;
  double * q_gi_Equ_dbulk_Node_Ptr;
  double * q_gi_Equ_di_Node_Ptr;
  double * q_gi_Equ_t_Node_Ptr;
  double * q_gi_Equ_bi_Node_Ptr;
  double * q_gi_Equ_gi_Node_Ptr;
  double * q_di_Equ_N1_Node_Ptr;
  double * q_N2_Equ_N2_Node_Ptr;
  double * q_bi_Equ_t_Node_Ptr;
  double * q_bi_Equ_si_Node_Ptr;
  double * q_bi_Equ_bi_Node_Ptr;
  double * q_bi_Equ_di_Node_Ptr;
  double * q_bi_Equ_gi_Node_Ptr;
  double * q_bi_Equ_sbulk_Node_Ptr;
  double * q_bi_Equ_dbulk_Node_Ptr;
  double * q_gm_Equ_gm_Node_Ptr;
  double * q_gm_Equ_si_Node_Ptr;
  double * q_si_Equ_gm_Node_Ptr;
  double * q_gm_Equ_bi_Node_Ptr;
  double * q_gm_Equ_di_Node_Ptr;
  double * q_gm_Equ_gi_Node_Ptr;
  double * q_gm_Equ_sbulk_Node_Ptr;
  double * q_gm_Equ_dbulk_Node_Ptr;
  double * q_gm_Equ_t_Node_Ptr;
  double * q_di_Equ_gm_Node_Ptr;
  double * q_bi_Equ_gm_Node_Ptr;
  double * q_d_Equ_si_Node_Ptr;
  double * q_d_Equ_bi_Node_Ptr;
  double * q_d_Equ_di_Node_Ptr;
  double * q_d_Equ_gi_Node_Ptr;
  double * q_d_Equ_t_Node_Ptr;
  double * q_d_Equ_d_Node_Ptr;
  double * q_di_Equ_d_Node_Ptr;
  double * q_s_Equ_si_Node_Ptr;
  double * q_s_Equ_bi_Node_Ptr;
  double * q_s_Equ_di_Node_Ptr;
  double * q_s_Equ_gi_Node_Ptr;
  double * q_s_Equ_t_Node_Ptr;
  double * q_s_Equ_s_Node_Ptr;
  double * q_si_Equ_s_Node_Ptr;
  double * q_g_Equ_t_Node_Ptr;
  double * q_g_Equ_si_Node_Ptr;
  double * q_g_Equ_bi_Node_Ptr;
  double * q_g_Equ_di_Node_Ptr;
  double * q_g_Equ_gi_Node_Ptr;
  double * q_g_Equ_g_Node_Ptr;
  double * q_g_Equ_gm_Node_Ptr;
  double * q_gm_Equ_g_Node_Ptr;
  double * q_gi_Equ_gm_Node_Ptr;
  double * q_t_Equ_s_Node_Ptr;
  double * q_t_Equ_si_Node_Ptr;
  double * q_t_Equ_d_Node_Ptr;
  double * q_t_Equ_di_Node_Ptr;
  double * q_t_Equ_bi_Node_Ptr;
  double * q_t_Equ_t_Node_Ptr;
  double * q_t_Equ_gi_Node_Ptr;
  double * q_t_Equ_sbulk_Node_Ptr;
  double * q_t_Equ_dbulk_Node_Ptr;
  double * q_t_Equ_gm_Node_Ptr;
  double * q_sbulk_Equ_bi_Node_Ptr;
  double * q_sbulk_Equ_sbulk_Node_Ptr;
  double * q_b_Equ_b_Node_Ptr;
  double * q_b_Equ_sbulk_Node_Ptr;
  double * q_sbulk_Equ_b_Node_Ptr;
  double * q_b_Equ_bi_Node_Ptr;
  double * q_bi_Equ_b_Node_Ptr;
  double * q_b_Equ_dbulk_Node_Ptr;
  double * q_dbulk_Equ_b_Node_Ptr;
  double * q_dbulk_Equ_dbulk_Node_Ptr;
  double * q_dbulk_Equ_bi_Node_Ptr;
  double * q_sbulk_Equ_t_Node_Ptr;
  double * q_b_Equ_t_Node_Ptr;
  double * q_dbulk_Equ_t_Node_Ptr;
  double * q_sbulk_Equ_si_Node_Ptr;
  double * q_sbulk_Equ_di_Node_Ptr;
  double * q_sbulk_Equ_gi_Node_Ptr;
  double * q_dbulk_Equ_si_Node_Ptr;
  double * q_dbulk_Equ_di_Node_Ptr;
  double * q_dbulk_Equ_gi_Node_Ptr;
  double * q_dbulk_Equ_sbulk_Node_Ptr;
  double * q_sbulk_Equ_dbulk_Node_Ptr;
  // Jacobian offsets
  int A_di_Equ_t_NodeOffset;
  int A_si_Equ_t_NodeOffset;
  int A_di_Equ_si_NodeOffset;
  int A_di_Equ_bi_NodeOffset;
  int A_si_Equ_si_NodeOffset;
  int A_si_Equ_bi_NodeOffset;
  int A_di_Equ_di_NodeOffset;
  int A_si_Equ_di_NodeOffset;
  int A_di_Equ_gi_NodeOffset;
  int A_si_Equ_gi_NodeOffset;
  int A_di_Equ_sbulk_NodeOffset;
  int A_si_Equ_sbulk_NodeOffset;
  int A_di_Equ_dbulk_NodeOffset;
  int A_si_Equ_dbulk_NodeOffset;
  int A_N2_Equ_sbulk_NodeOffset;
  int A_N2_Equ_si_NodeOffset;
  int A_N2_Equ_dbulk_NodeOffset;
  int A_N2_Equ_di_NodeOffset;
  int A_N2_Equ_t_NodeOffset;
  int A_N2_Equ_bi_NodeOffset;
  int A_N2_Equ_gi_NodeOffset;
  int A_N1_Equ_sbulk_NodeOffset;
  int A_N1_Equ_si_NodeOffset;
  int A_N1_Equ_dbulk_NodeOffset;
  int A_N1_Equ_di_NodeOffset;
  int A_N1_Equ_bi_NodeOffset;
  int A_N1_Equ_gi_NodeOffset;
  int A_N1_Equ_t_NodeOffset;
  int A_N1_Equ_N2_NodeOffset;
  int A_N1_Equ_N1_NodeOffset;
  int A_di_Equ_N2_NodeOffset;
  int A_si_Equ_N2_NodeOffset;
  int A_gi_Equ_N1_NodeOffset;
  int A_si_Equ_N1_NodeOffset;
  int A_gi_Equ_sbulk_NodeOffset;
  int A_gi_Equ_si_NodeOffset;
  int A_gi_Equ_dbulk_NodeOffset;
  int A_gi_Equ_di_NodeOffset;
  int A_gi_Equ_t_NodeOffset;
  int A_gi_Equ_bi_NodeOffset;
  int A_gi_Equ_gi_NodeOffset;
  int A_di_Equ_N1_NodeOffset;
  int A_N2_Equ_N2_NodeOffset;
  int A_bi_Equ_t_NodeOffset;
  int A_bi_Equ_si_NodeOffset;
  int A_bi_Equ_bi_NodeOffset;
  int A_bi_Equ_di_NodeOffset;
  int A_bi_Equ_gi_NodeOffset;
  int A_bi_Equ_sbulk_NodeOffset;
  int A_bi_Equ_dbulk_NodeOffset;
  int A_gm_Equ_gm_NodeOffset;
  int A_gm_Equ_si_NodeOffset;
  int A_si_Equ_gm_NodeOffset;
  int A_gm_Equ_bi_NodeOffset;
  int A_gm_Equ_di_NodeOffset;
  int A_gm_Equ_gi_NodeOffset;
  int A_gm_Equ_sbulk_NodeOffset;
  int A_gm_Equ_dbulk_NodeOffset;
  int A_gm_Equ_t_NodeOffset;
  int A_di_Equ_gm_NodeOffset;
  int A_bi_Equ_gm_NodeOffset;
  int A_d_Equ_si_NodeOffset;
  int A_d_Equ_bi_NodeOffset;
  int A_d_Equ_di_NodeOffset;
  int A_d_Equ_gi_NodeOffset;
  int A_d_Equ_t_NodeOffset;
  int A_d_Equ_d_NodeOffset;
  int A_di_Equ_d_NodeOffset;
  int A_s_Equ_si_NodeOffset;
  int A_s_Equ_bi_NodeOffset;
  int A_s_Equ_di_NodeOffset;
  int A_s_Equ_gi_NodeOffset;
  int A_s_Equ_t_NodeOffset;
  int A_s_Equ_s_NodeOffset;
  int A_si_Equ_s_NodeOffset;
  int A_g_Equ_t_NodeOffset;
  int A_g_Equ_si_NodeOffset;
  int A_g_Equ_bi_NodeOffset;
  int A_g_Equ_di_NodeOffset;
  int A_g_Equ_gi_NodeOffset;
  int A_g_Equ_g_NodeOffset;
  int A_g_Equ_gm_NodeOffset;
  int A_gm_Equ_g_NodeOffset;
  int A_gi_Equ_gm_NodeOffset;
  int A_t_Equ_s_NodeOffset;
  int A_t_Equ_si_NodeOffset;
  int A_t_Equ_d_NodeOffset;
  int A_t_Equ_di_NodeOffset;
  int A_t_Equ_bi_NodeOffset;
  int A_t_Equ_t_NodeOffset;
  int A_t_Equ_gi_NodeOffset;
  int A_t_Equ_sbulk_NodeOffset;
  int A_t_Equ_dbulk_NodeOffset;
  int A_t_Equ_gm_NodeOffset;
  int A_sbulk_Equ_bi_NodeOffset;
  int A_sbulk_Equ_sbulk_NodeOffset;
  int A_b_Equ_b_NodeOffset;
  int A_b_Equ_sbulk_NodeOffset;
  int A_sbulk_Equ_b_NodeOffset;
  int A_b_Equ_bi_NodeOffset;
  int A_bi_Equ_b_NodeOffset;
  int A_b_Equ_dbulk_NodeOffset;
  int A_dbulk_Equ_b_NodeOffset;
  int A_dbulk_Equ_dbulk_NodeOffset;
  int A_dbulk_Equ_bi_NodeOffset;
  int A_sbulk_Equ_t_NodeOffset;
  int A_b_Equ_t_NodeOffset;
  int A_dbulk_Equ_t_NodeOffset;
  int A_sbulk_Equ_si_NodeOffset;
  int A_sbulk_Equ_di_NodeOffset;
  int A_sbulk_Equ_gi_NodeOffset;
  int A_dbulk_Equ_si_NodeOffset;
  int A_dbulk_Equ_di_NodeOffset;
  int A_dbulk_Equ_gi_NodeOffset;
  int A_dbulk_Equ_sbulk_NodeOffset;
  int A_sbulk_Equ_dbulk_NodeOffset;
  // end of Jacobian and pointers
  // node numbers
  static const int admsNodeID_d = 0;
  static const int admsNodeID_g = 1;
  static const int admsNodeID_s = 2;
  static const int admsNodeID_b = 3;
  static const int admsNodeID_di = 4;
  static const int admsNodeID_si = 5;
  static const int admsNodeID_gi = 6;
  static const int admsNodeID_gm = 7;
  static const int admsNodeID_bi = 8;
  static const int admsNodeID_sbulk = 9;
  static const int admsNodeID_dbulk = 10;
  static const int admsNodeID_N1 = 11;
  static const int admsNodeID_N2 = 12;
  static const int admsNodeID_t = 13;
  static const int admsNodeID_GND = -1;
  // end node numbers
  // Additional IDs for branch equations
  // end branch numbers
  // Probe numbers
  static const int admsProbeID_V_bi_dbulk = 0;
  static const int admsProbeID_V_b_dbulk = 1;
  static const int admsProbeID_V_b_bi = 2;
  static const int admsProbeID_V_b_sbulk = 3;
  static const int admsProbeID_V_bi_sbulk = 4;
  static const int admsProbeID_V_di_si = 5;
  static const int admsProbeID_V_gm_gi = 6;
  static const int admsProbeID_V_g_gm = 7;
  static const int admsProbeID_V_s_si = 8;
  static const int admsProbeID_V_d_di = 9;
  static const int admsProbeID_V_gm_bi = 10;
  static const int admsProbeID_V_N1_GND = 11;
  static const int admsProbeID_V_N2_GND = 12;
  static const int admsProbeID_V_gm_si = 13;
  static const int admsProbeID_V_gm_di = 14;
  static const int admsProbeID_V_dbulk_di = 15;
  static const int admsProbeID_V_sbulk_si = 16;
  static const int admsProbeID_V_si_bi = 17;
  static const int admsProbeID_V_di_bi = 18;
  static const int admsProbeID_V_gi_bi = 19;
  static const int admsProbeID_Temp_t_GND = 20;
  // end probe numbers
  // Store LIDs
  // end store LIDs
  // bools for collapsing nodes
  bool collapseNode_di;
  bool collapseNode_si;
  bool collapseNode_gi;
  bool collapseNode_gm;
  bool collapseNode_bi;
  bool collapseNode_sbulk;
  bool collapseNode_dbulk;
  bool collapseNode_t;
  // Arrays to hold probes
  std::vector < AdmsFadType > probeVars;
  // Arrays to hold contributions
  // dynamic contributions are differentiated w.r.t time
  std::vector < AdmsFadType > staticContributions;
  std::vector < AdmsFadType > dynamicContributions;
  //vectors to hold noise information for communication between methods
  std::vector<double> noiseContribsPower;
  std::vector<double> noiseContribsExponent;


  // this is what we'll use when any model uses $temperature.  We'll
  // set it in updateTemperature, and initialize it to whatever
  // is in devOptions when the instance is constructed.
  double admsTemperature;

  // vt at $temperature;
  double adms_vt_nom;


  // This one is for the annoying bogus "XyceADMSInstTemp" parameter
  // that we need so we can set it from the device manager when there's no
  // "TEMP" parameter to use
  double admsInstTemp;


  JacobianStamp jacStamp;
  IdVector nodeMap;
  PairMap pairToJacStampMap;

  // These instance-owned vectors are for storage of lead current data
  std::vector<double> leadCurrentF;
  std::vector<double> leadCurrentQ;
};



namespace AnalogFunctions
{

// Analog Function lexp
template<typename ScalarT> ScalarT lexp(ScalarT x)
{


  ScalarT lexp;
  {
    if ((x>80.0))
    {
      lexp = (5.540622384e+34*((1.0+x)-80.0));
    }
    else
    {
      if ((x<(-80.0)))
      {
        lexp = 1.804851387e-35;
      }
      else
      {
        lexp = exp(x);
      }
    }
  }
  return(lexp);
}


// Analog Function lln
template<typename ScalarT> ScalarT lln(ScalarT x)
{


  ScalarT lln;
  {
    lln = log(std::max(x,static_cast<double>(1.0e-38)));
  }
  return(lln);
}


// Analog Function hypsmooth
template<typename ScalarT> ScalarT hypsmooth(ScalarT x, ScalarT c)
{


  ScalarT hypsmooth;
  {
    hypsmooth = (0.5*(x+sqrt(((x*x)+((4.0*c)*c)))));
  }
  return(hypsmooth);
}

}


//-----------------------------------------------------------------------------
// Class         : Model

// Purpose       :
// Special Notes :
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
class Model : public DeviceModel
{
  typedef std::vector<Instance *> InstanceVector;

  friend class ParametricData<Model>;
  friend class Instance;
#ifdef Xyce_ADMS_SENSITIVITIES
  friend class InstanceSensitivity;
  friend class ModelSensitivity;
#endif // Xyce_ADMS_SENSITIVITIES
  friend struct Traits;

public:
  Model(
     const Configuration &       configuration,
     const ModelBlock &          model_block,
     const FactoryBlock &        factory_block);

  ~Model();

private:
  Model(const Model &);
  Model &operator=(const Model &);

public:
  virtual void forEachInstance(DeviceInstanceOp &op) const /* override */;
  virtual std::ostream &printOutInstances(std::ostream &os) const;
  bool processParams();
  bool processInstanceParams();

private:

public:
  void addInstance(Instance *instance)
  {
    instanceContainer.push_back(instance);
  }

  void setupBaseInstanceContainer()
  {
    std::vector<Instance*>::iterator iter = instanceContainer.begin();
    std::vector<Instance*>::iterator end   = instanceContainer.end();
    for ( ; iter!=end; ++iter)
    {
      Xyce::Device::DeviceModel::baseInstanceContainer.push_back( static_cast<Xyce::Device::DeviceInstance *>(*iter) );
    }
  }

private:
  std::vector<Instance*> instanceContainer;

private:

  // This one is for the annoying bogus "XyceADMSInstTemp" parameter
  // that we need so we can set it from the device manager when there's no
  // "TEMP" model parameter to use
  double admsModTemp;
  // Begin verilog Model Variables
  //   Model Parameters
  double L;
  double VFBSDOFF;
  double XGW;
  int NGCON;
  int RGATEMOD;
  int RBODYMOD;
  int GEOMOD;
  int RGEOMOD;
  double RBPB;
  double RBPD;
  double RBPS;
  double RBDB;
  double RBSB;
  double SA;
  double SB;
  double SD;
  int SCA;
  int SCB;
  int SCC;
  int SC;
  int TYPE;
  int CVMOD;
  int COVMOD;
  int RDSMOD;
  int WPEMOD;
  int ASYMMOD;
  int GIDLMOD;
  int IGCMOD;
  int IGBMOD;
  int TNOIMOD;
  int SHMOD;
  int MOBSCALE;
  double LLONG;
  double LMLT;
  double WMLT;
  double XL;
  double WWIDE;
  double XW;
  double LINT;
  double LL;
  double LW;
  double LWL;
  double LLN;
  double LWN;
  double WINT;
  double WL;
  double WW;
  double WWL;
  double WLN;
  double WWN;
  double DLC;
  double LLC;
  double LWC;
  double LWLC;
  double DWC;
  double WLC;
  double WWC;
  double WWLC;
  double TOXE;
  double TOXP;
  double DTOX;
  double NDEP;
  double NDEPL1;
  double NDEPLEXP1;
  double NDEPL2;
  double NDEPLEXP2;
  double NDEPW;
  double NDEPWEXP;
  double NDEPWL;
  double NDEPWLEXP;
  double LNDEP;
  double WNDEP;
  double PNDEP;
  double NDEPCV;
  double NDEPCVL1;
  double NDEPCVLEXP1;
  double NDEPCVL2;
  double NDEPCVLEXP2;
  double NDEPCVW;
  double NDEPCVWEXP;
  double NDEPCVWL;
  double NDEPCVWLEXP;
  double LNDEPCV;
  double WNDEPCV;
  double PNDEPCV;
  double NGATE;
  double LNGATE;
  double WNGATE;
  double PNGATE;
  double EASUB;
  double NI0SUB;
  double BG0SUB;
  double EPSRSUB;
  double EPSROX;
  double XJ;
  double LXJ;
  double WXJ;
  double PXJ;
  double LVFB;
  double WVFB;
  double PVFB;
  double VFB;
  double VFBCV;
  double LVFBCV;
  double WVFBCV;
  double PVFBCV;
  double VFBCVL;
  double VFBCVLEXP;
  double VFBCVW;
  double VFBCVWEXP;
  double VFBCVWL;
  double VFBCVWLEXP;
  int PERMOD;
  double DWJ;
  double NSD;
  double LNSD;
  double WNSD;
  double PNSD;
  double DELVT0;
  double DVTP0;
  double LDVTP0;
  double WDVTP0;
  double PDVTP0;
  double DVTP1;
  double LDVTP1;
  double WDVTP1;
  double PDVTP1;
  double DVTP2;
  double LDVTP2;
  double WDVTP2;
  double PDVTP2;
  double DVTP3;
  double LDVTP3;
  double WDVTP3;
  double PDVTP3;
  double DVTP4;
  double LDVTP4;
  double WDVTP4;
  double PDVTP4;
  double DVTP5;
  double LDVTP5;
  double WDVTP5;
  double PDVTP5;
  double PHIN;
  double LPHIN;
  double WPHIN;
  double PPHIN;
  double ETA0;
  double LETA0;
  double WETA0;
  double PETA0;
  double ETA0R;
  double LETA0R;
  double WETA0R;
  double PETA0R;
  double DSUB;
  double ETAB;
  double ETABEXP;
  double LETAB;
  double WETAB;
  double PETAB;
  double K1;
  double K1L;
  double K1LEXP;
  double K1W;
  double K1WEXP;
  double K1WL;
  double K1WLEXP;
  double LK1;
  double WK1;
  double PK1;
  double K2;
  double K2L;
  double K2LEXP;
  double K2W;
  double K2WEXP;
  double K2WL;
  double K2WLEXP;
  double LK2;
  double WK2;
  double PK2;
  double ADOS;
  double BDOS;
  double QM0;
  double ETAQM;
  double CIT;
  double LCIT;
  double WCIT;
  double PCIT;
  double NFACTOR;
  double NFACTORL;
  double NFACTORLEXP;
  double NFACTORW;
  double NFACTORWEXP;
  double NFACTORWL;
  double NFACTORWLEXP;
  double LNFACTOR;
  double WNFACTOR;
  double PNFACTOR;
  double CDSCD;
  double CDSCDL;
  double CDSCDLEXP;
  double LCDSCD;
  double WCDSCD;
  double PCDSCD;
  double CDSCDR;
  double CDSCDLR;
  double LCDSCDR;
  double WCDSCDR;
  double PCDSCDR;
  double CDSCB;
  double CDSCBL;
  double CDSCBLEXP;
  double LCDSCB;
  double WCDSCB;
  double PCDSCB;
  double VSAT;
  double LVSAT;
  double WVSAT;
  double PVSAT;
  double VSATL;
  double VSATLEXP;
  double VSATW;
  double VSATWEXP;
  double VSATWL;
  double VSATWLEXP;
  double VSATR;
  double LVSATR;
  double WVSATR;
  double PVSATR;
  double DELTA;
  double LDELTA;
  double WDELTA;
  double PDELTA;
  double DELTAL;
  double DELTALEXP;
  double VSATCV;
  double LVSATCV;
  double WVSATCV;
  double PVSATCV;
  double VSATCVL;
  double VSATCVLEXP;
  double VSATCVW;
  double VSATCVWEXP;
  double VSATCVWL;
  double VSATCVWLEXP;
  double UP1;
  double LP1;
  double UP2;
  double LP2;
  double MULU0;
  double U0;
  double U0L;
  double U0LEXP;
  double LU0;
  double WU0;
  double PU0;
  double U0R;
  double LU0R;
  double WU0R;
  double PU0R;
  double ETAMOB;
  double UA;
  double UAL;
  double UALEXP;
  double UAW;
  double UAWEXP;
  double UAWL;
  double UAWLEXP;
  double LUA;
  double WUA;
  double PUA;
  double UAR;
  double LUAR;
  double WUAR;
  double PUAR;
  double EU;
  double LEU;
  double WEU;
  double PEU;
  double EUL;
  double EULEXP;
  double EUW;
  double EUWEXP;
  double EUWL;
  double EUWLEXP;
  double UD;
  double UDL;
  double UDLEXP;
  double LUD;
  double WUD;
  double PUD;
  double UDR;
  double LUDR;
  double WUDR;
  double PUDR;
  double UCS;
  double LUCS;
  double WUCS;
  double PUCS;
  double UCSR;
  double LUCSR;
  double WUCSR;
  double PUCSR;
  double UC;
  double UCL;
  double UCLEXP;
  double UCW;
  double UCWEXP;
  double UCWL;
  double UCWLEXP;
  double LUC;
  double WUC;
  double PUC;
  double UCR;
  double LUCR;
  double WUCR;
  double PUCR;
  double PCLM;
  double PCLML;
  double PCLMLEXP;
  double LPCLM;
  double WPCLM;
  double PPCLM;
  double PCLMR;
  double LPCLMR;
  double WPCLMR;
  double PPCLMR;
  double PCLMG;
  double PCLMCV;
  double PCLMCVL;
  double PCLMCVLEXP;
  double LPCLMCV;
  double WPCLMCV;
  double PPCLMCV;
  double PSCBE1;
  double LPSCBE1;
  double WPSCBE1;
  double PPSCBE1;
  double PSCBE2;
  double LPSCBE2;
  double WPSCBE2;
  double PPSCBE2;
  double PDITS;
  double LPDITS;
  double WPDITS;
  double PPDITS;
  double PDITSL;
  double PDITSD;
  double LPDITSD;
  double WPDITSD;
  double PPDITSD;
  double RSH;
  double PRWG;
  double LPRWG;
  double WPRWG;
  double PPRWG;
  double PRWB;
  double LPRWB;
  double WPRWB;
  double PPRWB;
  double PRWBL;
  double PRWBLEXP;
  double WR;
  double LWR;
  double WWR;
  double PWR;
  double RSWMIN;
  double LRSWMIN;
  double WRSWMIN;
  double PRSWMIN;
  double RSW;
  double LRSW;
  double WRSW;
  double PRSW;
  double RSWL;
  double RSWLEXP;
  double RDWMIN;
  double LRDWMIN;
  double WRDWMIN;
  double PRDWMIN;
  double RDW;
  double LRDW;
  double WRDW;
  double PRDW;
  double RDWL;
  double RDWLEXP;
  double RDSWMIN;
  double LRDSWMIN;
  double WRDSWMIN;
  double PRDSWMIN;
  double RDSW;
  double RDSWL;
  double RDSWLEXP;
  double LRDSW;
  double WRDSW;
  double PRDSW;
  double PSAT;
  double LPSAT;
  double WPSAT;
  double PPSAT;
  double PSATL;
  double PSATLEXP;
  double PSATB;
  double PSATR;
  double LPSATR;
  double WPSATR;
  double PPSATR;
  double LPSATB;
  double WPSATB;
  double PPSATB;
  double PSATX;
  double PTWG;
  double LPTWG;
  double WPTWG;
  double PPTWG;
  double PTWGL;
  double PTWGLEXP;
  double PTWGR;
  double LPTWGR;
  double WPTWGR;
  double PPTWGR;
  double PTWGLR;
  double PTWGLEXPR;
  double A1;
  double LA1;
  double WA1;
  double PA1;
  double A11;
  double LA11;
  double WA11;
  double PA11;
  double A2;
  double LA2;
  double WA2;
  double PA2;
  double A21;
  double LA21;
  double WA21;
  double PA21;
  double PDIBLC;
  double PDIBLCL;
  double PDIBLCLEXP;
  double LPDIBLC;
  double WPDIBLC;
  double PPDIBLC;
  double PDIBLCR;
  double PDIBLCLR;
  double PDIBLCLEXPR;
  double LPDIBLCR;
  double WPDIBLCR;
  double PPDIBLCR;
  double PDIBLCB;
  double LPDIBLCB;
  double WPDIBLCB;
  double PPDIBLCB;
  double PVAG;
  double LPVAG;
  double WPVAG;
  double PPVAG;
  double FPROUT;
  double FPROUTL;
  double FPROUTLEXP;
  double LFPROUT;
  double WFPROUT;
  double PFPROUT;
  double ALPHA0;
  double ALPHA0L;
  double ALPHA0LEXP;
  double LALPHA0;
  double WALPHA0;
  double PALPHA0;
  double BETA0;
  double LBETA0;
  double WBETA0;
  double PBETA0;
  double AIGBACC;
  double BIGBACC;
  double CIGBACC;
  double NIGBACC;
  double AIGBINV;
  double BIGBINV;
  double CIGBINV;
  double EIGBINV;
  double NIGBINV;
  double AIGC;
  double BIGC;
  double CIGC;
  double AIGS;
  double BIGS;
  double CIGS;
  double AIGD;
  double BIGD;
  double CIGD;
  double DLCIG;
  double DLCIGD;
  double POXEDGE;
  double NTOX;
  double TOXREF;
  double PIGCD;
  double AIGCL;
  double AIGCW;
  double AIGSL;
  double AIGSW;
  double AIGDL;
  double AIGDW;
  double PIGCDL;
  double LAIGBINV;
  double WAIGBINV;
  double PAIGBINV;
  double LBIGBINV;
  double WBIGBINV;
  double PBIGBINV;
  double LCIGBINV;
  double WCIGBINV;
  double PCIGBINV;
  double LEIGBINV;
  double WEIGBINV;
  double PEIGBINV;
  double LNIGBINV;
  double WNIGBINV;
  double PNIGBINV;
  double LAIGBACC;
  double WAIGBACC;
  double PAIGBACC;
  double LBIGBACC;
  double WBIGBACC;
  double PBIGBACC;
  double LCIGBACC;
  double WCIGBACC;
  double PCIGBACC;
  double LNIGBACC;
  double WNIGBACC;
  double PNIGBACC;
  double LAIGC;
  double WAIGC;
  double PAIGC;
  double LBIGC;
  double WBIGC;
  double PBIGC;
  double LCIGC;
  double WCIGC;
  double PCIGC;
  double LAIGS;
  double WAIGS;
  double PAIGS;
  double LBIGS;
  double WBIGS;
  double PBIGS;
  double LCIGS;
  double WCIGS;
  double PCIGS;
  double LAIGD;
  double WAIGD;
  double PAIGD;
  double LBIGD;
  double WBIGD;
  double PBIGD;
  double LCIGD;
  double WCIGD;
  double PCIGD;
  double LPOXEDGE;
  double WPOXEDGE;
  double PPOXEDGE;
  double LDLCIG;
  double WDLCIG;
  double PDLCIG;
  double LDLCIGD;
  double WDLCIGD;
  double PDLCIGD;
  double LNTOX;
  double WNTOX;
  double PNTOX;
  double AGIDL;
  double AGIDLL;
  double AGIDLW;
  double LAGIDL;
  double WAGIDL;
  double PAGIDL;
  double BGIDL;
  double LBGIDL;
  double WBGIDL;
  double PBGIDL;
  double CGIDL;
  double LCGIDL;
  double WCGIDL;
  double PCGIDL;
  double EGIDL;
  double LEGIDL;
  double WEGIDL;
  double PEGIDL;
  double AGISL;
  double AGISLL;
  double AGISLW;
  double LAGISL;
  double WAGISL;
  double PAGISL;
  double BGISL;
  double LBGISL;
  double WBGISL;
  double PBGISL;
  double CGISL;
  double LCGISL;
  double WCGISL;
  double PCGISL;
  double EGISL;
  double LEGISL;
  double WEGISL;
  double PEGISL;
  double CF;
  double LCF;
  double WCF;
  double PCF;
  double CFRCOEFF;
  double CGSO;
  double CGDO;
  double CGBO;
  double CGSL;
  double LCGSL;
  double WCGSL;
  double PCGSL;
  double CGDL;
  double LCGDL;
  double WCGDL;
  double PCGDL;
  double CKAPPAS;
  double LCKAPPAS;
  double WCKAPPAS;
  double PCKAPPAS;
  double CKAPPAD;
  double LCKAPPAD;
  double WCKAPPAD;
  double PCKAPPAD;
  double DMCG;
  double DMCI;
  double DMDG;
  double DMCGT;
  double XGL;
  double RSHG;
  double CJS;
  double CJD;
  double CJSWS;
  double CJSWD;
  double CJSWGS;
  double CJSWGD;
  double PBS;
  double PBD;
  double PBSWS;
  double PBSWD;
  double PBSWGS;
  double PBSWGD;
  double MJS;
  double MJD;
  double MJSWS;
  double MJSWD;
  double MJSWGS;
  double MJSWGD;
  double JSS;
  double JSD;
  double JSWS;
  double JSWD;
  double JSWGS;
  double JSWGD;
  double NJS;
  double NJD;
  double IJTHSFWD;
  double IJTHDFWD;
  double IJTHSREV;
  double IJTHDREV;
  double BVS;
  double BVD;
  double XJBVS;
  double XJBVD;
  double JTSS;
  double JTSD;
  double JTSSWS;
  double JTSSWD;
  double JTSSWGS;
  double JTSSWGD;
  double JTWEFF;
  double NJTS;
  double NJTSD;
  double NJTSSW;
  double NJTSSWD;
  double NJTSSWG;
  double NJTSSWGD;
  double VTSS;
  double VTSD;
  double VTSSWS;
  double VTSSWD;
  double VTSSWGS;
  double VTSSWGD;
  double XRCRG1;
  double XRCRG2;
  double GBMIN;
  double RBPS0;
  double RBPSL;
  double RBPSW;
  double RBPSNF;
  double RBPD0;
  double RBPDL;
  double RBPDW;
  double RBPDNF;
  double RBPBX0;
  double RBPBXL;
  double RBPBXW;
  double RBPBXNF;
  double RBPBY0;
  double RBPBYL;
  double RBPBYW;
  double RBPBYNF;
  double RBSBX0;
  double RBSBY0;
  double RBDBX0;
  double RBDBY0;
  double RBSDBXL;
  double RBSDBXW;
  double RBSDBXNF;
  double RBSDBYL;
  double RBSDBYW;
  double RBSDBYNF;
  double EF;
  double EM;
  double NOIA;
  double NOIB;
  double NOIC;
  double LINTNOI;
  double NTNOI;
  double RNOIA;
  double RNOIB;
  double RNOIC;
  double TNOIA;
  double TNOIB;
  double TNOIC;
  int BINUNIT;
  double DLBIN;
  double DWBIN;
  double TNOM;
  double DTEMP;
  double TBGASUB;
  double TBGBSUB;
  double TNFACTOR;
  double UTE;
  double LUTE;
  double WUTE;
  double PUTE;
  double UTEL;
  double UA1;
  double LUA1;
  double WUA1;
  double PUA1;
  double UA1L;
  double UC1;
  double LUC1;
  double WUC1;
  double PUC1;
  double UD1;
  double LUD1;
  double WUD1;
  double PUD1;
  double UD1L;
  double UCSTE;
  double LUCSTE;
  double WUCSTE;
  double PUCSTE;
  double TETA0;
  double PRT;
  double LPRT;
  double WPRT;
  double PPRT;
  double AT;
  double LAT;
  double WAT;
  double PAT;
  double ATL;
  double TDELTA;
  double PTWGT;
  double LPTWGT;
  double WPTWGT;
  double PPTWGT;
  double PTWGTL;
  double KT1;
  double KT1EXP;
  double KT1L;
  double LKT1;
  double WKT1;
  double PKT1;
  double KT2;
  double LKT2;
  double WKT2;
  double PKT2;
  double IIT;
  double LIIT;
  double WIIT;
  double PIIT;
  double IGT;
  double LIGT;
  double WIGT;
  double PIGT;
  double TGIDL;
  double LTGIDL;
  double WTGIDL;
  double PTGIDL;
  double TCJ;
  double TCJSW;
  double TCJSWG;
  double TPB;
  double TPBSW;
  double TPBSWG;
  double XTIS;
  double XTID;
  double XTSS;
  double XTSD;
  double XTSSWS;
  double XTSSWD;
  double XTSSWGS;
  double XTSSWGD;
  double TNJTS;
  double TNJTSD;
  double TNJTSSW;
  double TNJTSSWD;
  double TNJTSSWG;
  double TNJTSSWGD;
  double RTH0;
  double CTH0;
  double WTH0;
  double SAREF;
  double SBREF;
  double WLOD;
  double KU0;
  double KVSAT;
  double TKU0;
  double LKU0;
  double WKU0;
  double PKU0;
  double LLODKU0;
  double WLODKU0;
  double KVTH0;
  double LKVTH0;
  double WKVTH0;
  double PKVTH0;
  double LLODVTH;
  double WLODVTH;
  double STK2;
  double LODK2;
  double STETA0;
  double LODETA0;
  double WEB;
  double WEC;
  double KVTH0WE;
  double LKVTH0WE;
  double WKVTH0WE;
  double PKVTH0WE;
  double K2WE;
  double LK2WE;
  double WK2WE;
  double PK2WE;
  double KU0WE;
  double LKU0WE;
  double WKU0WE;
  double PKU0WE;
  double SCREF;
  int EDGEFET;
  double WEDGE;
  double DGAMMAEDGE;
  double DGAMMAEDGEL;
  double DGAMMAEDGELEXP;
  double DVTEDGE;
  double NFACTOREDGE;
  double CITEDGE;
  double CDSCDEDGE;
  double CDSCBEDGE;
  double ETA0EDGE;
  double ETABEDGE;
  double KT1EDGE;
  double KT1LEDGE;
  double KT2EDGE;
  double KT1EXPEDGE;
  double TNFACTOREDGE;
  double TETA0EDGE;
  double DVT0EDGE;
  double DVT1EDGE;
  double DVT2EDGE;
  //  Variables of global_model scope
  // end verilog model variables=====
};

void registerDevice(const DeviceCountMap& deviceMap = DeviceCountMap(),
                    const std::set<int>& levelSet = std::set<int>());

} // namespace ADMSbsim6
} // namespace Device
} // namespace Xyce
#endif //Xyce_N_DEV_ADMSbsim6_h
